/*****************************************************************************/
/* configuration.c : Configuration utilisateur
 * configuration.c : User configuration
 *
 *
 * TouxDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/stat.h>
#include <toutdoux.h>

#include "main.h"
#include "configuration.h"


/*****************************************************************************/
/* Installation */
/*****************************************************************************/
gboolean configuration_file (void)
{
  DIR *cur_repertoire;
  gchar *txt_tmp;

  /* Rpertoires */
  cur_repertoire = opendir (g_strdup_printf ("%s/.%s", g_get_home_dir(), TD_PACKAGE_DIR));
  if (cur_repertoire)
    {
      closedir (cur_repertoire);
      return TRUE;
    }
  td_palette_message (NULL, _("Installation"), "begin");
  if (mkdir (g_strdup_printf ("%s/.%s", g_get_home_dir (), TD_PACKAGE_DIR), S_IRWXU))
    {
      td_palette_message (NULL, _("Creating user's directory"), "failed");
      td_palette_message (NULL, _("Installation"), "end");
      return FALSE;
    }
  cur_repertoire = opendir (g_strdup_printf("%s/.%s/tmp", g_get_home_dir(), TD_PACKAGE_DIR));
  if (mkdir (g_strdup_printf ("%s/.%s/tmp", g_get_home_dir(), TD_PACKAGE_DIR), S_IRWXU))
    {
      td_palette_message (NULL, _("Creating user's directory of tempory file"), "failed");
      td_palette_message (NULL, _("Installation"), "end");
      return FALSE;
    }
  cur_repertoire = opendir (g_strdup_printf("%s/.%s/customize", g_get_home_dir(), TD_PACKAGE_DIR));
  if (mkdir (g_strdup_printf ("%s/.%s/customize", g_get_home_dir(), TD_PACKAGE_DIR), S_IRWXU))
    {
      td_palette_message (NULL, _("Creating user's directory of customizing file"), "failed");
      td_palette_message (NULL, _("Installation"), "end");
      return FALSE;
    }

  /* Fichier de configuration */
  if (!td_file_exist (g_strdup_printf ("%s/.%s/customize/app.xml", g_get_home_dir(), TD_PACKAGE_DIR)))
    {
      if (td_file_exist (g_strdup_printf ("%s/share/%s/app/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, _("en"))))
	txt_tmp = g_strdup_printf ("cp %s/share/%s/app/%s/customize.xml %s/.%s/customize/app.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, _("en"), g_get_home_dir(), TD_PACKAGE_DIR);
      else
	txt_tmp = g_strdup_printf ("cp %s/share/%s/app/%s/customize.xml %s/.%s/customize/app.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, "en", g_get_home_dir(), TD_PACKAGE_DIR);
      if (!td_system (txt_tmp))
	{
	  td_palette_message (NULL, _("Copying file config"), "failed");
	  td_palette_message (NULL, _("Installation"), "end");
	  return FALSE;
	}
    }
  td_palette_message (NULL, _("Installation"), "end");
  return TRUE;
}

/*****************************************************************************/
/* Correction */
/*****************************************************************************/
gboolean correction (void)
{
  int system_verif;
  gchar *txt_tmp;

  /* Correction bug du fichier XML des signets de la version 1.2.5 */
  txt_tmp = g_strdup_printf ("%s/.%s/bookmarks.xml", g_get_home_dir(), TD_PACKAGE_DIR);
  if (td_file_exist (txt_tmp))
    {
      txt_tmp = g_strdup_printf ("grep '<item field0=\"1\" field1=\"0\" field2=\"1\"/>' %s", txt_tmp);
      system_verif = system (txt_tmp);

      /* Appliquer le correctif */
      if (system_verif == 0)
	{
	  td_palette_message (NULL, _("Correction"), "begin");

	  /* Sauvegarde */
	  txt_tmp = g_strdup_printf ("mv %s/.%s/bookmarks.xml %s/.%s/bookmarks.xml.bak", g_get_home_dir(), TD_PACKAGE_DIR, g_get_home_dir(), TD_PACKAGE_DIR);
	  if (!td_system (txt_tmp))
	    {
	      td_palette_message (NULL, _("Copying bookmarks"), "failed");
	      td_palette_message (NULL, _("Correction"), "end");
	      return FALSE;
	    }

	  /* Go ! */
	  if (td_file_exist (g_strdup_printf ("%s/share/%s/help_bookmarks/%s/bookmarks.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, _("en"))))
	    txt_tmp = g_strdup_printf ("cp %s/share/%s/help_bookmarks/%s/bookmarks.xml %s/.%s/bookmarks.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, _("en"), g_get_home_dir(), TD_PACKAGE_DIR);
	  else
	    txt_tmp = g_strdup_printf ("cp %s/share/%s/help_bookmarks/%s/bookmarks.xml %s/.%s/bookmarks.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, "en", g_get_home_dir(), TD_PACKAGE_DIR);
	  if (!td_system (txt_tmp))
	    {
	      td_palette_message (NULL, _("Copying bookmarks"), "failed");
	      td_palette_message (NULL, _("Correction"), "end");
	      return FALSE;
	    }
	  td_palette_message (NULL, _("Correction"), "end");
	}
    }
  return TRUE;
}

/*****************************************************************************/
/* Configuration */
/*****************************************************************************/
void configuration_gui (void)
{
  FLAG_ENTRY_CHANGED_TIME = atoi (td_mod_custom_value ("app", "database", "delay", "delay between sql backend and gui"));
  FLAG_DRAG_AUTOEXPAND_TIME = atoi (td_mod_custom_value ("app", "interface", "delay", "delay for expanding tree's node"));
}
