/*****************************************************************************/
/* main.c : Donnes globales et fentre principale
 * main.c : Global data and main window
 *
 *
 * Greffon ToutDoux : messages
 * ToutDoux's plug-in : messages
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <toutdoux.h>

#include "main.h"


/*****************************************************************************/
/* Variables globales */
/*****************************************************************************/
GtkObject *mod;
GtkCTreeNode *node_current;

/*****************************************************************************/
/* Module */
/*****************************************************************************/
GtkObject *plugins_about_def (void)
{
  GtkObject *ret;
  ret = td_about_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", _("Messages"),
		  "type", _("dialog"),
		  "version", "1.0.1",
		  "authors", "Philippe Roy <ph_roy@toutdoux.org>",
		  "help", "(null)",
		  "category1", _("core"),
		  "category2", _("dialog"),
		  "category3", _("messages"),
		  "copyright", "Copyright (c) 2000-2001 Philippe Roy",
		  "license", _("Covered by the GNU General Public License"),
		  "web", "(null)", NULL);
  return ret;
}

GtkObject *mod_def (void)
{
  GtkObject *ret;

  GtkObject *box;
  GtkWidget *scrolledwindow;
  GtkWidget *ctree;
  GtkWidget *label;

  /* Module */
  ret = td_mod_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", "messages",
		  "name_intl", _("Messages"),
		  "id", -1, NULL);
  td_mod_set_about (TD_MOD (ret), plugins_about_def ());
  node_current = NULL;

  /* Arbre */
  box = td_mod_box_new ();
  gtk_object_set (GTK_OBJECT (box), 
		  "name", _("Messages"), NULL);
  td_mod_add_child (TD_MOD (ret), box);
  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  td_mod_box_set_widget (TD_MOD_BOX (box), scrolledwindow);
  ctree = gtk_ctree_new (2, 0);
  gtdk_clist_adjustheight (ctree);
  gtk_widget_show (ctree);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), ctree);
  gtk_widget_set_usize (ctree, 0, -2);
  gtk_clist_column_titles_show (GTK_CLIST (ctree));
  gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 0, TRUE);
  td_mod_box_set_widget_data (TD_MOD_BOX (box), ctree);
  label = gtk_label_new (_("Messages"));
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 0, label);
  gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 1, TRUE);
  label = gtk_label_new (_("Type"));
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 1, label);
  gtk_clist_set_column_visibility (GTK_CLIST (ctree), 1, FALSE);
  gtk_signal_connect (GTK_OBJECT (ctree), "button_press_event", GTK_SIGNAL_FUNC (td_mod_box_button_press), box);
  return ret;
}

void plugins_init (void)
{
  int i;
  if (!mod)
    {
      mod = mod_def ();
      td_mod_create (TD_MOD (mod));
      for (i=0; i<g_list_length (START_MSG); i++)
	{
	  plugins_new_message (g_list_nth_data (START_MSG, i), g_list_nth_data (START_MSG, i+1));
	  i++;
	}
      
      START_MSG = NULL;
    }
  else
    td_mod_show (TD_MOD (mod));
}

void plugins_destroy (gpointer user_data)
{
  td_mod_destroy (TD_MOD (mod));
  mod = NULL;
}


/*****************************************************************************/
/* Commandes */
/*****************************************************************************/
gboolean plugins_new_message (gchar *message, gchar *type)
{
  GtkCTreeNode *node = NULL;
  gchar *txt[2];
  gchar *pixmap1_tmp = NULL;
  gchar *pixmap2_tmp = NULL;
  GdkPixmap *pixmap1 = NULL;
  GdkPixmap *pixmap2 = NULL;
  GdkBitmap *mask1 = NULL;
  GdkBitmap *mask2 = NULL;
  GdkColor transparent;
  if (!TD_APP_PALETTE (TD_APP (APP)->palette)->window)
    return TRUE;

  /* Valeurs */
  txt[0] = message;
  txt[1] = type;

  /* Icones */
  if ((!strcmp (type, "done")) || (!strcmp (type, "tested")))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_OK;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_OK;
    }
  if (!strcmp (type, "plugins"))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_PLUGINS;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_PLUGINS;
    }
  if (!strcmp (type, "command"))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_COMMAND;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_COMMAND;
    }
  if (!strcmp (type, "sql"))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_DATABASE;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_DATABASE;
    }
  if ((!strcmp (type, "failed")) || (!strcmp (type, "stop")))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_WARNING;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_WARNING;
    }
  if (!strcmp (type, "begin"))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_BOOKCLOSE;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_BOOKOPEN;
    }
  if (!strcmp (type, "message"))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_LIGHT;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_LIGHT;
    }
  if (!strcmp (type, "yes"))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_YES;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_YES;
    }
  if (!strcmp (type, "no"))
    {
      pixmap1_tmp = (gchar*) TD_STOCK_LIST_NO;
      pixmap2_tmp = (gchar*) TD_STOCK_LIST_NO;
    }
  if (pixmap1_tmp)
    {
      pixmap1 = gdk_pixmap_create_from_xpm_d (GTK_WIDGET (TD_APP_PALETTE (TD_APP (APP)->palette)->window)->window, &mask1, &transparent, (gchar**) pixmap1_tmp);
      pixmap2 = gdk_pixmap_create_from_xpm_d (GTK_WIDGET (TD_APP_PALETTE (TD_APP (APP)->palette)->window)->window, &mask2, &transparent, (gchar**) pixmap2_tmp);
    }

  /* Node */
  if (mod)
    {
      if (!strcmp (type, "begin"))
	node_current = gtk_ctree_insert_node (GTK_CTREE (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (mod)->child, 0))->widget)->child), node_current, NULL, txt, 5, pixmap1, mask1, pixmap2, mask2, FALSE, FALSE);
      else
	{
	  if (!strcmp (type, "end"))
	    {
	      if (node_current)
		{
		  if (GTK_CTREE_ROW (node_current)->parent)
		    node_current = GTK_CTREE_ROW (node_current)->parent;
		  else
		    node_current = NULL;
		}
	    }
	  else
	    node = gtk_ctree_insert_node (GTK_CTREE (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (mod)->child, 0))->widget)->child), node_current, NULL, txt, 5, pixmap1, mask1, pixmap2, mask2, FALSE, FALSE);
	  if (!strcmp (type, "failed"))
	    gtdk_ctree_invert_expand_recursive (GTK_CTREE (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (mod)->child, 0))->widget)->child), node);
	}
    }
  return TRUE;
}

void plugins_load (gchar *file)
{

  /* Fichier support */
/*    flower[0] = td_flower_new (); */
/*    gtk_object_set (GTK_OBJECT (flower[0]), */
/*  		  "space", "toutdoux-messages/1.0.1", */
/*  		  "root", "messages",  */
/*  		  "branch", "message", */
/*  		  "mode_level", TRUE, NULL);  */
/*    td_flower_add_branch_attribut (TD_FLOWER (flower[0]), "text"); */
/*    td_flower_add_branch_attribut (TD_FLOWER (flower[0]), "type"); */

/*    GList *data; */
/*    int i; */
/*    td_palette_message (NULL, g_strdup_printf (_("XML Log")), "begin"); */
/*    td_flower_load_file (TD_FLOWER (flower[0]), file); */
/*    for (i=0; i<g_list_length (TD_FLOWER (flower[0])->items_value); i++) */
/*      { */
/*        data = g_list_nth_data (TD_FLOWER (flower[0])->items_value , i); */
/*        td_palette_message (NULL, g_list_nth_data (data, 0), g_list_nth_data (data, 1)); */
/*        data = NULL; */
/*      } */
/*    td_palette_message (NULL, g_strdup_printf (_("XML Log")), "end"); */
}
