/*****************************************************************************/
/* main.c : Donnes globales et fentre principale
 * main.c : Global data and main window
 *
 *
 * Greffon ToutDoux : personnalisation
 * ToutDoux's plug-in : customize
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <dlfcn.h>
#include <toutdoux.h>

#include "main.h"


/*****************************************************************************/
/* Module */
/*****************************************************************************/
GtkObject *plugins_about_def (void)
{
  GtkObject *ret;
  ret = td_about_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", _("Customize"),
		  "type", _("help"),
		  "version", "1.0.1",
		  "authors", "Philippe Roy <ph_roy@toutdoux.org>",
		  "help", "(null)",
		  "category1", _("core"),
		  "category2", _("help"),
		  "category3", _("customize"),
		  "copyright", "Copyright (c) 2000-2001 Philippe Roy",
		  "license", _("Covered by the GNU General Public License"),
		  "web", "(null)", NULL);
  return ret;
}

GtkObject *mod_def (GtkObject *mod)
{
  GtkObject *ret;
  GtkObject *box[2];

  GtkWidget *hbox1;
  GtkWidget *label_tree;
  GtkWidget *scrolledwindow1;
  GtkWidget *vseparator1;
  GtkWidget *viewport;
  GtkWidget *custom_box0;

  GtkWidget *scrolledwindow2;
  GtkWidget *ctree;
  GtkWidget *label;

  GtkWidget *toolbar;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *button4;
  gchar *txt_tmp;
  gboolean bool_tmp;
  int i, j, k, m;

  /* Module */
  ret = td_mod_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", "customize",
		  "name_intl", g_strdup_printf ("%s : %s", TD_MOD (mod)->name_intl, _("Customize")), NULL);
  td_mod_set_about (TD_MOD (ret), plugins_about_def ());

  /* Barre d'outils */
  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_show (toolbar);
  td_mod_set_window_toolbar (TD_MOD (ret), toolbar);
  button4 = gtdk_toolbar_append_button_stock (GTK_TOOLBAR (toolbar), _("Reset to standard"), TD_MOD (ret)->window, GNOME_STOCK_PIXMAP_CLEAR);
  gtk_signal_connect_object (GTK_OBJECT (button4), "clicked", GTK_SIGNAL_FUNC (reset_standard), GTK_OBJECT (mod));
  button1 = gtdk_toolbar_append_button_stock (GTK_TOOLBAR (toolbar), _("Reset to saved"), TD_MOD (ret)->window, GNOME_STOCK_PIXMAP_REVERT);
  gtk_signal_connect_object (GTK_OBJECT (button1), "clicked", GTK_SIGNAL_FUNC (reset_saved), GTK_OBJECT (mod));
  button2 = gtdk_toolbar_append_button_stock (GTK_TOOLBAR (toolbar), _("Save for future sessions"), TD_MOD (ret)->window, GNOME_STOCK_PIXMAP_SAVE);
  gtk_signal_connect_object (GTK_OBJECT (button2), "clicked", GTK_SIGNAL_FUNC (save), GTK_OBJECT (mod));
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  button3 = gtdk_toolbar_append_button_stock (GTK_TOOLBAR (toolbar), _("Close"), TD_MOD (ret)->window, GNOME_STOCK_PIXMAP_CLOSE);
  gtk_signal_connect_object (GTK_OBJECT (button3), "clicked", GTK_SIGNAL_FUNC (td_mod_hide), GTK_OBJECT (ret));

  /* Navigateur */
  box[0] = td_mod_box_new ();
  gtk_object_set (GTK_OBJECT (box[0]), 
		  "name", _("Browser"), NULL);
  td_mod_add_child (TD_MOD (ret), box[0]);
  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  td_mod_box_set_widget (TD_MOD_BOX (box[0]), hbox1);
  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (hbox1), scrolledwindow1, FALSE, FALSE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  TD_MOD (mod)->custom_tree = gtk_ctree_new (2, 0);
  gtk_widget_show (TD_MOD (mod)->custom_tree);
  gtdk_clist_adjustheight (TD_MOD (mod)->custom_tree);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), TD_MOD (mod)->custom_tree);
  if (!TD_APP (APP)->mode_debug)
    gtk_clist_set_column_visibility (GTK_CLIST (TD_MOD (mod)->custom_tree), 1, FALSE);
  gtk_clist_column_titles_show (GTK_CLIST (TD_MOD (mod)->custom_tree));
  label_tree = gtk_label_new (_("Category"));
  gtk_widget_show (label_tree);
  gtk_clist_set_column_widget (GTK_CLIST (TD_MOD (mod)->custom_tree), 0, label_tree);
  if (TD_APP (APP)->mode_debug)
    gtk_clist_set_column_width (GTK_CLIST (TD_MOD (mod)->custom_tree), 1, gtdk_ctree_optimal_column_width (TD_MOD (mod)->custom_tree, 1));
  vseparator1 = gtk_vseparator_new ();
  gtk_widget_show (vseparator1);
  gtk_box_pack_start (GTK_BOX (hbox1), vseparator1, FALSE, TRUE, 0);
  viewport = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport);
  gtk_box_pack_start (GTK_BOX (hbox1), viewport, TRUE, TRUE, 0);
  TD_MOD (mod)->custom_notebook = gtk_notebook_new ();
  gtk_widget_show (TD_MOD (mod)->custom_notebook);
  if (!TD_APP (APP)->mode_debug)
    gtk_notebook_set_show_tabs (GTK_NOTEBOOK (TD_MOD (mod)->custom_notebook), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (TD_MOD (mod)->custom_notebook), FALSE);
  gtk_container_add (GTK_CONTAINER (viewport), TD_MOD (mod)->custom_notebook);
  custom_box0 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (GTK_WIDGET (custom_box0));
  gtk_container_add (GTK_CONTAINER (TD_MOD (mod)->custom_notebook), GTK_WIDGET (custom_box0));
  gtk_signal_connect (GTK_OBJECT (TD_MOD (mod)->custom_tree), "button_press_event", GTK_SIGNAL_FUNC (browser_press), GTK_OBJECT (TD_MOD (mod)->custom_notebook));
  gtk_signal_connect (GTK_OBJECT (TD_MOD (mod)->custom_tree), "tree_expand", GTK_SIGNAL_FUNC (gtdk_ctree_expand), scrolledwindow1);

  /* Arbre */
  box[1] = td_mod_box_new ();
  gtk_object_set (GTK_OBJECT (box[1]), 
		  "name", _("List"), NULL);
  td_mod_add_child (TD_MOD (ret), box[1]);
  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  td_mod_box_set_widget (TD_MOD_BOX (box[1]), scrolledwindow2);
  ctree = gtk_ctree_new (5, 0);
  gtdk_clist_adjustheight (ctree);
  gtk_widget_show (ctree);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), ctree);
  gtk_widget_set_usize (ctree, 0, -2);
  gtk_clist_column_titles_show (GTK_CLIST (ctree));
  label = gtk_label_new ("name");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 0, label);
  gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 0, TRUE);
  label = gtk_label_new ("name_intl");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 1, label);
  gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 1, TRUE);
  label = gtk_label_new ("value");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 2, label);
  gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 2, TRUE);
  label = gtk_label_new ("widget");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 3, label);
  gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 3, TRUE);
  label = gtk_label_new ("type");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 4, label);
  gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 4, TRUE);
  return ret;
}


void plugins_init (void)
{
}

void plugins_destroy (gpointer user_data)
{
}

void plugins_create (GtkObject *mod)
{
  GtkObject *flower;

  /* Interface vierge */
  TD_MOD (mod)->custom_browser = mod_def (mod);
  td_mod_create (TD_MOD (TD_MOD (mod)->custom_browser));

  /* Donnes */
  load_file (mod, NULL);
}

/*****************************************************************************/
/* Fichier */
/*****************************************************************************/
void reset_standard (GtkObject *mod)
{
  if (td_file_exist (g_strdup_printf("%s/share/%s/%s/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, TD_MOD (mod)->name, _("en"))))
    load_file (mod, g_strdup_printf("%s/share/%s/%s/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, TD_MOD (mod)->name, _("en")));
  else
    load_file (mod, g_strdup_printf("%s/share/%s/%s/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, TD_MOD (mod)->name, "en"));
}

void reset_saved (GtkObject *mod)
{
  load_file (mod, g_strdup_printf ("%s/customize/%s.xml", TD_PACKAGE_VAR_DIR, TD_MOD (mod)->name));
}

gboolean load_file (GtkObject *mod, gchar* file)
{
  GtkObject *flower;
  GList *node_list = NULL;
  GList *predef_list = NULL;
  gchar *txt[5];
  GtkCTreeNode *node_browser;
  gchar *txt2[2];
  int i, j;
  int k = -1;
  int l = -1;
  GtkWidget *custom_box0;
  GtkWidget *custom_box = NULL;
  GtkWidget *custom_scrolledwindow = NULL;
  GtkWidget *custom_viewport = NULL;
  GtkWidget *custom_box2 = NULL;
  GtkWidget *group_frame = NULL;
  GtkWidget *box_tmp = NULL;
  GtkWidget *group_table = NULL;
  GtkWidget *option_line_box = NULL;
  GtkWidget *option_label = NULL;
  GtkWidget *option_widget = NULL;
  GtkWidget *option_menu = NULL;
  td_palette_message (NULL, _("Loading customize"), "begin");

  /* Fichier */
  flower = td_flower_new ();
  gtk_object_set (GTK_OBJECT (flower),
		  "space", "toutdoux-customize/1.0.1",
		  "root", "customize", 
		  "branch", "category",
		  "petal", "group",
		  "foliole", "option",
		  "stomate", "predef",
		  "mode_type", TRUE, NULL);
  td_flower_add_branch_attribut (TD_FLOWER (flower), "name");
  td_flower_add_branch_attribut (TD_FLOWER (flower), "name_intl");
  td_flower_add_petal_attribut (TD_FLOWER (flower), "name");
  td_flower_add_petal_attribut (TD_FLOWER (flower), "name_intl");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "name");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "name_intl");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "value");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "widget");
  td_flower_add_stomate_attribut (TD_FLOWER (flower), "name");
  td_flower_add_stomate_attribut (TD_FLOWER (flower), "value");
  if (!file)
    {
      if (!td_flower_load_file (TD_FLOWER (flower), g_strdup_printf ("%s/customize/%s.xml", TD_PACKAGE_VAR_DIR, TD_MOD (mod)->name)))
	if (!td_flower_load_file (TD_FLOWER (flower), g_strdup_printf("%s/share/%s/%s/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, TD_MOD (mod)->name, _("en"))))
	  if (!td_flower_load_file (TD_FLOWER (flower), g_strdup_printf("%s/share/%s/%s/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, TD_MOD (mod)->name, "en")))
	    {
	      td_palette_message (NULL, _("Opening standard support file"), "failed");
	      td_palette_message (NULL, _("Loading customize"), "end");
	      return FALSE;
	    }
    }
  else
    if (!td_flower_load_file (TD_FLOWER (flower), file))
      {
	td_palette_message (NULL, g_strdup_printf (_("Opening support file '%s'"), file), "failed");
	td_palette_message (NULL, _("Loading customize"), "end");
	return FALSE;
      }
  gtdk_ctree_clear (TD_MOD (mod)->custom_tree);
  gtk_container_foreach (GTK_CONTAINER (TD_MOD (mod)->custom_notebook), (GtkCallback) gtk_widget_destroy, NULL);
  custom_box0 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (GTK_WIDGET (custom_box0));
  gtk_container_add (GTK_CONTAINER (TD_MOD (mod)->custom_notebook), GTK_WIDGET (custom_box0));
  TD_MOD (mod)->custom_widget = NULL;
  gtdk_ctree_clear (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (TD_MOD (mod)->custom_browser)->child, 1))->widget)->child);

  /* Valeurs */
  for (i=0; i<g_list_length (TD_FLOWER (flower)->items_value); i++)
    {
      txt[0] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 0);

      /* Catgorie */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 0)
	{
	  txt[1] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[2] = "";
	  txt[3] = "";
	  txt[4] = "category";
	  k++;
	  txt2[0] = txt[1];
	  txt2[1] = g_strdup_printf ("%d", k+1);
	  node_browser = gtk_ctree_insert_node (GTK_CTREE (TD_MOD (mod)->custom_tree), NULL, NULL, txt2, 5, NULL, NULL, NULL, NULL, FALSE, FALSE);
	  custom_box = gtk_vbox_new (FALSE, 0);
	  gtk_widget_show (custom_box);
	  gtk_container_add (GTK_CONTAINER (TD_MOD (mod)->custom_notebook), custom_box);
	  gtdk_vbox_title (txt[1], custom_box);
	  custom_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	  gtk_widget_show (custom_scrolledwindow);
	  gtk_box_pack_start (GTK_BOX (custom_box), custom_scrolledwindow, TRUE, TRUE, 0);
	  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (custom_scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	  custom_viewport = gtk_viewport_new (NULL, NULL);
	  gtk_viewport_set_shadow_type (GTK_VIEWPORT (custom_viewport), GTK_SHADOW_NONE);
	  gtk_widget_show (custom_viewport);
	  gtk_container_add (GTK_CONTAINER (custom_scrolledwindow), custom_viewport);
	  custom_box2 = gtk_vbox_new (FALSE, 0);
	  gtk_widget_show (custom_box2);
	  gtk_container_add (GTK_CONTAINER (custom_viewport), custom_box2);
	}

      /* Groupe */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 1)
	{
	  txt[1] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[2] = "";
	  txt[3] = "";
	  txt[4] = "group";
	  group_frame = gtk_frame_new (txt[1]);
	  gtk_widget_show (group_frame);
  	  gtk_container_set_border_width (GTK_CONTAINER (group_frame), 5);
	  gtk_box_pack_start (GTK_BOX (custom_box2), group_frame, FALSE, TRUE, 0);
	  group_table = gtk_table_new (20, 2, FALSE); /* FIXME: table dynamique : +tards */
	  gtk_widget_show (group_table);
	  gtk_container_set_border_width (GTK_CONTAINER (group_table), 5);
	  gtk_container_add (GTK_CONTAINER (group_frame), group_table);
	  l = -1;
	}

      /* Option */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 2)
	{
	  txt[1] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[2] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 2);
	  txt[3] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 3);
	  txt[4] = "option";

	  /* check_button */
	  if (!strcmp (txt[3], "check_button"))
	    {
	      option_widget = gtk_check_button_new();
	      if (!strcmp (txt[2], "t"))
		gtk_button_clicked (GTK_BUTTON (option_widget));
	      td_mod_add_custom_widget (TD_MOD (mod), option_widget);
	    }

	  /* radio_button */
	  /* if (!strcmp (txt[3], "radio_button")) */
	  /* { */
	  /* option_widget = gtk_radio_button_new_with_label (txt[0]); */
	  /* if (!strcmp (txt[1], "t")) */
	  /* gtk_button_clicked (GTK_BUTTON (option_widget)); */
	  /*  td_mod_add_custom_widget (mod, option_widget); */
	  /* } */

	  /* entry */
	  if (!strcmp (txt[3], "entry"))
	    {
	      option_widget = gtk_entry_new ();
	      gtk_widget_set_usize (option_widget, 50, -2);
	      gtk_entry_set_text (GTK_ENTRY (option_widget), txt[2]);
	      td_mod_add_custom_widget (TD_MOD (mod), option_widget);
	    }

	  /* spin_button */
	  if (!strcmp (txt[2], "spin_button"))
	    {
	      option_widget = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new (1, 0, 10000, 1, 10, 10)), 1, 0);
	      gtk_spin_button_set_value (GTK_SPIN_BUTTON (option_widget), atoi (txt[2]));
	      td_mod_add_custom_widget (TD_MOD (mod), option_widget);
	    }

	  /* Widget */
	  l++;
	  /* gtk_table_resize (GTK_TABLE (group_table), l+1, 3); *//* FIXME: table dynamique : +tards */
	  option_label = gtk_label_new (g_strdup_printf ("%s:", txt[1]));
	  gtk_widget_show (option_label);
  	  gtk_misc_set_padding (GTK_MISC (option_label), 5, 0);
  	  gtk_label_set_line_wrap (GTK_LABEL (option_label), TRUE);
	  /* gtk_label_set_justify (GTK_LABEL (option_label), GTK_JUSTIFY_RIGHT); *//* FIXME: marche p - not work : +tards */
  	  gtk_table_attach (GTK_TABLE (group_table), option_label, 0, 1, l, l+1, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
	  box_tmp = gtk_hbox_new (FALSE, 0);
	  gtk_table_attach (GTK_TABLE (group_table), box_tmp, 1, 2, l, l+1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	  gtk_widget_show (box_tmp);
  	  gtk_widget_show (option_widget);
	  gtk_box_pack_start (GTK_BOX (box_tmp), option_widget, TRUE, TRUE, 0);
  	  option_menu = gtk_option_menu_new ();
  	  gtk_widget_show (option_menu);
	  gtk_box_pack_start (GTK_BOX (box_tmp), option_menu, FALSE, FALSE, 0);
	  predef_list = NULL;
	}

      /* Valeurs prdfinis */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 3)
	{
	  txt[1] = "";
	  txt[2] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[3] = "";
	  txt[4] = "predef";
	  predef_list = g_list_append (predef_list, g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1));
	  gtdk_option_menu_set_popdown_strings (option_menu, predef_list, option_widget);
	}

      /* Node */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 0)
	{
	  node_list = g_list_append (node_list, gtk_ctree_insert_node (GTK_CTREE (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (TD_MOD (mod)->custom_browser)->child, 1))->widget)->child), NULL, NULL, txt, 5, NULL, NULL, NULL, NULL, FALSE, FALSE));
	  continue;
	}
      j = 1;
      while (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i-j)) != GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i))-1)
	j++;
      node_list = g_list_append (node_list, gtk_ctree_insert_node (GTK_CTREE (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (TD_MOD (mod)->custom_browser)->child, 1))->widget)->child), g_list_nth_data (node_list, i-j), NULL, txt, 5, NULL, NULL, NULL, NULL, FALSE, FALSE));
    }
  gtk_clist_set_column_width (GTK_CLIST (TD_MOD (mod)->custom_tree), 0, gtdk_ctree_optimal_column_width (TD_MOD (mod)->custom_tree, 0));
  td_palette_message (NULL, _("Loading customize"), "done");
  td_palette_message (NULL, _("Loading customize"), "end");
  return TRUE;
}

gboolean save (GtkObject *mod)
{
  GtkObject *flower;
  gchar *file;
  GList *node_list = NULL;
  gchar *txt[5];
  FILE *fp;
  xmlDocPtr doc;
  xmlNsPtr ns;
  xmlNodePtr tree;
  xmlNodePtr item_category = NULL;
  xmlNodePtr item_group = NULL;
  xmlNodePtr item_option = NULL;
  xmlNodePtr item_predef = NULL;
  int i, j;
  int k = -1;
  td_palette_message (NULL, _("Saving customize"), "begin");

  /* Fichier dfinition */
  flower = td_flower_new ();
  gtk_object_set (GTK_OBJECT (flower),
		  "space", "toutdoux-customize/1.0.1",
		  "root", "customize", 
		  "branch", "category",
		  "petal", "group",
		  "foliole", "option",
		  "stomate", "predef",
		  "mode_type", TRUE, NULL);
  td_flower_add_branch_attribut (TD_FLOWER (flower), "name");
  td_flower_add_branch_attribut (TD_FLOWER (flower), "name_intl");
  td_flower_add_petal_attribut (TD_FLOWER (flower), "name");
  td_flower_add_petal_attribut (TD_FLOWER (flower), "name_intl");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "name");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "name_intl");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "value");
  td_flower_add_foliole_attribut (TD_FLOWER (flower), "widget");
  td_flower_add_stomate_attribut (TD_FLOWER (flower), "name");
  td_flower_add_stomate_attribut (TD_FLOWER (flower), "value");

  /* Fichier utilisateur */
  file = g_strdup_printf ("%s/customize/%s.xml", TD_PACKAGE_VAR_DIR, TD_MOD (mod)->name);
  fp = fopen (file, "w");
  if (!fp)
    {
      td_palette_message (NULL, g_strdup_printf (_("Opening support file '%s'"), file), "failed");
      td_palette_message (NULL, _("Saving customize"), "end");
      return FALSE;
    }
  fclose (fp);
  doc = xmlNewDoc ("1.0");
  ns = xmlNewGlobalNs (doc, TD_FLOWER (flower)->space, g_strdup_printf("ToutDoux-%s", TD_FLOWER (flower)->root));
  doc->root = xmlNewDocNode (doc, ns, TD_FLOWER (flower)->root, NULL);
  tree = doc->xmlRootNode;

  /* Valeurs */
  if (!td_flower_load_file (TD_FLOWER (flower), g_strdup_printf("%s/share/%s/%s/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, TD_MOD (mod)->name, _("en"))))
    if (!td_flower_load_file (TD_FLOWER (flower), g_strdup_printf("%s/share/%s/%s/%s/customize.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR, TD_MOD (mod)->name, "en")))
      {
	td_palette_message (NULL, _("Opening standard support file"), "failed");
	td_palette_message (NULL, _("Saving customize"), "end");
	return FALSE;
      }
  gtdk_ctree_clear (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (TD_MOD (mod)->custom_browser)->child, 1))->widget)->child);
  for (i=0; i<g_list_length (TD_FLOWER (flower)->items_value); i++)
    {
      txt[0] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 0);

      /* Catgorie */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 0)
	{
	  txt[1] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[2] = "";
	  txt[3] = "";
	  txt[4] = "category";
	  item_category = xmlNewChild (tree, ns, TD_FLOWER (flower)->branch, NULL);
	  xmlSetProp (item_category, "name", txt[0]);
	  xmlSetProp (item_category, "name_intl", txt[1]);
	}

      /* Groupe */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 1)
	{
	  txt[1] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[2] = "";
	  txt[3] = "";
	  txt[4] = "group";
	  item_group = xmlNewChild (item_category, ns, TD_FLOWER (flower)->petal, NULL);
	  xmlSetProp (item_group, "name", txt[0]);
	  xmlSetProp (item_group, "name_intl", txt[1]);
	}

      /* Option */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 2)
	{
	  k++;

	  /* entry */
	  if (GTK_IS_ENTRY (g_list_nth_data (TD_MOD (mod)->custom_widget, k)))
	    {
	      txt[2] = g_strdup_printf ("%s", gtk_entry_get_text (GTK_ENTRY (g_list_nth_data (TD_MOD (mod)->custom_widget, k))));
	      txt[3] = "entry";
	    }

	  /* check_button */
	  if (GTK_IS_CHECK_BUTTON (g_list_nth_data (TD_MOD (mod)->custom_widget, k)))
	    {
	      if (GTK_TOGGLE_BUTTON (g_list_nth_data (TD_MOD (mod)->custom_widget, k))->active)
		txt[2] = "t";
	      else
		txt[2] = "f";
	      txt[3] = "check_button";
	    }

	  /* spin_button */
	  if (GTK_IS_SPIN_BUTTON (g_list_nth_data (TD_MOD (mod)->custom_widget, k)))
	    {
	      txt[2] = g_strdup_printf ("%d", gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (g_list_nth_data (TD_MOD (mod)->custom_widget, k))));
	      txt[3] = "spin_button";
	    }
	  txt[1] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[4] = "option";
	  item_option = xmlNewChild (item_group, ns, TD_FLOWER (flower)->foliole, NULL);
	  xmlSetProp (item_option, "name", txt[0]);
	  xmlSetProp (item_option, "name_intl", txt[1]);
	  xmlSetProp (item_option, "value", txt[2]);
	  xmlSetProp (item_option, "widget", txt[3]);
	}

      /* Valeurs prdfinis */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 3)
	{
	  txt[1] = "";
	  txt[2] = g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 1);
	  txt[3] = "";
	  txt[4] = "predef";
	  item_predef = xmlNewChild (item_option, ns, TD_FLOWER (flower)->stomate, NULL);
	  xmlSetProp (item_predef, "name", txt[0]);
	  xmlSetProp (item_predef, "value", txt[2]);
	}

      /* Node */
      if (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i)) == 0)
	{
	  node_list = g_list_append (node_list, gtk_ctree_insert_node (GTK_CTREE (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (TD_MOD (mod)->custom_browser)->child, 1))->widget)->child), NULL, NULL, txt, 5, NULL, NULL, NULL, NULL, FALSE, FALSE));
	  continue;
	}
      j = 1;
      while (GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i-j)) != GPOINTER_TO_INT (g_list_nth_data (TD_FLOWER (flower)->items_level, i))-1)
	j++;
      node_list = g_list_append (node_list, gtk_ctree_insert_node (GTK_CTREE (GTK_BIN (TD_MOD_BOX (g_list_nth_data (TD_MOD (TD_MOD (mod)->custom_browser)->child, 1))->widget)->child), g_list_nth_data (node_list, i-j), NULL, txt, 5, NULL, NULL, NULL, NULL, FALSE, FALSE));
    }

  /* Fin */
  if (xmlSaveFile (file, doc) < 0)
    {
      xmlFreeDoc (doc);
      td_palette_message (NULL, g_strdup_printf (_("Saving support file '%s'"), file), "failed");
      td_palette_message (NULL, _("Saving customize"), "end");
      return FALSE;
    }
  xmlFreeDoc (doc);
  td_palette_message (NULL, g_strdup_printf (_("Saving support file '%s'"), file), "done");
  td_palette_message (NULL, _("Saving customize"), "end");
  return TRUE;
}

/*****************************************************************************/
/* Navigateur */
/*****************************************************************************/
gboolean browser_press (GtkWidget *widget, GdkEvent *event, GtkWidget *notebook)
{
  GtkCTreeNode *node;
  GdkEventButton *bevent;
  gint row, column;
  gchar *txt_tmp;
  int i;

  /* Capture de la catgorie */
  bevent = (GdkEventButton *) event;
  gtk_clist_get_selection_info (GTK_CLIST (widget), bevent->x, bevent->y, &row, &column);
  row = gtdk_clist_correction (widget, row, bevent->y);
  if (row < 0)
    {
      gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 0);
      gtk_clist_unselect_all (GTK_CLIST (widget));
      return TRUE;
    }
  node = gtk_ctree_node_nth (GTK_CTREE (widget), row);
  gtk_ctree_node_get_text (GTK_CTREE (widget), node, 1, &txt_tmp);
  if (!strcmp (txt_tmp, "(null)"))
    {
      gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 0);
      return TRUE;
    }
  gtk_notebook_set_page (GTK_NOTEBOOK (notebook), atoi (txt_tmp));
  return TRUE;
}
