/*****************************************************************************/
/* main.c : Donnes globales et fentre principale
 * main.c : Global data and main window
 *
 *
 * Greffon ToutDoux : Import-Export
 * ToutDoux's plug-in : Import-Export
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <fcntl.h>

#include "main.h"


/*****************************************************************************/
/* Module */
/*****************************************************************************/
GtkObject *plugins_about_def (void)
{
  GtkObject *ret;
  ret = td_about_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", _("Import-Export"),
		  "type", _("tools"),
		  "version", "1.0.1",
		  "authors", "Philippe Roy <ph_roy@toutdoux.org>",
		  "help", _("Csv conversion and external command"),
		  "category1", _("core"),
		  "category2", _("tools"),
		  "category3", _("import-export"),
		  "copyright", "Copyright (c) 2000-2001 Philippe Roy",
		  "license", _("Covered by the GNU General Public License"),
		  "web", "(null)", NULL);
  return ret;
}

void plugins_init (void)
{
}

void plugins_destroy (gpointer user_data)
{
}

/*****************************************************************************/
/* Commande */
/*****************************************************************************/
gboolean plugins_command (GList *data, GtkObject *mod)
{
  gchar *file;

  /* Fichier */
  if (!strlen (g_strstrip (g_list_nth_data (data, 1))))
    {
      file = g_strdup_printf ("%s", td_fileselector_create_window (_("Choose file"), NULL, NULL, TRUE));
      if ((!file) || (!strcmp (file, "(null)")))
	{
	  td_fileselector_create_window (NULL, NULL, NULL, FALSE);
	  return TRUE;
	}
      td_fileselector_create_window (NULL, NULL, NULL, FALSE);
    }
  else
    file = g_list_nth_data (data, 1);

  /* Exporter */
  if ((strstr (g_list_nth_data (data, 4), "export"))||
      (strstr (g_list_nth_data (data, 4), "script")))
    {
      td_palette_message (NULL, _("Exporting"), "begin");
      td_palette_message (_("Exporting..."), NULL, "message");
      if (!strcmp (g_list_nth_data (data, 3), "csv")) /* csv */
	save_csv (mod, file, "|", TRUE);
      if (!strcmp (g_list_nth_data (data, 3), "svg")) /* svg */
	save_svg (mod, file);
      if (!strcmp (g_list_nth_data (data, 3), "png")) /* png */
	save_png (mod, file);
      if (!strcmp (g_list_nth_data (data, 3), "LaTeX")) /* LaTeX */
	save_latex (mod, file);
      td_palette_message (NULL, _("Exporting"), "end");
    }

  /* Commande */
  if (strlen (g_strstrip (g_list_nth_data (data, 2))))
    {
      td_palette_message (NULL, _("Executing external command"), "begin");
      td_palette_message (_("Executing external command..."), NULL, "message");
      if (strstr (g_list_nth_data (data, 4), "script"))
	td_system (g_list_nth_data (data, 2));
      else
	td_shell (g_list_nth_data (data, 2));
      td_palette_message (NULL, _("Executing external command"), "end");
    }
    
  /* Importer */
  if ((strstr (g_list_nth_data (data, 4), "import"))||
      (strstr (g_list_nth_data (data, 4), "script")))
    {
      td_palette_message (NULL, _("Importing"), "begin");
      td_palette_message (_("Importing..."), NULL, "message");
      if (!strcmp (g_list_nth_data (data, 3), "csv")) /* csv */
	{
	  if (strstr (g_list_nth_data (data, 4), "replace"))
	    load_csv (mod, file, "|", TRUE, TRUE);
	  else
	    load_csv (mod, file, "|", TRUE, FALSE);
	}
      td_palette_message (NULL, _("Importing"), "end");
    }
  return TRUE;
}

/*****************************************************************************/
/* Texte */
/*****************************************************************************/
gboolean save_csv (GtkObject *mod, gchar *file, gchar *separator, gboolean title_mode)
{
  FILE *fp;
  GtkObject *mod_child;
  GtkCTreeNode *node;
  GList *data;
  GList *ret = NULL;
  gchar *txt_tmp;
  guint8 spacing_tmp;
  GdkPixmap *pixmap_tmp;
  GdkBitmap *mask_tmp;
  int i;

  /* Ouverture */
  td_palette_message (NULL, _("Saving csv file"), "begin");
  if ((fp = fopen (file, "w")) == NULL)
    {
      td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Saving cvs file"), file), "failed");
      td_palette_message (NULL, _("Saving csv file"), "end");
      return FALSE;
    }
  mod_child = g_list_nth_data (TD_MOD (mod)->child, 0);

  /* mod_tree */
  if (TD_IS_MOD_TREE (mod_child))
    {

      /* Titre */
      /* if (title_mode) */
      /* { */
      /* for (i=0; i<g_list_length (TD_MOD_TREE (mod_child)->field); i++) */
      /* { */
      /* fprintf (fp, TD_FIELD (g_list_nth_data (TD_MOD_TREE (mod_child)->field, i))->name_intl); */
      /* if (i != g_list_length (TD_MOD_TREE (mod_child)->field)-1) */
      /* fprintf (fp, separator); */
      /* } */
      /* fprintf (fp, separator); */
      /* fprintf (fp, _("Level")); */
      /* fprintf (fp, "\n"); */
      /* } */

      /* Valeurs */
      node = gtk_ctree_node_nth (GTK_CTREE (GTK_BIN (TD_MOD_TREE (mod_child)->widget)->child), 0);
      while (node)
	{
	  gtk_ctree_node_get_pixtext (GTK_CTREE (GTK_BIN (TD_MOD_TREE (mod_child)->widget)->child), node, 0, &txt_tmp, &spacing_tmp, &pixmap_tmp, &mask_tmp);
	  fprintf (fp, txt_tmp);
	  for (i=1; i<GTK_CLIST (GTK_BIN (TD_MOD_TREE (mod_child)->widget)->child)->columns; i++)
	    {
	      gtk_ctree_node_get_text (GTK_CTREE (GTK_BIN (TD_MOD_TREE (mod_child)->widget)->child), node, i, &txt_tmp);
	      fprintf (fp, separator);
	      fprintf (fp, txt_tmp);
	    }
	  fprintf (fp, separator);
	  fprintf (fp, g_strdup_printf ("%d", GTK_CTREE_ROW (node)->level));
	  fprintf (fp, "\n");
	  node = gtdk_ctree_next (node);
	}
    }

  /* mod_box */
  if ((TD_IS_MOD_BOX (mod_child)) && (GTK_IS_CTREE (TD_MOD_BOX (mod_child)->widget_data)))
    {

      /* Titre */
      /* if (title_mode) */
      /* { */
      /* for (i=0; i<g_list_length (TD_MOD_TREE (mod_child)->field); i++) */
      /* { */
      /* fprintf (fp, TD_FIELD (g_list_nth_data (TD_MOD_TREE (mod_child)->field, i))->name_intl); */
      /* if (i != g_list_length (TD_MOD_TREE (mod_child)->field)-1) */
      /* fprintf (fp, separator); */
      /* } */
      /* fprintf (fp, separator); */
      /* fprintf (fp, _("Level")); */
      /* fprintf (fp, "\n"); */
      /* } */

      /* Valeurs */
      node = gtk_ctree_node_nth (GTK_CTREE (TD_MOD_BOX (mod_child)->widget_data), 0);
      while (node)
	{
	  gtk_ctree_node_get_pixtext (GTK_CTREE (TD_MOD_BOX (mod_child)->widget_data), node, 0, &txt_tmp, &spacing_tmp, &pixmap_tmp, &mask_tmp);
	  fprintf (fp, txt_tmp);
	  for (i=1; i<GTK_CLIST (TD_MOD_BOX (mod_child)->widget_data)->columns; i++)
	    {
	      gtk_ctree_node_get_text (GTK_CTREE (TD_MOD_BOX (mod_child)->widget_data), node, i, &txt_tmp);
	      fprintf (fp, separator);
	      fprintf (fp, txt_tmp);
	    }
	  fprintf (fp, separator);
	  fprintf (fp, g_strdup_printf ("%d", GTK_CTREE_ROW (node)->level));
	  fprintf (fp, "\n");
	  node = gtdk_ctree_next (node);
	}
    }

  /* Fermeture */
  fclose (fp);
  td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Saving csv file"), file), "done");
  td_palette_message (NULL, _("Saving csv file"), "end");
  return TRUE;
}

gboolean load_csv (GtkObject *mod, gchar *file, gchar *separator, gboolean title_mode, gboolean replace_mode)
{
  /* FIXME: +tards */
  return TRUE;
}


/*****************************************************************************/
/* Graphique */
/*****************************************************************************/
gboolean save_svg (GtkObject *mod, gchar *file)
{
  td_palette_message (NULL, _("Saving svg file"), "begin");
  /* FIXME: +tards */
  td_palette_message (NULL, _("Saving svg file"), "end");
  return TRUE;
}

gboolean save_png (GtkObject *mod, gchar *file)
{
  td_palette_message (NULL, _("Saving png file"), "begin");
  /* FIXME: +tards */
  td_palette_message (NULL, _("Saving png file"), "end");
  return TRUE;
}

gboolean save_latex (GtkObject *mod, gchar *file)
{
  td_palette_message (NULL, _("Saving LaTeX file"), "begin");
  /* FIXME: +tards */
  td_palette_message (NULL, _("Saving LaTeX file"), "end");
  return TRUE;
}
