/*****************************************************************************/
/* main.c : Donnes globales et fentre principale
 * main.c : Global data and main window
 *
 *
 * Greffon ToutDoux : actionplan : Plan d'action
 * ToutDoux's plug-in : actionplan : Plan of action
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"
#include "icons.h"


/*****************************************************************************/
/* Module */
/*****************************************************************************/
GtkObject *plugins_about_def (void)
{
  GtkObject *ret;
  ret = td_about_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", _("Plan of actions"),
		  "type", _("view"),
		  "version", "1.0.2",
		  "authors", "Philippe Roy <ph_roy@toutdoux.org>",
		  "help", _("The first ToutDoux's form"),
		  "category1", _("project manager"),
		  "category2", "actionplan",
		  "category3", "(null)",
		  "copyright", "Copyright (c) 2000-2001 Philippe Roy",
		  "license", _("Covered by the GNU General Public License"),
		  "icon", g_strdup_printf ("%s/share/pixmaps/toutdoux/icon_actionplan.xpm", TD_PACKAGE_DATA_DIR),
		  "web", _("http://toutdoux.sourceforge.net/en/"), NULL);
  return ret;
}

GtkObject *plugins_init (int mod_id, gchar *mod_name)
{
  GtkObject *mod;

  /* Module */
  mod = td_mod_new ();
  if (!mod_name)
    mod_name = _("Plan of actions"),
  gtk_object_set (GTK_OBJECT (mod),
		  "name", "actionplan",
		  "name_intl", mod_name,
		  "group", _("Project"),
		  "id", mod_id,
		  "fields", "name, version, status, size, priority, responsible, td_id, td_obso", NULL);
  td_mod_set_about (TD_MOD (mod), plugins_about_def ());

  /* Dfinitions */
  child_def (mod);
  editprop_def (mod);
  menu_def (mod);

  /* Cration */
  td_mod_create (TD_MOD (mod));
  return mod;
}


/*****************************************************************************/
/* Enfant */
/*****************************************************************************/
void child_def (GtkObject *mod)
{
  GtkObject *child[1];
  GtkObject *icon[4];
  
  /* Arbre */
  child[0] = td_mod_tree_new ();
  gtk_object_set (GTK_OBJECT (child[0]),
		  "name", "actionplan",
		  "name_intl", _("Plan of actions"),
		  "reorderable", TRUE,
		  "customize", TRUE,
		  "table", "task",
		  "table_tree", "task_tree",
		  "column_oid", 6, NULL);
  td_mod_add_child (TD_MOD (mod), child[0]);

  icon[0] = td_mod_tree_icon_new ();
  gtk_object_set (GTK_OBJECT (icon[0]),
		  "pixmap1", (gchar*) TD_STOCK_LIST_NO,
		  "pixmap2", (gchar*) TD_STOCK_LIST_NO, 
  		  "query_visible", g_strdup_printf ("TD_CURRENT td_id WHERE (status = '%s');", _("not planned")), NULL);
  td_mod_tree_add_icon (TD_MOD_TREE (child[0]), icon[0]);

  icon[1] = td_mod_tree_icon_new ();
  gtk_object_set (GTK_OBJECT (icon[1]),
		  "pixmap1", (gchar*) list_package_xpm,
		  "pixmap2", (gchar*) list_package_xpm,
  		  "query_visible", g_strdup_printf ("TD_CURRENT td_id WHERE (status = '%s');", _("finished")), NULL);
  td_mod_tree_add_icon (TD_MOD_TREE (child[0]), icon[1]);

  icon[2] = td_mod_tree_icon_new ();
  gtk_object_set (GTK_OBJECT (icon[2]),
		  "pixmap1", (gchar*) list_eclair_xpm,
		  "pixmap2", (gchar*) list_eclair_xpm,
  		  "query_visible", g_strdup_printf ("TD_CURRENT td_id WHERE (status = '%s');", _("in progress")), NULL);
  td_mod_tree_add_icon (TD_MOD_TREE (child[0]), icon[2]);

  icon[3] = td_mod_tree_icon_new ();
  gtk_object_set (GTK_OBJECT (icon[3]),
		  "pixmap1", (gchar*) TD_STOCK_LIST_BOOKCLOSE,
		  "pixmap2", (gchar*) TD_STOCK_LIST_BOOKOPEN,
  		  "query_visible", g_strdup_printf ("TD_CURRENT td_id WHERE (status = '%s');", _("planned")), NULL);
  td_mod_tree_add_icon (TD_MOD_TREE (child[0]), icon[3]);
}

/*****************************************************************************/
/* Proprit */
/*****************************************************************************/
void editprop_def (GtkObject *mod)
{
  GtkObject *editprop[6];

  /* Nom */
  editprop[0] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[0]),
		  "name", "name",
		  "name_intl", _("Name"),
		  "query_refresh", "TD_CURRENT name;",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[0]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[0]);

  /* Version */
  editprop[1] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[1]),
		  "name", "version",
		  "name_intl", _("Version"),
		  "query_refresh", "TD_CURRENT version;",
		  "query_visible", "TD_CURRENT td_id;",
  		  "query_list", "(null)", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[1]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[1]);

  /* Etat */
  editprop[2] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[2]), 
		  "name", "status",
		  "name_intl", _("Status"),
		  "query_refresh", "TD_CURRENT status;",
		  "query_visible", "TD_CURRENT td_id;",
  		  "query_list", "(null)", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[2]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[2]);

  /* Taille */
  editprop[3] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[3]),
		  "name", "size",
		  "name_intl", _("Size"),
		  "query_refresh", "TD_CURRENT size;",
		  "query_visible", "TD_CURRENT td_id;",
  		  "query_list", "(null)", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[3]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[3]);

  /* Priorit */
  editprop[4] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[4]),
		  "name", "priority",
		  "name_intl", _("Priority"),
		  "query_refresh", "TD_CURRENT priority;",
		  "query_visible", "TD_CURRENT td_id;",
  		  "query_list", "(null)", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[4]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[4]);

  /* Responsable */
  editprop[5] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[5]),
		  "name", "responsible",
		  "name_intl", _("Responsible"),
		  "query_refresh", "TD_CURRENT responsible;",
		  "query_visible", "TD_CURRENT td_id;",
  		  "query_list", "(null)", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[5]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[5]);

  /* FIXME: ajouter temps et argent : +tards */
}

void plugins_editprop_refresh (GtkObject *mod, GtkObject *mod_tree)
{
  td_database_set_current (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
}

void plugins_editprop_action (GtkObject *mod, gchar *value, int editprop_id)
{

  /* Go ! */
  FLAG_NO_REFRESH_NODE = TRUE;
  switch (editprop_id)
    {
    case 0:
      td_database_command (g_strdup_printf ("UPDATE task SET name = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      td_mod_tree_update_node (TD_MOD_TREE (g_list_nth_data (TD_MOD (mod)->child, 0)));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    case 1:
      td_database_command (g_strdup_printf ("UPDATE task SET version = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      td_mod_tree_update_node (TD_MOD_TREE (g_list_nth_data (TD_MOD (mod)->child, 0)));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    case 2:
      td_database_command (g_strdup_printf ("UPDATE task SET status = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      td_mod_tree_update_node (TD_MOD_TREE (g_list_nth_data (TD_MOD (mod)->child, 0)));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    case 3:
      td_database_command (g_strdup_printf ("UPDATE task SET size = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      td_mod_tree_update_node (TD_MOD_TREE (g_list_nth_data (TD_MOD (mod)->child, 0)));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    case 4:
      td_database_command (g_strdup_printf ("UPDATE task SET priority = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      td_mod_tree_update_node (TD_MOD_TREE (g_list_nth_data (TD_MOD (mod)->child, 0)));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    case 5:
      td_database_command (g_strdup_printf ("UPDATE task SET responsible = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      td_mod_tree_update_node (TD_MOD_TREE (g_list_nth_data (TD_MOD (mod)->child, 0)));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    default:
      break;
    }
  FLAG_NO_REFRESH_NODE = FALSE;
}


/*****************************************************************************/
/* Menu */
/*****************************************************************************/
void menu_def (GtkObject *mod)
{
  GtkObject *menu[6];

  /* Crer */
  menu[0] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[0]), 
		  "label", _("Create"),
		  "icon", "list_new.xpm",
		  "query_visible", "TD_CURRENT (null);", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[0]);

  menu[1] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[1]), 
		  "label", _("Create"),
		  "icon", "list_new.xpm",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[1]);

  menu[2] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[2]), 
		  "label", _("Insert-Create"),
		  "icon", "list_empty.xpm",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[2]);
  td_mod_add_menu (TD_MOD (mod), td_mod_menu_new ());

  /* Supprimer */
  menu[4] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[4]), 
		  "label", _("Remove"),
		  "icon", "list_trash.xpm", 
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[4]);

  menu[5] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[5]), 
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[5]);
}

void plugins_menu_refresh (GtkObject *mod, GtkObject *mod_tree)
{
  td_database_set_current (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
}

void plugins_menu_action (GtkObject *mod, int menu_id)
{
  GtkCTreeNode *node;
  TdModTree *mod_tree;
  GList *row;
  int id_sibling;
  int id_parent;
  mod_tree = TD_MOD_TREE (g_list_nth_data (TD_MOD (mod)->child, 0));

  /* Action */
  switch (menu_id)
    {
    case 0:
      td_database_insert ("INSERT INTO task (name) VALUES (NULL);", "task");
      row = td_database_row (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
      node = td_mod_tree_add_node (mod_tree, NULL, NULL, row);
      td_database_command (g_strdup_printf ("UPDATE task_tree SET td_id_sibling = %d WHERE (td_id_parent = 0) AND (td_id_sibling = 0);", CURRENT_ID));
      td_database_insert (g_strdup_printf ("INSERT INTO task_tree (td_id, td_id_parent, td_id_sibling) VALUES (%d, 0, 0);", CURRENT_ID), "task_tree");
      break;

    case 1:
      CURRENT_ID_PARENT = CURRENT_ID;
      td_database_insert ("INSERT INTO task (name) VALUES (NULL);", "task");
      row = td_database_row (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
      node = td_mod_tree_add_node (mod_tree, mod_tree->selected, NULL, row);
      gtk_ctree_expand (GTK_CTREE (mod_tree->widget_data), mod_tree->selected);
      td_database_command (g_strdup_printf ("UPDATE task_tree SET td_id_sibling = %d WHERE (td_id_parent = %d) AND (td_id_sibling = 0);", CURRENT_ID, CURRENT_ID_PARENT));
      td_database_insert (g_strdup_printf ("INSERT INTO task_tree (td_id, td_id_parent, td_id_sibling) VALUES (%d, %d, 0);", CURRENT_ID, CURRENT_ID_PARENT), "task_tree");
      break;

    case 2:
      id_sibling = CURRENT_ID;
      id_parent = atoi (td_database_value (g_strdup_printf ("SELECT td_id_parent FROM task_tree WHERE td_id = %d;", id_sibling)));
      td_database_insert ("INSERT INTO task (name) VALUES (NULL);", "task");
      row = td_database_row (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
      node = td_mod_tree_add_node (mod_tree, GTK_CTREE_ROW (mod_tree->selected)->parent, mod_tree->selected, row);
      td_database_command (g_strdup_printf ("UPDATE task_tree SET td_id_sibling = %d WHERE (td_id_parent = %d) AND (td_id_sibling = %d);", CURRENT_ID, id_parent, id_sibling));
      td_database_insert (g_strdup_printf ("INSERT INTO task_tree (td_id, td_id_parent, td_id_sibling) VALUES (%d, %d, %d);", CURRENT_ID, id_parent, id_sibling), "task_tree");
      break;

    case 4:
      id_sibling = atoi (td_database_value (g_strdup_printf ("SELECT td_id_sibling FROM task_tree WHERE td_id = %d;", CURRENT_ID)));
      id_parent = atoi (td_database_value (g_strdup_printf ("SELECT td_id_parent FROM task_tree WHERE td_id = %d;", CURRENT_ID)));
      td_database_command (g_strdup_printf ("UPDATE task_tree SET td_id_sibling = %d WHERE (td_id_parent = %d) AND (td_id_sibling = %d);", id_sibling, CURRENT_ID_PARENT, CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task WHERE td_id IN (SELECT td_id FROM task_tree WHERE td_id_parent = %d);", CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task_tree WHERE td_id = %d;", CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task_tree WHERE td_id_parent = %d;", CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task WHERE td_id = %d;", CURRENT_ID));
      gtk_ctree_remove_node (GTK_CTREE (mod_tree->widget_data), mod_tree->selected);
      CURRENT_ID = 0;
      CURRENT_ROW = NULL;
      mod_tree->selected = NULL;
      break;
    default:
      break;
    }
}

void plugins_menu_link_action (GtkObject *mod, int menu_id)
{
}







