/*****************************************************************************/
/* main.c : Donnes globales et fentre principale
 * main.c : Global data and main window
 *
 *
 * Greffon ToutDoux : gantt : Diagramme GANTT
 * ToutDoux's plug-in : gantt : GANTT diagram
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"

/*****************************************************************************/
/* Module */
/*****************************************************************************/
GtkObject *plugins_about_def (void)
{
  GtkObject *ret;
  ret = td_about_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", _("GANTT diagram"),
		  "type", _("view"),
		  "version", "1.0.2",
		  "authors", "Philippe Roy <ph_roy@toutdoux.org>",
		  "help", "(null)",
		  "category1", _("project manager"),
		  "category2", "gantt",
		  "category3", "(null)",
		  "copyright", "Copyright (c) 2001 Philippe Roy",
		  "license", _("Covered by the GNU General Public License"),
		  "icon", g_strdup_printf ("%s/share/pixmaps/toutdoux/icon_gantt.xpm", TD_PACKAGE_DATA_DIR),
		  "web", "(null)", NULL);
  return ret;
}

GtkObject *plugins_init (int mod_id, gchar *mod_name)
{
  GtkObject *mod;

  /* Module */
  mod = td_mod_new ();
  if (!mod_name)
    mod_name = _("GANTT diagram");
  gtk_object_set (GTK_OBJECT (mod),
		  "name", "gantt",
		  "name_intl", mod_name,
		  "group", _("Project"),
		  "id", mod_id,
		  "fields", "name, begin, eend, duration, td_id, td_obso, td_y", NULL);
  td_mod_set_about (TD_MOD (mod), plugins_about_def ());

  /* Dfinitions */
  child_def (mod);
  editprop_def (mod);
  menu_def (mod);

  /* Cration */
  td_mod_create (TD_MOD (mod));
  return mod;
}


/*****************************************************************************/
/* Enfant */
/*****************************************************************************/
void child_def (GtkObject *mod)
{
  GtkObject *gantt;
  GtkObject *timeruler;
  gchar *txt_tmp;

  /* Box */
  gantt = td_mod_gantt_new ();
  gtk_object_set (GTK_OBJECT (gantt), 
		  "name", "gantt",
		  "name_intl", _("GANTT diagram"),
		  "reorderable", TRUE,
		  "customize", TRUE,
		  "table", "task",
		  "table_net", "task_net",
		  "column_oid", 4, NULL);
  td_mod_add_child (TD_MOD (mod), gantt);

  /* Timeruler */
  timeruler = td_timeruler_new ();
  td_mod_gantt_set_timeruler (TD_MOD_GANTT (gantt), timeruler);
  txt_tmp = td_database_value ("SELECT MIN (begin) FROM task;");
  if (strlen (g_strstrip (txt_tmp)))
    td_timeruler_set_lower (TD_TIMERULER (timeruler), td_date_parse (txt_tmp, "year-month-day"));
  else
    td_timeruler_set_lower (TD_TIMERULER (timeruler), td_date_current());
}

void plugins_refresh (GtkObject *mod, GtkObject *mod_gantt)
{
  GtkObject *datatable;
  int i;

  /* Go ! */
  datatable = td_database_select (g_strdup_printf ("SELECT %s FROM task;", TD_MOD (mod)->fields));
  td_timeruler_set_scale (TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler), 13);
  for (i=0; i<g_list_length (TD_DB_DATATABLE (datatable)->item); i++)
    add_node (mod_gantt, g_list_nth_data (TD_DB_DATATABLE (datatable)->item, i), 0);
  td_mod_unmodified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id));
}


/*****************************************************************************/
/* Symbole */
/*****************************************************************************/
void plugins_symbol_def (GtkObject *mod_gantt)
{
}

gboolean add_node (GtkObject *mod_gantt, GList *row, int id_parent)
{
  GnomeCanvasGroup *root;
  GnomeCanvasGroup *group;
  GnomeCanvasItem *line;
  GnomeCanvasItem *arrow;
  GnomeCanvasPoints *points;
  GList *row_parent;
  GtkObject *draw[2];
  GdkColor fill_color1;
  GdkColor line_color1;
  GdkColor fill_color2;
  int y;
  float x;
  root = gnome_canvas_root (GNOME_CANVAS (TD_MOD_GANTT (mod_gantt)->widget_data));

  /* Rgion */
  td_mod_gantt_set_scrollbar_region (TD_MOD_GANTT (mod_gantt), atoi (g_list_nth_data (row, 6)) + TD_MOD_GANTT (mod_gantt)->row_height);

  /* Action */
  x = td_date_period_day (TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->lower, td_date_parse (g_list_nth_data (row, 1), "year-month-day"))
    *TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
  group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (root, gnome_canvas_group_get_type (), "x", (double) gtdk_canvas_w2c_x (x), "y", (double) atoi (g_list_nth_data (row, 6)), NULL));
  node_setup (mod_gantt, GNOME_CANVAS_ITEM (group));
  gtk_object_set_user_data (GTK_OBJECT (group), g_list_nth_data (row, TD_MOD_GANTT (mod_gantt)->column_oid));

  /* Cadre */
  node_setup (mod_gantt, GNOME_CANVAS_ITEM (group));
  fill_color1 = gtdk_color_parse ("base[normal]");
  line_color1 = gtdk_color_parse ("fg[normal]");
  x = td_date_period_day (td_date_parse (g_list_nth_data (row, 1), "year-month-day"), td_date_parse (g_list_nth_data (row, 2), "year-month-day"))*TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
  gnome_canvas_item_new (group, gnome_canvas_rect_get_type (),
			 "x1", (double) 0,
			 "y1", (double) 0,
			 "x2", (double) x,
			 "y2", (double) 6+(gdk_string_height (gtk_widget_get_default_style()->font, "0")),
			 "fill_color_gdk", &fill_color1,
			 "outline_color_gdk", &line_color1,
			 "width_units", 1.0, NULL);

  /* Nom */
  fill_color2 = gtdk_color_parse ("text[normal]");
  gnome_canvas_item_new (group, gnome_canvas_text_get_type (),
			 "text", g_list_nth_data (row, 0),
			 "x", (double) 16,
			 "y", (double) 2,
			 "font", "-misc-fixed-medium-r-semicondensed-*-*-120-*-*-c-*-iso8859-1",
			 "justification", GTK_JUSTIFY_LEFT,
			 "anchor", GTK_ANCHOR_NORTH_WEST,
			 "fill_color_gdk", &fill_color2, NULL);
  td_mod_modified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id));
  return TRUE;
}

void remove_node (GtkObject *mod_gantt)
{
  GnomeCanvasGroup *root;
  GnomeCanvasItem *item;
  gchar *txt_id;
  int i = 0;
  root = gnome_canvas_root (GNOME_CANVAS (TD_MOD_GANTT (mod_gantt)->widget_data));

  /* Rgion *//* FIXME: +tards */
  /* td_mod_gantt_set_scrollbar_region (TD_MOD_GANTT (mod_gantt), atoi (g_list_nth_data (row, 6)) + TD_MOD_GANTT (mod_gantt)->row_height); */

  /* Placement *//* FIXME: monter le bordel : +tards */

  /* Go ! */
  item = g_list_nth_data (root->item_list, i);
  while (item)
    {
      txt_id = gtk_object_get_user_data (GTK_OBJECT (item));
      if (strstr (txt_id, g_strdup_printf ("%d", CURRENT_ID)))
	{
	  gtk_object_destroy (GTK_OBJECT (item));
	  item = g_list_nth_data (root->item_list, i);
	}
      else
	item = g_list_nth_data (root->item_list, ++i);
    }
  td_mod_modified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id));
}

void node_setup (GtkObject *mod_gantt, GnomeCanvasItem *item)
{
  gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (node_event), mod_gantt);
}

gboolean node_event (GnomeCanvasItem *item, GdkEvent *event, GtkObject *mod_gantt)
{
  GnomeCanvasGroup *root;
  GnomeCanvasItem *node;
  GdkColor color;
  double new_x, new_y, item_x, item_y;
  GdkCursor *cursor;
  GList *row;
  GDate *begin;
  GDate *end;
  int duration;
  double x, x1, x2, x3, y1;
  int delta_x;
  item_x = event->button.x;
  item_y = event->button.y;
  root = gnome_canvas_root (GNOME_CANVAS (TD_MOD_GANTT (mod_gantt)->widget_data));

  /* Go ! */
  switch (event->type) 
    {

      /* Eclairage */
    case GDK_ENTER_NOTIFY:
      if (GNOME_IS_CANVAS_GROUP (item))
	{
	  color = gtdk_color_parse ("bg[prelight]");
  	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 0), 
				 "fill_color_gdk", &color, NULL);
	}
      break;
    case GDK_LEAVE_NOTIFY:
      if ((GNOME_IS_CANVAS_GROUP (item)) && (!(event->crossing.state & GDK_BUTTON1_MASK)))
	{
	  color = gtdk_color_parse ("base[normal]");
	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 0), 
				 "fill_color_gdk", &color, NULL);
	}
      break;

      /* Selection */
    case GDK_BUTTON_PRESS:

      /* Taille */
      if ((event->button.button == 1) && (event->button.state & GDK_SHIFT_MASK) && (GNOME_IS_CANVAS_GROUP (item)))
	{
	  cursor = gtdk_cursor (TD_STOCK_CURSOR_RESIZE, TD_STOCK_CURSOR_RESIZE_MASK);
	  gnome_canvas_item_grab (item, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, cursor, event->button.time);
	  gdk_cursor_destroy (cursor);
	  TD_MOD_GANTT (mod_gantt)->init_x = item_x;
	  TD_MOD_GANTT (mod_gantt)->init_y = item_y;
	  TD_MOD_GANTT (mod_gantt)->dragging_x = item_x;
	  TD_MOD_GANTT (mod_gantt)->dragging_y = item_y;
	  x1 = gtdk_canvas_w2i_x (item, item_x);
	  x2 = GNOME_CANVAS_RE (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 0))->x2;
	  if (x1<x2/2)
	    TD_MOD_GANTT (mod_gantt)->dragging = 2;
	  else
	    TD_MOD_GANTT (mod_gantt)->dragging = 3;
	}

      /* Dplacement de la node */
      else
	if ((event->button.button == 1) && (GNOME_IS_CANVAS_GROUP (item)))
	  {
	    cursor = gtdk_cursor (TD_STOCK_CURSOR_MOVE, TD_STOCK_CURSOR_MOVE_MASK);
	    gnome_canvas_item_grab (item, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, cursor, event->button.time);
	    gdk_cursor_destroy (cursor);
	    TD_MOD_GANTT (mod_gantt)->init_x = item_x;
	    TD_MOD_GANTT (mod_gantt)->init_y = item_y;
	    TD_MOD_GANTT (mod_gantt)->dragging_x = item_x;
	    TD_MOD_GANTT (mod_gantt)->dragging_y = item_y;
	    TD_MOD_GANTT (mod_gantt)->dragging = 1;
	  }
      break;
      
      /* Dplacement */
    case GDK_MOTION_NOTIFY:

      /* Dplacement de la node */
      if (TD_MOD_GANTT (mod_gantt)->dragging == 1)
	{
	  new_x = item_x;
	  new_y = item_y;
	  gnome_canvas_item_move (item, new_x - TD_MOD_GANTT (mod_gantt)->dragging_x, 0);
	  TD_MOD_GANTT (mod_gantt)->dragging_x = new_x;
	  TD_MOD_GANTT (mod_gantt)->dragging_y = new_y;
	}

      /* Taille dbut */
      if (TD_MOD_GANTT (mod_gantt)->dragging == 2)
	{
	  new_x = item_x;
	  new_y = item_y;
	  gnome_canvas_item_move (item, new_x - TD_MOD_GANTT (mod_gantt)->dragging_x, 0);
	  x2 = GNOME_CANVAS_RE (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 0))->x2;
	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 0), 
				 "x2", x2-(new_x-TD_MOD_GANTT (mod_gantt)->dragging_x), NULL);
	  TD_MOD_GANTT (mod_gantt)->dragging_x = new_x;
	  TD_MOD_GANTT (mod_gantt)->dragging_y = new_y;
	}

      /* Taille fin */
      if (TD_MOD_GANTT (mod_gantt)->dragging == 3)
	{
	  new_x = item_x;
	  new_y = item_y;
	  x2 = GNOME_CANVAS_RE (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 0))->x2;
	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 0), 
				 "x2", x2+(new_x-TD_MOD_GANTT (mod_gantt)->dragging_x), NULL);
	  TD_MOD_GANTT (mod_gantt)->dragging_x = new_x;
	  TD_MOD_GANTT (mod_gantt)->dragging_y = new_y;
	}
      break;

      /* Application */
    case GDK_BUTTON_RELEASE:
      gnome_canvas_item_ungrab (item, event->button.time);

      /* Dplacement de la node */
      if (TD_MOD_GANTT (mod_gantt)->dragging == 1)
	{

	  /* Nouvelles dates */
       	  FLAG_NO_REFRESH_NODE = TRUE;
	  delta_x = TD_MOD_GANTT (mod_gantt)->dragging_x - TD_MOD_GANTT (mod_gantt)->init_x;
	  duration = td_date_period_day (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), 
					 td_date_parse (td_database_current ("TD_CURRENT eend;"), "year-month-day"));
	  if (delta_x>0)
	    begin = td_date_add_days (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), ((delta_x/TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)+0.5));
	  else
	    begin = td_date_add_days (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), ((delta_x/TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)-0.5));
	  end = td_date_add_days (begin, duration);
	  if (strcmp (td_date_print (begin, "year-month-day"), td_database_current ("TD_CURRENT begin;")))
	    td_mod_modified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id));

	  /* Mouvement */
  	  x = td_date_period_day (TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->lower, td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"))*TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
  	  x2 = (td_date_period_day (TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->lower, begin)*TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)-x-delta_x;
  	  gnome_canvas_item_move (item, x2, 0);
  	  td_database_command (g_strdup_printf ("UPDATE task SET begin = %s, eend = %s WHERE td_id = %d;", td_database_adaptvalue (td_date_print (begin, "year-month-day"), "date"), td_database_adaptvalue (td_date_print (end, "year-month-day"), "date"), CURRENT_ID));
	  td_mod_refresh_editprop (TD_MOD (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id)), GTK_OBJECT (mod_gantt));
	  TD_MOD_GANTT (mod_gantt)->dragging = FALSE;
  	  FLAG_NO_REFRESH_NODE = FALSE;
	}

      /* Taille dbut */
      if (TD_MOD_GANTT (mod_gantt)->dragging == 2)
	{

	  /* Nouvelle date */
       	  FLAG_NO_REFRESH_NODE = TRUE;
	  delta_x = TD_MOD_GANTT (mod_gantt)->dragging_x - TD_MOD_GANTT (mod_gantt)->init_x;
	  if (delta_x>0)
	    begin = td_date_add_days (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), ((delta_x/TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)+0.5));
	  else
	    begin = td_date_add_days (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), ((delta_x/TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)-0.5));
	  if (strcmp (td_date_print (begin, "year-month-day"), td_database_current ("TD_CURRENT begin;")))
	    td_mod_modified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id));

	  /* Mouvement */
  	  x = td_date_period_day (TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->lower, td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"))*TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
  	  x2 = (td_date_period_day (TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->lower, begin)*TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)-x-delta_x;
  	  gnome_canvas_item_move (item, x2, 0);
	  x3 = GNOME_CANVAS_RE (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 0))->x2;
	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 0),
				 "x2", x3-x2, NULL);
  	  td_database_command (g_strdup_printf ("UPDATE task SET begin = %s WHERE td_id = %d;", td_database_adaptvalue (td_date_print (begin, "year-month-day"), "date"), CURRENT_ID));
	  td_mod_refresh_editprop (TD_MOD (g_list_nth_data (TD_APP(APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id)), GTK_OBJECT (mod_gantt));
	  TD_MOD_GANTT (mod_gantt)->dragging = FALSE;
  	  FLAG_NO_REFRESH_NODE = FALSE;
	}

      /* Taille fin */
      if (TD_MOD_GANTT (mod_gantt)->dragging == 3)
	{

	  /* Nouvelle date */
       	  FLAG_NO_REFRESH_NODE = TRUE;
	  delta_x = TD_MOD_GANTT (mod_gantt)->dragging_x - TD_MOD_GANTT (mod_gantt)->init_x;
	  if (delta_x>0)
	    end = td_date_add_days (td_date_parse (td_database_current ("TD_CURRENT eend;"), "year-month-day"), ((delta_x/TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)+0.5));
	  else
	    end = td_date_add_days (td_date_parse (td_database_current ("TD_CURRENT eend;"), "year-month-day"), ((delta_x/TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width)-0.5));
	  if (strcmp (td_date_print (end, "year-month-day"), td_database_current ("TD_CURRENT eend;")))
	    td_mod_modified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id));

	  /* Mouvement */
  	  x = td_date_period_day (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), end)*TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 0),
				 "x2", x, NULL);
  	  td_database_command (g_strdup_printf ("UPDATE task SET eend = %s WHERE td_id = %d;", td_database_adaptvalue (td_date_print (end, "year-month-day"), "date"), CURRENT_ID));
	  td_mod_refresh_editprop (TD_MOD (g_list_nth_data (TD_APP(APP)->mod_data, TD_MOD_GANTT (mod_gantt)->id)), GTK_OBJECT (mod_gantt));
	  TD_MOD_GANTT (mod_gantt)->dragging = FALSE;
  	  FLAG_NO_REFRESH_NODE = FALSE;
	}
      break;
    default:
      break;
    }
  return FALSE;
}

/*****************************************************************************/
/* Proprit */
/*****************************************************************************/
void editprop_def (GtkObject *mod)
{
  GtkObject *editprop[4];
  GtkWidget *prop_widget[2];

  /* Nom */
  editprop[0] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[0]), 
		  "name", "name",
		  "name_intl", _("Name"),
		  "query_refresh", "TD_CURRENT name;",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[0]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[0]);

  /* Dbut */
  editprop[1] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[1]), 
		  "name", "begin",
		  "name_intl", _("Begin"),
		  "query_refresh", "TD_CURRENT begin;",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  prop_widget[0] = gtdk_entry_date_selector (NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[1]),
			      g_list_nth_data (gtk_container_children (GTK_CONTAINER (prop_widget[0])), 0),
			      prop_widget[0]);
  td_mod_add_editprop (TD_MOD (mod), editprop[1]);

  /* Fin */
  editprop[2] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[2]), 
		  "name", "end",
		  "name_intl", _("End"),
		  "query_refresh", "TD_CURRENT eend;",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  prop_widget[1] = gtdk_entry_date_selector (NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[2]),
			      g_list_nth_data (gtk_container_children (GTK_CONTAINER (prop_widget[1])), 0),
			      prop_widget[1]);
  td_mod_add_editprop (TD_MOD (mod), editprop[2]);

  /* Dure *//* FIXME: +tards */
/*    editprop[3] = td_mod_editprop_new (); */
/*    gtk_object_set (GTK_OBJECT (editprop[3]),  */
/*  		  "name", "duration", */
/*  		  "name_intl", _("Duration"), */
/*  		  "query_refresh", "TD_CURRENT duration;", */
/*  		  "query_visible", "TD_CURRENT td_id;", NULL); */
/*    td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[3]), gtk_entry_new (), NULL); */
/*    td_mod_add_editprop (TD_MOD (mod), editprop[3]); */
}

void plugins_editprop_refresh (GtkObject *mod, GtkObject *mod_net)
{
  td_database_set_current (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
}

void plugins_editprop_action (GtkObject *mod, gchar *value, int editprop_id)
{
  TdModGantt *mod_gantt;
  double x;
  mod_gantt = TD_MOD_GANTT (g_list_nth_data (TD_MOD (mod)->child, 0));

  /* Go ! */
  FLAG_NO_REFRESH_NODE = TRUE;
  switch (editprop_id)
    {
    case 0:
      gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 1), 
			     "text", value, NULL);
      td_database_command (g_strdup_printf ("UPDATE task SET name = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    case 1:
      if (!td_date_parse (value, "year-month-day"))
	break;
/*        printf ("***** debug txt_tmp:%s \n", td_date_print (date_tmp, "year-month-day")); */
      x = td_date_period_day (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), td_date_parse (value, "year-month-day")) *TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
      gnome_canvas_item_move (GNOME_CANVAS_ITEM (TD_MOD_GANTT (mod_gantt)->selected), x, 0);
      x = td_date_period_day (td_date_parse (value, "year-month-day"), td_date_parse (td_database_current ("TD_CURRENT eend;"), "year-month-day")) *TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
      gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 0), 
			     "x2", x, NULL);
      td_database_command (g_strdup_printf ("UPDATE task SET begin = %s WHERE td_id = %d;", td_database_adaptvalue (value, "date"), CURRENT_ID));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    case 2:
      if (!td_date_parse (value, "year-month-day"))
	break;
      x = td_date_period_day (td_date_parse (td_database_current ("TD_CURRENT begin;"), "year-month-day"), td_date_parse (value, "year-month-day"))*TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width;
      gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_GANTT (mod_gantt)->selected)->item_list, 0), 
			     "x2", gtdk_canvas_w2c_x (x), NULL);
      td_database_command (g_strdup_printf ("UPDATE task SET eend = %s WHERE td_id = %d;", td_database_adaptvalue (value, "date"), CURRENT_ID));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
      /* FIXME: +tards */
/*      case 3: */
/*        td_database_command (g_strdup_printf ("UPDATE task SET duration = %s WHERE td_id = %d;", value, CURRENT_ID)); */
/*        td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0))); */
/*        break; */
    default:
      break;
    }
  FLAG_NO_REFRESH_NODE = FALSE;
}


/*****************************************************************************/
/* Menu */
/*****************************************************************************/
void menu_def (GtkObject *mod)
{
  GtkObject *menu[3];

  /* Crer */
  menu[0] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[0]),
		  "label", _("Create"),
		  "icon", "list_new.xpm",
		  "query_visible", "TD_CURRENT (null);", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[0]);

  /* Supprimer */
  menu[1] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[1]),
		  "label", _("Remove"),
		  "icon", "list_trash.xpm",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[1]);
  td_mod_add_menu (TD_MOD (mod), td_mod_menu_new ());
}

void plugins_menu_refresh (GtkObject *mod, GtkObject *mod_net)
{
  td_database_set_current (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
}

void plugins_menu_action (GtkObject *mod, int menu_id)
{
  GList *row;
  TdModGantt *mod_gantt;
  int id_sibling;
  int id_parent;
  GDate *day_tmp;
  mod_gantt = TD_MOD_GANTT (g_list_nth_data (TD_MOD (mod)->child, 0));

  /* Action */
  switch (menu_id)
    {
    case 0:

      day_tmp = td_date_add_days (TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->lower, TD_MOD_GANTT (mod_gantt)->init_x/TD_TIMERULER (TD_MOD_GANTT (mod_gantt)->timeruler)->width);
      td_database_insert (g_strdup_printf ("INSERT INTO task (name, begin, eend, td_y) VALUES (NULL, %s, %s, %d);",
					   td_database_adaptvalue (td_date_print (day_tmp, "year-month-day"), "date"),
					   td_database_adaptvalue (td_date_print (td_date_add_days (day_tmp, 1), "year-month-day"), "date"),
					   (atoi (td_database_value ("SELECT COUNT (td_id) FROM task;")))*mod_gantt->row_height), "task");
      row = td_database_row (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
      add_node (GTK_OBJECT (mod_gantt), row, 0);
      break;
    case 1:
      td_database_command (g_strdup_printf ("DELETE FROM task_net WHERE td_id = %d;", CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task_net WHERE td_id_parent = %d;", CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task WHERE td_id = %d;", CURRENT_ID));
      remove_node (GTK_OBJECT (mod_gantt));
      CURRENT_ID = 0;
      CURRENT_ID_PARENT = 0;
      CURRENT_ROW = NULL;
      mod_gantt->selected = NULL;
      break;
    default:
      break;
    }
}
