/*****************************************************************************/
/* main.c : Donnes globales et fentre principale
 * main.c : Global data and main window
 *
 *
 * Greffon ToutDoux : pert : Diagramme PERT
 * ToutDoux's plug-in : pert : PERT diagram
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"

/*****************************************************************************/
/* Module */
/*****************************************************************************/
GtkObject *plugins_about_def (void)
{
  GtkObject *ret;
  ret = td_about_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", _("PERT diagram"),
		  "type", _("view"),
		  "version", "1.0.2",
		  "authors", "Philippe Roy <ph_roy@toutdoux.org>",
		  "help", "(null)",
		  "category1", _("project manager"),
		  "category2", "pert",
		  "category3", "(null)",
		  "copyright", "Copyright (c) 2001 Philippe Roy",
		  "license", _("Covered by the GNU General Public License"),
		  "icon", g_strdup_printf ("%s/share/pixmaps/toutdoux/icon_pert.xpm", TD_PACKAGE_DATA_DIR),
		  "web", "(null)", NULL);
  return ret;
}

GtkObject *plugins_init (int mod_id, gchar *mod_name)
{
  GtkObject *mod;

  /* Module */
  mod = td_mod_new ();
  if (!mod_name)
    mod_name = _("PERT diagram");
  gtk_object_set (GTK_OBJECT (mod),
		  "name", "pert",
		  "name_intl", mod_name,
		  "group", _("Project"),
		  "id", mod_id,
		  "fields", "name, td_id, td_obso, td_x, td_y", NULL);
  td_mod_set_about (TD_MOD (mod), plugins_about_def ());

  /* Dfinitions */
  child_def (mod);
  editprop_def (mod);
  menu_def (mod);

  /* Cration */
  td_mod_create (TD_MOD (mod));
  return mod;
}


/*****************************************************************************/
/* Enfant */
/*****************************************************************************/
void child_def (GtkObject *mod)
{
  GtkObject *canvas;

  /* Box */
  canvas = td_mod_canvas_new ();
  gtk_object_set (GTK_OBJECT (canvas), 
		  "name", "pert",
		  "name_intl", _("PERT diagram"),
		  "reorderable", TRUE,
		  "customize", TRUE,
		  "table", "task",
		  "table_net", "task_net",
		  "column_oid", 1, NULL);
  td_mod_add_child (TD_MOD (mod), canvas);
}

void plugins_refresh (GtkObject *mod, GtkObject *mod_canvas)
{
  GtkObject *datatable;
  int i;

  /* Rangement *//* FIXME: + tards */

  /* Node */
  datatable = td_database_select (g_strdup_printf ("SELECT %s FROM task;", TD_MOD (mod)->fields));
  for (i=0; i<g_list_length (TD_DB_DATATABLE (datatable)->item); i++)
    add_node (mod_canvas, g_list_nth_data (TD_DB_DATATABLE (datatable)->item, i), 0);
  add_link (mod_canvas);
}


/*****************************************************************************/
/* Symbole */
/*****************************************************************************/
void plugins_symbol_def (GtkObject *mod_canvas)
{
}

void add_link (GtkObject *mod_canvas)
{
  int i;
  GnomeCanvasGroup *root;
  GnomeCanvasItem *line;
  GnomeCanvasItem *arrow;
  GnomeCanvasPoints *points;
  GdkColor line_color;
  GtkObject *datatable_net;
  int item_x1, item_y1, item_x2, item_y2;
  gchar *txt_tmp;
  root = gnome_canvas_root (GNOME_CANVAS (TD_MOD_CANVAS (mod_canvas)->widget_data));

  /* Go ! */
  line_color = gtdk_color_parse ("fg[normal]");
  datatable_net = td_database_select (g_strdup_printf ("SELECT b.name, b.td_id, b.td_x, b.td_y, c.name, c.td_id, c.td_x, c.td_y FROM task_net a, task b, task c WHERE b.td_id=a.td_id AND c.td_id= a.td_id_parent;"));
  for (i=0; i<g_list_length (TD_DB_DATATABLE (datatable_net)->item); i++)
    {

      /* Ligne */
      points = gnome_canvas_points_new (2);
      points->coords[0] = atoi (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 2));
      points->coords[1] = atoi (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 3));
      points->coords[2] = atoi (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 6));
      points->coords[3] = atoi (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 7));
      line = gnome_canvas_item_new (root, gnome_canvas_line_get_type (),
				    "points", points,
				    "fill_color_gdk", &line_color,
				    "width_units", 2.0, NULL);
      link_setup (mod_canvas, GNOME_CANVAS_ITEM (line));
      gnome_canvas_item_lower_to_bottom (line);
      gtk_object_set_user_data (GTK_OBJECT (line), g_strdup_printf ("%s>%s", 
								    (gchar*) (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 1)),
								    (gchar*) (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 5))));

      /* Flche */
      arrow = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
				     "x1", (double) 0, "y1", (double) 0, "x2", (double) 0, "y2", (double) 0,
				     "fill_color_gdk", &line_color,
				     "outline_color_gdk", &line_color,
				     "width_units", 1.0, NULL);
      gnome_canvas_item_hide (arrow);
      gnome_canvas_item_lower_to_bottom (arrow);
      gtk_object_set_user_data (GTK_OBJECT (arrow), g_strdup_printf ("%s>%s_arrow", 
								     (gchar*) (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 1)),
								     (gchar*) (g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 5))));
      txt_tmp = g_list_nth_data (g_list_nth_data (TD_DB_DATATABLE (datatable_net)->item, i), 0);
      item_x1 = points->coords[0]-(10+gdk_string_width (gtk_widget_get_default_style()->font, txt_tmp)/2);
      item_y1 = points->coords[1]-(10+gdk_string_height (gtk_widget_get_default_style()->font, txt_tmp)/2);
      item_x2 = points->coords[0]+(10+gdk_string_width (gtk_widget_get_default_style()->font, txt_tmp)/2);
      item_y2 = points->coords[1]+(10+gdk_string_height (gtk_widget_get_default_style()->font, txt_tmp)/2);
      gtdk_canvas_link_xy (item_x1, item_y1, item_x2, item_y2, 
  			   points->coords[0], points->coords[1], points->coords[2], points->coords[3],
			   arrow);
      gnome_canvas_item_show (arrow);
      gnome_canvas_points_free (points);
    }
}

gboolean add_node (GtkObject *mod_canvas, GList *row, int id_parent)
{
  GnomeCanvasGroup *root;
  GnomeCanvasGroup *group;
  GnomeCanvasItem *line;
  GnomeCanvasItem *arrow;
  GnomeCanvasPoints *points;
  GList *row_parent;
  GtkObject *draw[2];
  GdkColor fill_color1;
  GdkColor line_color1;
  GdkColor fill_color2;
  gchar *txt_tmp;
  int x, y;
  root = gnome_canvas_root (GNOME_CANVAS (TD_MOD_CANVAS (mod_canvas)->widget_data));

  /* Action */
  x = atoi (g_list_nth_data (row, 3));
  y = atoi (g_list_nth_data (row, 4));
  group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (root, gnome_canvas_group_get_type (), "x", (double) x, "y", (double) y, NULL));
  node_setup (mod_canvas, GNOME_CANVAS_ITEM (group));
  gtk_object_set_user_data (GTK_OBJECT (group), g_list_nth_data (row, TD_MOD_CANVAS (mod_canvas)->column_oid));

  /* Cadre */
  txt_tmp = g_list_nth_data (row, 0);
  node_setup (mod_canvas, GNOME_CANVAS_ITEM (group));
  fill_color1 = gtdk_color_parse ("base[normal]");
  line_color1 = gtdk_color_parse ("fg[normal]");
  gnome_canvas_item_new (group, gnome_canvas_rect_get_type (),
			 "x1", (double) -10-gdk_string_width (gtk_widget_get_default_style()->font, txt_tmp)/2,
			 "y1", (double) -10-gdk_string_height (gtk_widget_get_default_style()->font, txt_tmp)/2,
			 "x2", (double) 10+gdk_string_width (gtk_widget_get_default_style()->font, txt_tmp)/2,
			 "y2", (double) 10+(gdk_string_height (gtk_widget_get_default_style()->font, txt_tmp)/2),
			 "fill_color_gdk", &fill_color1,
			 "outline_color_gdk", &line_color1,
			 "width_units", 1.0, NULL);

  /* Nom */
  fill_color2 = gtdk_color_parse ("text[normal]");
  gnome_canvas_item_new (group, gnome_canvas_text_get_type (),
			 "text", txt_tmp,
			 "x", (double) 0,
			 "y", (double) 0,
			 "font", "-misc-fixed-medium-r-semicondensed-*-*-120-*-*-c-*-iso8859-1",
			 "justification", GTK_JUSTIFY_CENTER,
			 "anchor", GTK_ANCHOR_CENTER,
			 "fill_color_gdk", &fill_color2, NULL);

  /* Ligne */
  if (id_parent == 0)
    return TRUE;
  row_parent = td_database_row (g_strdup_printf ("SELECT td_x, td_y FROM task WHERE td_id = %d;", id_parent));
  points = gnome_canvas_points_new (2);
  points->coords[0] = x;
  points->coords[1] = y;
  points->coords[2] = atoi (g_list_nth_data (row_parent, 0));
  points->coords[3] = atoi (g_list_nth_data (row_parent, 1));
  line = gnome_canvas_item_new (root, gnome_canvas_line_get_type (),
				"points", points,
				"fill_color_gdk", &line_color1,
				"width_units", 2.0, NULL);
  link_setup (mod_canvas, GNOME_CANVAS_ITEM (line));
  gnome_canvas_item_lower_to_bottom (line);
  gtk_object_set_user_data (GTK_OBJECT (line), g_strdup_printf ("%s>%d", (gchar*) g_list_nth_data (row, 1), id_parent));
  gnome_canvas_points_free (points);

  /* Flche */
  arrow = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
				 "x1", (double) 0, "y1", (double) 0, "x2", (double) 0, "y2", (double) 0,
				 "fill_color_gdk", &line_color1,
				 "outline_color_gdk", &line_color1,
				 "width_units", 1.0, NULL);
  gnome_canvas_item_hide (arrow);
  gnome_canvas_item_lower_to_bottom (arrow);
  gtk_object_set_user_data (GTK_OBJECT (arrow), g_strdup_printf ("%s>%d_arrow", (gchar*) g_list_nth_data (row, 1), id_parent));
  gtdk_canvas_link (g_list_nth_data (GNOME_CANVAS_GROUP (group)->item_list, 0), line, arrow);
  gnome_canvas_item_show (arrow);
  return TRUE;
}

void update_node (GtkObject *mod_canvas)
{
  GList *row;
  gchar *txt_tmp;
  row = td_database_row (g_strdup_printf ("SELECT name FROM %s WHERE td_id = %d;", TD_MOD_CANVAS (mod_canvas)->table, CURRENT_ID));

  /* Cadre */
  txt_tmp = g_list_nth_data (row, 0);
  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_CANVAS (mod_canvas)->selected)->item_list, 0),
			 "x1", (double) -10-gdk_string_width (gtk_widget_get_default_style()->font, txt_tmp)/2,
			 "y1", (double) -10-gdk_string_height (gtk_widget_get_default_style()->font, txt_tmp)/2,
			 "x2", (double) 10+gdk_string_width (gtk_widget_get_default_style()->font, txt_tmp)/2,
			 "y2", (double) 10+(gdk_string_height (gtk_widget_get_default_style()->font, txt_tmp)/2), NULL);

  /* Nom */
  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_CANVAS (mod_canvas)->selected)->item_list, 1),
			 "text", txt_tmp, NULL);
}

void remove_link (GtkObject *mod_canvas)
{
  int i;
  for (i=0; i<g_list_length (TD_MOD_CANVAS (mod_canvas)->selected_link_in); i++)
    {
      gtk_object_destroy (GTK_OBJECT (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i)));
      gtk_object_destroy (GTK_OBJECT (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_arrow_in, i)));
    }
  for (i=0; i<g_list_length (TD_MOD_CANVAS (mod_canvas)->selected_link_out); i++)
    {
      gtk_object_destroy (GTK_OBJECT (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i)));
      gtk_object_destroy (GTK_OBJECT (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_arrow_out, i)));
    }
}

void remove_node (GtkObject *mod_canvas)
{
  GnomeCanvasGroup *root;
  GnomeCanvasItem *item;
  gchar *txt_id;
  int i = 0;
  root = gnome_canvas_root (GNOME_CANVAS (TD_MOD_CANVAS (mod_canvas)->widget_data));

  /* Go ! */
  item = g_list_nth_data (root->item_list, i);
  while (item)
    {
      txt_id = gtk_object_get_user_data (GTK_OBJECT (item));
      if (strstr (txt_id, g_strdup_printf ("%d", CURRENT_ID)))
	{
	  gtk_object_destroy (GTK_OBJECT (item));
	  item = g_list_nth_data (root->item_list, i);
	}
      else
	item = g_list_nth_data (root->item_list, ++i);
    }
}

void link_setup (GtkObject *mod_canvas, GnomeCanvasItem *item)
{
  gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (link_event), mod_canvas);
}

gboolean link_event (GnomeCanvasItem *item, GdkEvent *event, GtkObject *mod_canvas)
{
  GdkColor color;
  gboolean bool_tmp;
  int i;

  /* Go ! */
  switch (event->type) 
    {

      /* Eclairage */
    case GDK_ENTER_NOTIFY:
      color = gtdk_color_parse ("bg[prelight]");
      gnome_canvas_item_set (item, 
			     "fill_color_gdk", &color, NULL);
      break;
    case GDK_LEAVE_NOTIFY:
      if (!(event->crossing.state & GDK_BUTTON1_MASK))
	{
	  bool_tmp = FALSE;
	  for (i=0; i<g_list_length (TD_MOD_CANVAS (mod_canvas)->selected_link_in); i++)
	    if (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i) == item)
	      {
		bool_tmp = TRUE;
		break;
	      }
	  if (!bool_tmp)
	    for (i=0; i<g_list_length (TD_MOD_CANVAS (mod_canvas)->selected_link_out); i++)
	      if (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i) == item)
		{
		  bool_tmp = TRUE;
		  break;
		}
	  if (bool_tmp)
	    {
	      color = gtdk_color_invert (gtdk_color_parse ("base[normal]"));
	      gdk_color_alloc (gtk_widget_get_default_colormap (), &color);
	    }
	  else
	    color = gtdk_color_parse ("fg[normal]");
	  gnome_canvas_item_set (item, 
				 "fill_color_gdk", &color, NULL);
	}
      break;
    default:
      break;
    }
  return FALSE;
}

void node_setup (GtkObject *mod_canvas, GnomeCanvasItem *item)
{
  gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (node_event), mod_canvas);
}

gboolean node_event (GnomeCanvasItem *item, GdkEvent *event, GtkObject *mod_canvas)
{
  GnomeCanvasGroup *root;
  GnomeCanvasItem *node;
  GnomeCanvasItem *line;
  GnomeCanvasItem *arrow;
  GList *row;
  GList *row_parent;
  GnomeCanvasPoints *points;
  GdkColor color;
  GdkCursor *fleur;
  double new_x, new_y, item_x, item_y, x, y;
  int i, cx, cy;
  GdkCursor *cursor;
  int delta_x, delta_y;
  gchar *txt_tmp;
  item_x = event->button.x;
  item_y = event->button.y;
  gnome_canvas_item_w2i (item->parent, &item_x, &item_y);
  root = gnome_canvas_root (GNOME_CANVAS (TD_MOD_CANVAS (mod_canvas)->widget_data));

  /* Go ! */
  switch (event->type) 
    {

      /* Eclairage */
    case GDK_ENTER_NOTIFY:
      if (GNOME_IS_CANVAS_GROUP (item))
	{
	  color = gtdk_color_parse ("bg[prelight]");
  	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 0), 
				 "fill_color_gdk", &color, NULL);
	  color = gtdk_color_parse ("text[prelight]");
  	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 1), 
				 "fill_color_gdk", &color, NULL);
	}
      break;
    case GDK_LEAVE_NOTIFY:
      if ((GNOME_IS_CANVAS_GROUP (item)) && (!(event->crossing.state & GDK_BUTTON1_MASK)))
	{
	  color = gtdk_color_parse ("base[normal]");
	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 0), 
				 "fill_color_gdk", &color, NULL);
	  color = gtdk_color_parse ("text[normal]");
  	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (item)->item_list, 1), 
				 "fill_color_gdk", &color, NULL);
	}
      break;

      /* Selection */
    case GDK_BUTTON_PRESS:

      /* Liaison */
      if ((event->button.button == 1) && (event->button.state & GDK_SHIFT_MASK) && (GNOME_IS_CANVAS_GROUP (item)))
	{
	  color = gtdk_color_parse ("fg[normal]");
	  cursor = gtdk_cursor (TD_STOCK_CURSOR_PENCIL, TD_STOCK_CURSOR_PENCIL_MASK);
	  gnome_canvas_item_grab (item, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, cursor, event->button.time);
  	  gdk_cursor_destroy (cursor);
	  TD_MOD_CANVAS (mod_canvas)->dragging_x = item_x;
	  TD_MOD_CANVAS (mod_canvas)->dragging_y = item_y;
	  TD_MOD_CANVAS (mod_canvas)->dragging = 2;
	  points = gnome_canvas_points_new (2);
	  points->coords[0] = item_x;
	  points->coords[1] = item_y;
	  points->coords[2] = item_x;
	  points->coords[3] = item_y;
	  TD_MOD_CANVAS (mod_canvas)->link = gnome_canvas_item_new (root, gnome_canvas_line_get_type (),
								    "points", points,
								    "fill_color_gdk", &color,
								    "width_units", 2.0, NULL);
	  gtk_object_set_user_data (GTK_OBJECT (TD_MOD_CANVAS (mod_canvas)->link), "link");
	  gnome_canvas_points_free (points);
	}

      /* Dplacement de la node */
      else
	if ((event->button.button == 1) && (GNOME_IS_CANVAS_GROUP (item)))
	  {
	    cursor = gtdk_cursor (TD_STOCK_CURSOR_MOVE, TD_STOCK_CURSOR_MOVE_MASK);
	    gnome_canvas_item_grab (item, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, cursor, event->button.time);
	    gdk_cursor_destroy (cursor);
	    TD_MOD_CANVAS (mod_canvas)->dragging_x = item_x;
	    TD_MOD_CANVAS (mod_canvas)->dragging_y = item_y;
	    TD_MOD_CANVAS (mod_canvas)->init_x = item_x;
	    TD_MOD_CANVAS (mod_canvas)->init_y = item_y;
	    TD_MOD_CANVAS (mod_canvas)->dragging = 1;
	  }
      break;

      /* Dplacement */
    case GDK_MOTION_NOTIFY:
      if ((TD_MOD_CANVAS (mod_canvas)->dragging == 1) &&
	  (event->motion.state & GDK_BUTTON1_MASK)) /* Dplacement de la node */
	{
	  new_x = item_x;
	  new_y = item_y;

	  /* Node */
	  gnome_canvas_item_move (item, new_x - TD_MOD_CANVAS (mod_canvas)->dragging_x, new_y - TD_MOD_CANVAS (mod_canvas)->dragging_y);

	  /* Liens entrants */
	  for (i=0; i<g_list_length (TD_MOD_CANVAS (mod_canvas)->selected_link_in); i++)
	    {
	      points = gnome_canvas_points_new (2);
  	      points->coords[0] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i))->coords[0] + new_x - TD_MOD_CANVAS (mod_canvas)->dragging_x;
  	      points->coords[1] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i))->coords[1] + new_y - TD_MOD_CANVAS (mod_canvas)->dragging_y;
  	      points->coords[2] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i))->coords[2];
  	      points->coords[3] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i))->coords[3];
	      gnome_canvas_item_set (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i), 
				     "points", points, NULL);

	      /* Flche entrante */
	      gtdk_canvas_link (g_list_nth_data (GNOME_CANVAS_GROUP (TD_MOD_CANVAS (mod_canvas)->selected)->item_list, 0),
				g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_in, i),
				g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_arrow_in, i));

	      /* Flche sortante *//* FIXME: + tards */
	      gnome_canvas_points_free (points);
	    }

	  /* Liens sortants */
  	  for (i=0; i<g_list_length (TD_MOD_CANVAS (mod_canvas)->selected_link_out); i++)
	    {
	      points = gnome_canvas_points_new (2);
  	      points->coords[0] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i))->coords[0];
  	      points->coords[1] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i))->coords[1];
	      points->coords[2] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i))->coords[2] + new_x - TD_MOD_CANVAS (mod_canvas)->dragging_x;
  	      points->coords[3] = GNOME_CANVAS_LINE (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i))->coords[3] + new_y - TD_MOD_CANVAS (mod_canvas)->dragging_y;
	      gnome_canvas_item_set (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i), 
				     "points", points, NULL);

	      /* Flche entrante *//* FIXME: + tards */

	      /* Flche sortante */
	      gtdk_canvas_link (g_list_nth_data (GNOME_CANVAS_GROUP (g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_child, i))->item_list, 0),
				g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_out, i),
				g_list_nth_data (TD_MOD_CANVAS (mod_canvas)->selected_link_arrow_out, i));
  	      gnome_canvas_points_free (points);
	    }
	  TD_MOD_CANVAS (mod_canvas)->dragging_x = new_x;
	  TD_MOD_CANVAS (mod_canvas)->dragging_y = new_y;
	}

      /* Liaison *//* FIXME: prendre en compte les items : +tards */
      if ((TD_MOD_CANVAS (mod_canvas)->dragging == 2) && 
	  (event->motion.state & GDK_BUTTON1_MASK))
	{
	  new_x = item_x;
	  new_y = item_y;
	  points = gnome_canvas_points_new (2);
	  points->coords[0] = GNOME_CANVAS_LINE (TD_MOD_CANVAS (mod_canvas)->link)->coords[0];
	  points->coords[1] = GNOME_CANVAS_LINE (TD_MOD_CANVAS (mod_canvas)->link)->coords[1];
	  points->coords[2] = new_x;
	  points->coords[3] = new_y;
	  gnome_canvas_item_set (TD_MOD_CANVAS (mod_canvas)->link, 
				 "points", points, NULL);
	  gnome_canvas_points_free (points);
	}
      break;

    case GDK_BUTTON_RELEASE:
      gnome_canvas_item_ungrab (item, event->button.time);

      /* Dplacement de la node */
      if (TD_MOD_CANVAS (mod_canvas)->dragging == 1)
	{
	  delta_x = TD_MOD_CANVAS (mod_canvas)->dragging_x-TD_MOD_CANVAS (mod_canvas)->init_x;
	  delta_y = TD_MOD_CANVAS (mod_canvas)->dragging_y-TD_MOD_CANVAS (mod_canvas)->init_y;
	  td_database_command (g_strdup_printf ("UPDATE task SET td_x = %d, td_y = %d WHERE td_id = %d;",
						(int) atoi (td_database_current ("TD_CURRENT td_x;"))+delta_x,
						(int) atoi (td_database_current ("TD_CURRENT td_y;"))+delta_y, CURRENT_ID));
	  td_mod_modified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_CANVAS (mod_canvas)->id));
	  TD_MOD_CANVAS (mod_canvas)->dragging = FALSE;
	  break;
	}

      /* Liaison */
      if (TD_MOD_CANVAS (mod_canvas)->dragging != 2)
	{
	  TD_MOD_CANVAS (mod_canvas)->dragging = FALSE;
	  break;
	}
      gtk_object_destroy (GTK_OBJECT (TD_MOD_CANVAS (mod_canvas)->link));
      TD_MOD_CANVAS (mod_canvas)->link = NULL;
      node = gnome_canvas_get_item_at (GNOME_CANVAS (TD_MOD_CANVAS (mod_canvas)->widget_data), item_x, item_y);
      if ((!node) || (GNOME_IS_CANVAS_LINE (node)))
	{
	  TD_MOD_CANVAS (mod_canvas)->dragging = FALSE;
	  break;
	}
      CURRENT_ID_PARENT = CURRENT_ID;
      CURRENT_ID = atoi (gtk_object_get_user_data (GTK_OBJECT (node->parent)));
      if (CURRENT_ID == CURRENT_ID_PARENT)
	{
	  TD_MOD_CANVAS (mod_canvas)->dragging = FALSE;
	  break;
	}
      if ((td_database_value (g_strdup_printf ("SELECT td_id FROM task_net WHERE td_id = %d AND td_id_parent = %d;", CURRENT_ID, CURRENT_ID_PARENT))) ||
	  (td_database_value (g_strdup_printf ("SELECT td_id FROM task_net WHERE td_id = %d AND td_id_parent = %d;", CURRENT_ID_PARENT, CURRENT_ID))))
	{
	  TD_MOD_CANVAS (mod_canvas)->dragging = FALSE;
	  break;
	}
      row = td_database_row (g_strdup_printf ("SELECT td_x, td_y FROM task WHERE td_id = %d;", CURRENT_ID));
      row_parent = td_database_row (g_strdup_printf ("SELECT td_x, td_y FROM task WHERE td_id = %d;", CURRENT_ID_PARENT));
      points = gnome_canvas_points_new (2);
      points->coords[0] = atoi (g_list_nth_data (row, 0));
      points->coords[1] = atoi (g_list_nth_data (row, 1));
      points->coords[2] = atoi (g_list_nth_data (row_parent, 0));
      points->coords[3] = atoi (g_list_nth_data (row_parent, 1));
      color = gtdk_color_parse ("fg[normal]");
      line = gnome_canvas_item_new (root, gnome_canvas_line_get_type (),
				    "points", points,
				    "fill_color_gdk", &color,
				    "width_units", 2.0, NULL);
      gnome_canvas_item_lower_to_bottom (line);
      gtk_object_set_user_data (GTK_OBJECT (line), g_strdup_printf ("%d>%d", CURRENT_ID, CURRENT_ID_PARENT));
      link_setup (mod_canvas, line);
      arrow = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
				     "x1", (double) 0, "y1", (double) 0, "x2", (double) 0, "y2", (double) 0,
				     "fill_color_gdk", &color,
				     "outline_color_gdk", &color,
				     "width_units", 1.0, NULL);
      gnome_canvas_item_hide (arrow);
      gnome_canvas_item_lower_to_bottom (arrow);
      gtk_object_set_user_data (GTK_OBJECT (arrow), g_strdup_printf ("%d>%d_arrow", CURRENT_ID, CURRENT_ID_PARENT));
      gtdk_canvas_link (g_list_nth_data (GNOME_CANVAS_GROUP (node->parent)->item_list, 0), line, arrow);
      gnome_canvas_item_show (arrow);
      td_database_insert (g_strdup_printf ("INSERT INTO task_net (td_id, td_id_parent) VALUES (%d, %d);", CURRENT_ID, CURRENT_ID_PARENT), "task_net");
      td_mod_modified (g_list_nth_data (TD_APP (APP)->mod_data, TD_MOD_CANVAS (mod_canvas)->id));
      TD_MOD_CANVAS (mod_canvas)->dragging = FALSE;
      break;
    default:
      break;
    }
  return FALSE;
}

/*****************************************************************************/
/* Proprit */
/*****************************************************************************/
void editprop_def (GtkObject *mod)
{
  GtkObject *editprop[1];

  /* Nom */
  editprop[0] = td_mod_editprop_new ();
  gtk_object_set (GTK_OBJECT (editprop[0]), 
		  "name", "name",
		  "name_intl", _("Name"),
		  "query_refresh", "TD_CURRENT name;",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_editprop_set_widget (TD_MOD_EDITPROP (editprop[0]), gtk_entry_new (), NULL);
  td_mod_add_editprop (TD_MOD (mod), editprop[0]);
}

void plugins_editprop_refresh (GtkObject *mod, GtkObject *mod_net)
{
  td_database_set_current (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
}

void plugins_editprop_action (GtkObject *mod, gchar *value, int editprop_id)
{

  /* Go ! */
  FLAG_NO_REFRESH_NODE = TRUE;
  switch (editprop_id)
    {
    case 0:
      td_database_command (g_strdup_printf ("UPDATE task SET name = %s WHERE td_id = %d;", td_database_adaptvalue (value, "text"), CURRENT_ID));
      update_node (GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      td_mod_refresh_editprop (TD_MOD (mod), GTK_OBJECT (g_list_nth_data (TD_MOD (mod)->child, 0)));
      break;
    default:
      break;
    }
  FLAG_NO_REFRESH_NODE = FALSE;
}


/*****************************************************************************/
/* Menu */
/*****************************************************************************/
void menu_def (GtkObject *mod)
{
  GtkObject *menu[5];
  GtkObject *menu_link[1];

  /* Crer */
  menu[0] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[0]),
		  "label", _("Create"),
		  "icon", "list_new.xpm",
		  "query_visible", "TD_CURRENT (null);", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[0]);

  menu[1] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[1]),
		  "label", _("Create linked"),
		  "icon", "list_new.xpm",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[1]);
  td_mod_add_menu (TD_MOD (mod), td_mod_menu_new ());

  /* Supprimer */
  menu[3] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[3]),
		  "label", _("Remove"),
		  "icon", "list_trash.xpm",
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[3]);

  menu[4] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu[4]),
		  "query_visible", "TD_CURRENT td_id;", NULL);
  td_mod_add_menu (TD_MOD (mod), menu[4]);

  /* Menu des liens : Crer */
  menu_link[0] = td_mod_menu_new ();
  gtk_object_set (GTK_OBJECT (menu_link[0]), 
		  "label", _("Remove"),
		  "icon", "list_trash.xpm", NULL);
  td_mod_add_menu_link (TD_MOD (mod), menu_link[0]);
  td_mod_add_menu_link (TD_MOD (mod), td_mod_menu_new ());
}

void plugins_menu_refresh (GtkObject *mod, GtkObject *mod_net)
{
  td_database_set_current (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
}

void plugins_menu_action (GtkObject *mod, int menu_id)
{
  GList *row;
  TdModCanvas *mod_canvas;
  int id_sibling;
  int id_parent;
  mod_canvas = TD_MOD_CANVAS (g_list_nth_data (TD_MOD (mod)->child, 0));

  /* Action */
  switch (menu_id)
    {
    case 0:
      td_database_insert (g_strdup_printf ("INSERT INTO task (td_x, td_y) VALUES (%d, %d);", (int) TD_MOD_CANVAS (mod_canvas)->init_x, (int) TD_MOD_CANVAS (mod_canvas)->init_y), "task");
      row = td_database_row (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
      add_node (GTK_OBJECT (mod_canvas), row, 0);
      break;
      
    case 1:
      CURRENT_ID_PARENT = CURRENT_ID;
      td_database_insert (g_strdup_printf ("INSERT INTO task (td_x, td_y) VALUES (%d, %d);", (int) TD_MOD_CANVAS (mod_canvas)->init_x, (int) TD_MOD_CANVAS (mod_canvas)->init_y), "task");
      row = td_database_row (g_strdup_printf ("SELECT %s FROM task WHERE td_id = %d;", TD_MOD (mod)->fields, CURRENT_ID));
      add_node (GTK_OBJECT (mod_canvas), row, CURRENT_ID_PARENT);
      td_database_insert (g_strdup_printf ("INSERT INTO task_net (td_id, td_id_parent) VALUES (%d, %d);", CURRENT_ID, CURRENT_ID_PARENT), "task_net");
      break;

    case 3:
      td_database_command (g_strdup_printf ("DELETE FROM task_net WHERE td_id = %d;", CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task_net WHERE td_id_parent = %d;", CURRENT_ID));
      td_database_command (g_strdup_printf ("DELETE FROM task WHERE td_id = %d;", CURRENT_ID));
      remove_node (GTK_OBJECT (mod_canvas));
      CURRENT_ID = 0;
      CURRENT_ID_PARENT = 0;
      CURRENT_ROW = NULL;
      mod_canvas->selected = NULL;
      mod_canvas->selected_child = NULL;
      mod_canvas->selected_link_in = NULL;
      mod_canvas->selected_link_out = NULL;
      mod_canvas->selected_link_arrow_in = NULL;
      mod_canvas->selected_link_arrow_out = NULL;
      break;
    default:
      break;
    }
}

void plugins_menu_link_action (GtkObject *mod, int menu_id)
{
  TdModCanvas *mod_canvas;
  mod_canvas = TD_MOD_CANVAS (g_list_nth_data (TD_MOD (mod)->child, 0));
  td_database_command (g_strdup_printf ("DELETE FROM task_net WHERE td_id = %d AND td_id_parent = %d;", CURRENT_ID, CURRENT_ID_PARENT));
  remove_link (GTK_OBJECT (mod_canvas));
  mod_canvas->selected_link_in = NULL;
  mod_canvas->selected_link_out = NULL;
  mod_canvas->selected_link_arrow_in = NULL;
  mod_canvas->selected_link_arrow_out = NULL;
  CURRENT_ID = 0;
  CURRENT_ID_PARENT = 0;
}
