/*****************************************************************************/
/*** commons.h : Variables et fonctions communes
 *** commons.h : Commons variables and commons functions
 * 
 * 
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <gmodule.h>
#include <fcntl.h>
#include <dlfcn.h>
#include <dirent.h>
#include <sys/stat.h>
#include <toutdoux.h>

/*****************************************************************************/
/*** Initialisation */
/*****************************************************************************/

/**
 * td_init:
 * 
 * fr: Initialisation de la bibliothque
 *
 * en: Library initialisation
 **/

void td_init (void)
{
  /*** Rpertoires - Directory */
  TD_PACKAGE_VAR_DIR = g_strconcat (g_get_home_dir(), "/.", TD_PACKAGE_DIR, NULL);
  TD_PACKAGE_TMP_DIR = g_strconcat (g_get_home_dir(), "/.", TD_PACKAGE_DIR, "/tmp", NULL);

  /*** Greffons fondamentaux - Core plug-ins */
  PLUGINS_CORE_BOOKMARKS = dlopen (g_strdup_printf ("%s%s", TD_PACKAGE_PLUGINS_DIR, "libhelp_bookmarks.so"), RTLD_LAZY);

  /*** Drapeux internes - Internal flags */
  FLAG_ANTI_AUTOEXPAND = FALSE;

  /*** Messages de dmarage - Starting messages */
  START_MSG = NULL;
}

/*****************************************************************************/
/*** Ok */
/*****************************************************************************/

/**
 * td_set_ok:
 * @widget: event's widget
 * @dialog_result: dialog's flag
 * 
 * fr: Quitte la boucle Gtk+
 *
 * en: Quit the Gtk+ loop
 **/

void td_set_ok (GtkWidget *widget, gboolean *dialog_result)
{
  *dialog_result = TRUE;
  gtk_main_quit ();
}

/*****************************************************************************/
/*** Fonctions - Functions */
/*****************************************************************************/

/**
 * td_func_comp:
 * @a: left-hand member
 * @b: right-hand member
 * 
 * fr: Fonction : comparaison 
 *
 * en: Function : comparaison
 * 
 * Return value: result
 **/

gint td_func_comp (gconstpointer a, gconstpointer b)
{
  return (strcmp ( (gchar*) a, (gchar*) b));
}

/*****************************************************************************/
/*** Message */
/*****************************************************************************/

/**
 * td_message_create_window:
 * @message: pop window's message
 * 
 * fr: Cr et affiche une fentre  message
 *
 * en: Creates and display a window with message
 **/

void td_message_create_window (gchar *message)
{
  GtkWidget *window;
  GtkWidget *eventbox1;
  GtkWidget *hbox1;
  gchar *pixmap1_filename;
  GtkWidget *pixmap1;
  GtkWidget *label1;

  /*** Fentre - Window */
  printf ("%s\n", message);
  window = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (window), g_strconcat (TD_PACKAGE, " :", _("Message"), NULL));
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, FALSE);
  eventbox1 = gtk_event_box_new ();
  gtk_widget_show (eventbox1);
  gtk_container_add (GTK_CONTAINER (window), eventbox1);
  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (eventbox1), hbox1);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);

  /*** Image */
  pixmap1 = gtk_type_new (gnome_pixmap_get_type ());
  pixmap1_filename = gnome_pixmap_file ("gnome-error.png");
  if (pixmap1_filename)
    gnome_pixmap_load_file (GNOME_PIXMAP (pixmap1), pixmap1_filename);
  else
    printf (_("Couldn't find pixmap file: %s"), "gnome-error.png");
  g_free (pixmap1_filename);
  gtk_widget_show (pixmap1);
  gtk_box_pack_start (GTK_BOX (hbox1), pixmap1, FALSE, FALSE, 0);

  /*** Texte - Text */
  label1 = gtk_label_new (message);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 5);

  /*** Appels - Calls */
  gtk_widget_realize (window);
  gdk_window_set_cursor (window->window, gdk_cursor_new (GDK_ARROW));
  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),GTK_OBJECT (window));
  gtk_signal_connect_object (GTK_OBJECT (eventbox1), "button_press_event",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),GTK_OBJECT (window));
  gtk_widget_show (window);
}


/*****************************************************************************/
/*** Question */
/*****************************************************************************/

/**
 * td_question_create_window:
 * @question: question
 * @title: window's title
 * 
 * fr: Cr et affiche une fentre  question
 *
 * en: Creates and display window with question
 * 
 * Return value: 0 for yes, 1 for no and 2 for cancel
 **/

int td_question_create_window (gchar *question, gchar *title)
{
  int ret;
  gboolean oui = FALSE;
  gboolean non = FALSE;

  GtkWidget *window;
  GtkWidget *vbox1;
  GtkWidget *hbox1;
  gchar *pixmap1_filename;
  GtkWidget *pixmap1;
  GtkWidget *label1;
  GtkWidget *hseparator1;
  GtkWidget *hbox2;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;

  /*** Fentre - Window */
  window = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (window), title);
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window), vbox1);
  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  /*** Image */
  pixmap1 = gtk_type_new (gnome_pixmap_get_type ());
  pixmap1_filename = gnome_pixmap_file ("gnome-warning.png");
  if (pixmap1_filename)
    gnome_pixmap_load_file (GNOME_PIXMAP (pixmap1), pixmap1_filename);
  else
    printf (_("Couldn't find pixmap file: %s"), "gnome-warning.png");
  g_free (pixmap1_filename);
  gtk_widget_show (pixmap1);
  gtk_box_pack_start (GTK_BOX (hbox1), pixmap1, FALSE, FALSE, 0);

  /*** Texte - Text */
  label1 = gtk_label_new (question);
  gtk_widget_show (label1);
  gtk_box_pack_end (GTK_BOX (hbox1), label1, TRUE, TRUE, GNOME_PAD);
  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 0);

  /*** Boutons - Buttons */	
  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);
  button1 = gnome_stock_button (GNOME_STOCK_BUTTON_YES);
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox2), button1, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
  button2 = gnome_stock_button (GNOME_STOCK_BUTTON_NO);
  gtk_widget_show (button2);
  gtk_box_pack_start (GTK_BOX (hbox2), button2, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);
  button3 = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_show (button3);
  gtk_box_pack_start (GTK_BOX (hbox2), button3, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button3);

  /*** Appels - Calls */
  gtk_signal_connect (GTK_OBJECT (button1), "clicked", GTK_SIGNAL_FUNC (td_set_ok), &oui);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked", GTK_SIGNAL_FUNC (td_set_ok), &non);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked", GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_widget_show (GTK_WIDGET (window));
  gtk_grab_add (GTK_WIDGET (window));
  gtk_main ();
  if (oui)
    /*** oui - yes */
    ret = 0;
  else
    if (non)
      /*** non -no */
      ret = 1;
    else
      /*** annuler - cancel */
      ret = 2;
	
  gtk_widget_destroy (GTK_WIDGET (window));
  return ret;
}


/*****************************************************************************/
/*** Selecteur de fichier - File selector */
/*****************************************************************************/

/**
 * td_fileselector_create_window:
 * @title: window's title
 * @file: default file
 * @extension: extension of file
 * @show: TRUE for show window, FALSE for hide window
 * 
 * fr: Cr et affiche ou cache une fentre de selection de fichier
 *
 * en: Creates and display or hide window for file selection
 * 
 * Return value: file location
 **/

gchar *td_fileselector_create_window (gchar *title, gchar *file, gchar *extension, gboolean show)
{
  gchar *txt_tmp = NULL;
  gchar *file2;
  gchar *extension2;
  gboolean ok = FALSE;

  /*** Annuler - Cancel */
  if (!show)
    {
      if (WINDOW_FILE_SELECTION)
	gtk_widget_destroy (WINDOW_FILE_SELECTION);
      return NULL;
    }

  /*** Fentre - Window */
  WINDOW_FILE_SELECTION = gtk_file_selection_new (title);
  if (file)
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (WINDOW_FILE_SELECTION), file);
  
  /*** Appels - Calls */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (WINDOW_FILE_SELECTION)->ok_button), "clicked", GTK_SIGNAL_FUNC (td_set_ok), &ok);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (WINDOW_FILE_SELECTION)->cancel_button), "clicked",GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (WINDOW_FILE_SELECTION)), "delete_event", GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_widget_show (GTK_WIDGET (WINDOW_FILE_SELECTION));
  gtk_main ();
  
  /*** Slection - Selection */
  if (ok)
    {
      txt_tmp = g_strdup_printf ("%s", gtk_file_selection_get_filename (GTK_FILE_SELECTION (WINDOW_FILE_SELECTION)));
      if (!strcmp (txt_tmp, "(null)"))
	return NULL;

      /*** Dtection de l'extension du fichier - Detecting file extension */
      if (extension)
	{
	  file2 = g_strdup_printf ("%s", txt_tmp);
	  extension2 = strtok (file2,".");
	  extension2 = strtok (NULL,".");
	  if (!extension2)
	    txt_tmp = g_strdup_printf ("%s.%s", file2, extension);
	  else
	    txt_tmp = g_strdup_printf ("%s.%s", file2, extension2);
	}
      return (txt_tmp);
    }
  return NULL;
}


/*****************************************************************************/
/*** Extension */
/*****************************************************************************/

/**
 * td_system:
 * @command: shell command
 * 
 * fr: Execute une commande shell sans nouveau processus
 *
 * en: Execute shell command without new process
 * 
 * Return value: FALSE on error
 **/

gboolean td_system (gchar *command)
{
  int system_verif;
  gchar *txt_tampon;

  /*** Go ! */
  system_verif = system (command);
  if (system_verif != 0)
    {
      td_palette_message (_("Executing command"), g_strconcat (_("Executing command"), " '", command, "'", NULL), "failed");
      return FALSE;
    }
  td_palette_message (NULL, g_strconcat (_("Executing command"), " '", command, "'", NULL), "command");
  return TRUE;
}

/**
 * td_shell:
 * @command: shell command
 * 
 * fr: Execute une commande shell avec un nouveau processus
 *
 * en: Execute shell command with new process
 * 
 * Return value: FALSE on error
 **/

gboolean td_shell (gchar *command)
{
  int system_verif;
  gchar *txt_tampon;

  /*** Go ! */
  system_verif = gnome_execute_shell (NULL, command);
  if (system_verif == -1)
    {
      td_palette_message (_("Executing command"), g_strconcat (_("Executing command"), " '", command, "'", NULL), "failed");
      return FALSE;
    }
  td_palette_message (NULL, g_strconcat (_("Executing command"), " '", command, "'", NULL), "command");
  return TRUE;
}

/**
 * td_dlsym:
 * @module: module
 * @dir: directory of module (NULL => default directory (TD_PACKAGE_PLUGINS_DIR))
 * @file: file of module
 * @symbol: symbol called
 * 
 * fr: Retourne le symbole du greffon 
 *
 * en: Return the symbol's module
 * 
 * Return value: pointer
 **/

gpointer *td_dlsym (void *module, gchar *dir, gchar *file, gchar *symbol)
{
  gpointer *ret;

  /*** Go ! */
  if (!dir)
    dir = TD_PACKAGE_PLUGINS_DIR;
  file = g_strconcat (dir, file, NULL);
  ret = dlsym (module, symbol);
  if (dlerror() != NULL)
    {
      td_palette_message (_("Executing plug-in"), g_strconcat (_("Executing plug-in : file"), " '", file,"' : ", _("symbol"), " '", symbol, "' ", _("not find"), NULL), "failed");
      return NULL;
    }
  td_palette_message (NULL, g_strconcat (_("Executing plug-in : file"), " '", file,"' : ", _("symbol"), " '", symbol, "'", NULL), "plugins");
  return ret;
}

/**
 * td_dataplugins_filelist:
 * 
 * fr: Retourne la liste des fichiers des greffons de donnes
 *
 * en: Return file list of data plug-ins
 * 
 * Return value: list
 **/

GList *td_dataplugins_filelist (void)
{
  GtkObject *flower;
  GList *ret = NULL;
  int i;

  /*** Lecture du fichier - Reading file */
  flower = td_flower_new ();
  gtk_object_set (GTK_OBJECT (flower),
		  "space", "toutdoux-dataplugins/1.0.1",
		  "root", "dataplugins", 
		  "branch", "dataplugin", NULL);
  td_flower_add_branch_attribut (TD_FLOWER (flower), "name");
  td_flower_add_branch_attribut (TD_FLOWER (flower), "directory");
  td_flower_add_branch_attribut (TD_FLOWER (flower), "file");
  td_flower_load_file (TD_FLOWER (flower), g_strdup_printf("%s/share/%s/data-plugins.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR));
  for (i=0; i<g_list_length (TD_FLOWER (flower)->items_value); i++)
    ret = g_list_append (ret, g_strconcat (TD_PACKAGE_PLUGINS_DIR,  g_list_nth_data (g_list_nth_data (TD_FLOWER (flower)->items_value, i), 2)));
  return ret;
}

/**
 * td_dataplugins_list:
 * 
 * fr: Retourne la liste des descriptions des greffons de donnes
 *
 * en: Return description list of data plug-ins
 * 
 * Return value: description list
 **/

GtkObject *td_dataplugins_list (void)
{
  GtkObject *ret = NULL;
  int i;

  /*** Lecture du fichier - Reading file */
  ret = td_flower_new ();
  gtk_object_set (GTK_OBJECT (ret),
		  "space", "toutdoux-dataplugins/1.0.1",
		  "root", "dataplugins", 
		  "branch", "dataplugin", NULL);
  td_flower_add_branch_attribut (TD_FLOWER (ret), "name");
  td_flower_add_branch_attribut (TD_FLOWER (ret), "directory");
  td_flower_add_branch_attribut (TD_FLOWER (ret), "file");
  td_flower_load_file (TD_FLOWER (ret), g_strdup_printf("%s/share/%s/data-plugins.xml", TD_PACKAGE_DATA_DIR, TD_PACKAGE_DIR));
  return ret;
}

/*****************************************************************************/
/*** File - Fichier */
/*****************************************************************************/

/**
 * td_file_exist:
 * @file: file location
 * 
 * fr: Retourne TRUE si le fichier existe
 *
 * en: Return TRUE if file exist
 * 
 * Return value: boolean
 **/

gboolean td_file_exist (gchar *file)
{
  int fp = 0;
  fp = open (file, O_RDONLY);
  if (fp == -1)
    return FALSE;
  close (fp);
  return TRUE;
}

/*****************************************************************************/
/*** Date */
/*****************************************************************************/

/**
 * td_date_current:
 * 
 * fr: Retourne la date courante
 *
 * en: Return the current date
 * 
 * Return value: date
 **/

GDate *td_date_current (void)
{
  time_t curtime;
  struct tm *curtime2;
  curtime = time (NULL);
  curtime2 = gmtime (&curtime);
  return g_date_new_dmy (curtime2->tm_mday, curtime2->tm_mon+1, curtime2->tm_year+1900);
}

/**
 * td_date_parse:
 * @date: string
 * @format: format
 * 
 * format : day/month/year, year-month-day
 * 
 * fr: Retourne la date correspondant  la chane de caractres
 *
 * en: Return the date corresponding the string
 * 
 * Return value: date
 **/

GDate *td_date_parse (gchar *date, gchar *format)
{
  int day, month, year;
  if (!strcmp (format , "day/month/year"))
    sscanf(date, "%d/%d/%d", &day, &month, &year);
  if (!strcmp (format , "year-month-day"))
    sscanf(date, "%d-%d-%d", &year, &month, &day);
  if (!g_date_valid_dmy (day, month, year))
    return NULL;
  return g_date_new_dmy (day, month, year);
}

/**
 * td_date_print:
 * @date: date
 * @format: format
 * 
 * format : day, day_name, day_abbrev, day/month, day/month/year, month-day, year-month-day, 
 * week day/month/year, week year-month-day, 
 * day month_name, day month_name year, day month_abbrev, day month_abbrev year, 
 * day_name month_name, day_name month_name year, 
 * day_abbrev month_abbrev, day_abbrev month_abbrev year,
 * day_name day month_name, day_name day month_name year, day_abbrev day month_abbrev, 
 * day_abbrev day month_abbrev year, day_name month_name day, day_name month_name day year, day_abbrev month_abbrev day, day_abbrev month_abbrev day year
 * 
 * fr: Retourne la chane de caractres correspondant  la date
 *
 * en: Return the string corresponding the date
 * 
 * Return value: string
 **/

gchar *td_date_print (GDate *date, gchar *format)
{
  return td_date_day (g_date_day (date), g_date_month (date), g_date_year (date), format);
}

/**
 * td_date_month_name:
 * @month: month number
 * @abbrev: TRUE for abbreviation
 * 
 * fr: Retourne le nom du mois
 *
 * en: Return the month's name
 * 
 * Return value: string
 **/

gchar *td_date_month_name (int month, gboolean abbrev)
{
  if (abbrev)
    switch (month)
      {
      case 1:
	return _("jan");
      case 2:
	return _("feb");
      case 3:
	return _("mar");
      case 4:
	return _("apr");
      case 5:
	return _("may");
      case 6:
	return _("jun");
      case 7:
	return _("jul");
      case 8:
	return _("aug");
      case 9:
	return _("sep");
      case 10:
	return _("oct");
      case 11:
	return _("nov");
      case 12:
	return _("dec");
      default:
	break;
      }
  else
    switch (month)
      {
      case 1:
	return _("January");
      case 2:
	return _("February");
      case 3:
	return _("March");
      case 4:
	return _("April");
      case 5:
	return _("May");
      case 6:
	return _("June");
      case 7:
	return _("July");
      case 8:
	return _("August");
      case 9:
	return _("September");
      case 10:
	return _("October");
      case 11:
	return _("November");
      case 12:
	return _("December");
      default:
	break;
      }
  return NULL;
}

/**
 * td_date_month:
 * @month: month number
 * @year: year number
 * @format : format
 * 
 * format : month, month_name, month_abbrev, month/year, year-month, month_name year, month_abbrev year
 * 
 * fr: Retourne la chane de caractres formate de la date
 *
 * en: Return the formated string of the date
 * 
 * Return value: string
 **/

gchar *td_date_month (int month, int year, gchar *format)
{
  gchar *tampon;
  GDate* date_tmp;

  /*** Chiffre - Number */
  if (!strcmp (format , "month"))
    return (g_strdup_printf ("%d", month));

  /*** Nom - Name */
  if (!strcmp (format , "month_name"))
    return (td_date_month_name (month, FALSE));
  if (!strcmp (format , "month_abbrev"))
    return (td_date_month_name (month, TRUE));

  /*** Chiffres - Numbers */
  if (!strcmp (format , "month/year"))
    return (g_strdup_printf ("%02d/%d", month, year));
  if (!strcmp (format , "year-month"))
    return (g_strdup_printf ("%d-%02d", year,month));

  /*** Noms - Names */
  if (!strcmp (format , "month_name year"))
    return (g_strdup_printf ("%s %d", td_date_month_name (month, FALSE), year));
  if (!strcmp (format , "month_abbrev year"))
    return (g_strdup_printf ("%s %d", td_date_month_name (month, TRUE), year));
  return NULL;
}

/**
 * td_date_week:
 * @week: week number
 * @year: year number
 * @format : format
 * 
 * format : week, week/year, year-week
 * 
 * fr: Retourne la chane de caractres formate de la date
 *
 * en: Return the formated string of the date
 * 
 * Return value: string
 **/

gchar *td_date_week (int week, int year, gchar *format)
{
  if (!strcmp (format , "week"))
    return (g_strdup_printf ("%s%d", _("W"), week));
  if (!strcmp (format , "week/year"))
    return (g_strdup_printf ("%s%d/%d", _("W"), week, year));
  if (!strcmp (format , "year-week"))
    return (g_strdup_printf ("%d-%s%d", year, _("W"), week));
  return NULL;
}

/**
 * td_date_day_name:
 * @date: week number
 * @abbrev: TRUE for abbreviation
 * 
 * fr: Retourne le nom du jour
 *
 * en: Return the day's name
 * 
 * Return value: string
 **/

gchar *td_date_day_name (GDate *date, gboolean abbrev)
{
  GDateWeekday day_name;
  day_name = g_date_weekday (date);
  if (abbrev)
    switch (day_name)
      {
      case 1:
	return _("mon");
      case 2:
	return _("tue");
      case 3:
	return _("wed");
      case 4:
	return _("thu");
      case 5:
	return _("fri");
      case 6:
	return _("sat");
      case 7:
	return _("sun");
      default:
	break;
      }
  else
    switch (day_name)
      {
      case 1:
	return _("Monday");
      case 2:
	return _("Tuesday");
      case 3:
	return _("Wednesday");
      case 4:
	return _("Thursday");
      case 5:
	return _("Friday");
      case 6:
	return _("Saturday");
      case 7:
	return _("Sunday");
      default:
	break;
      }
  return NULL;
}

/**
 * td_date_day:
 * @day: day number
 * @month: month number
 * @year: year number
 * @format : format
 * 
 * format : day, day_name, day_abbrev, day/month, day/month/year, month-day, year-month-day, 
 * week day/month/year, week year-month-day, 
 * day month_name, day month_name year, day month_abbrev, day month_abbrev year, 
 * day_name month_name, day_name month_name year, 
 * day_abbrev month_abbrev, day_abbrev month_abbrev year,
 * day_name day month_name, day_name day month_name year, day_abbrev day month_abbrev, 
 * day_abbrev day month_abbrev year, day_name month_name day, day_name month_name day year, day_abbrev month_abbrev day, day_abbrev month_abbrev day year
 * 
 * fr: Retourne la chane de caractres formate de la date
 *
 * en: Return the formated string of the date
 * 
 * Return value: string
 **/

gchar *td_date_day (int day, int month, int year, gchar *format)
{
  GDate *date;

  /*** Chiffre - Number */
  if (!strcmp (format, "day"))
    return (g_strdup_printf ("%02d", day));

  /*** Nom - Name */
  date = g_date_new_dmy (day, month, year);
  if (!strcmp (format, "day_name"))
    return (td_date_day_name (date, FALSE));
  if (!strcmp (format, "day_abbrev"))
    return (td_date_day_name (date, TRUE));

  /*** Chiffres - Numbers */
  if (!strcmp (format, "day/month"))
    return (g_strdup_printf ("%02d/%02d", day, month));
  if (!strcmp (format, "day/month/year"))
    return (g_strdup_printf ("%02d/%02d/%d", day, month, year));
  if (!strcmp (format, "month-day"))
    return (g_strdup_printf ("%02d-%02d", month, day));
  if (!strcmp (format, "year-month-day"))
    return (g_strdup_printf ("%d-%02d-%02d", year, month, day));
  if (!strcmp (format, "week day/month/year"))
    return (g_strdup_printf ("%s%d %02d/%02d/%d", _("W"), g_date_monday_week_of_year (date), day, month, year));
  if (!strcmp (format, "week year-month-day"))
    return (g_strdup_printf ("%s%d %02d-%02d-%d", _("W"), g_date_monday_week_of_year (date), year, month, day));

  /*** Chiffre+nom - Numbers+name */
  if (!strcmp (format, "day month_name"))
    return (g_strdup_printf ("%d %s", day, td_date_month_name (month, FALSE)));
  if (!strcmp (format, "day month_name year"))
    return (g_strdup_printf ("%d %s %d", day, td_date_month_name (month, FALSE), year));
  if (!strcmp (format, "day month_abbrev"))
    return (g_strdup_printf ("%d %s", day, td_date_month_name (month, TRUE)));
  if (!strcmp (format, "day month_abbrev year"))
    return (g_strdup_printf ("%d %s %d", day, td_date_month_name (month, TRUE), year));

  /*** Noms - Names */
  if (!strcmp (format, "day_name month_name"))
    return (g_strdup_printf ("%s %s", td_date_day_name (date, FALSE), td_date_month_name (month, FALSE)));
  if (!strcmp (format, "day_name month_name year"))
    return (g_strdup_printf ("%s %s %d", td_date_day_name (date, FALSE), td_date_month_name (month, FALSE), year));
  if (!strcmp (format, "day_abbrev month_abbrev"))
    return (g_strdup_printf ("%s %s", td_date_day_name (date, TRUE), td_date_month_name (month, TRUE)));
  if (!strcmp (format, "day_abbrev month_abbrev year"))
    return (g_strdup_printf ("%s %s %d", td_date_day_name (date, TRUE), td_date_month_name (month, TRUE), year));

  /*** Chiffre+noms EU - Numbers+names EU */
  if (!strcmp (format, "day_name day month_name"))
    return (g_strdup_printf ("%s %d %s", td_date_day_name (date, FALSE), day, td_date_month_name (month, FALSE)));
  if (!strcmp (format, "day_name day month_name year"))
    return (g_strdup_printf ("%s %d %s %d", td_date_day_name (date, FALSE), day, td_date_month_name (month, FALSE), year));
  if (!strcmp (format, "day_abbrev day month_abbrev"))
    return (g_strdup_printf ("%s %d %s", td_date_day_name (date, TRUE), day, td_date_month_name (month, TRUE)));
  if (!strcmp (format, "day_abbrev day month_abbrev year"))
    return (g_strdup_printf ("%s %d %s %d", td_date_day_name (date, TRUE), day, td_date_month_name (month, TRUE), year));

  /*** Chiffre+noms US - Number+names US */
  if (!strcmp (format, "day_name month_name day"))
    return (g_strdup_printf ("%s %s %d", td_date_day_name (date, FALSE), td_date_month_name (month, FALSE), day));
  if (!strcmp (format, "day_name month_name day year"))
    return (g_strdup_printf ("%s %s %d %d", td_date_day_name (date, FALSE), td_date_month_name (month, FALSE), day, year));
  if (!strcmp (format, "day_abbrev month_abbrev day"))
    return (g_strdup_printf ("%s %s %d", td_date_day_name (date, TRUE), td_date_month_name (month, TRUE), day));
  if (!strcmp (format, "day_abbrev month_abbrev day year"))
    return (g_strdup_printf ("%s %s %d %d", td_date_day_name (date, TRUE), td_date_month_name (month, TRUE), day, year));
  return NULL;
}

/**
 * td_date_period_day:
 * @lower: begin date
 * @upper: end date 
 * 
 * fr: Retourne la distance en jours entre les deux dates
 *
 * en: Return the distance between dates
 * 
 * Return value: integer
 **/

int td_date_period_day (GDate *lower, GDate *upper)
{
  int day;
  int i, j;

  /*** Annes, mois et jours - Years, months and day */
  day = 0;
  for (i=g_date_year (lower); i<g_date_year (upper)+1; i++)
    {

      /*** Premire et dernire anne - First and last year */
      if ((i==g_date_year (lower)) && (i==g_date_year (upper)))
	for (j=g_date_month (lower); j<g_date_month (upper)+1; j++)
	  {
	    if ((j==g_date_month (lower)) && (j==g_date_month (upper))) /*** Premier et dernier mois - First and last month */
	      day = day + g_date_day (upper) - g_date_day (lower);
	    if ((j==g_date_month (lower)) && (j!=g_date_month (upper))) /*** Premier mois - First month */
	      day = day + g_date_days_in_month (j, i) - g_date_day (lower);
	    if ((j!=g_date_month (lower)) && (j==g_date_month (upper))) /*** Dernier mois - Last month */
	      day = day + g_date_day (upper);
	    if ((j!=g_date_month (lower)) && (j!=g_date_month (upper)))
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Premire anne - First year */
      if ((i==g_date_year (lower)) && (i!=g_date_year (upper)))
	for (j=g_date_month (lower); j<13; j++)
	  {
	    if (j==g_date_month (lower)) /*** Premier mois - First month */
	      day = day + g_date_days_in_month (j, i) - g_date_day (lower);
	    else
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Dernire anne - Last year */
      if ((i!=g_date_year (lower)) && (i==g_date_year (upper)))
	for (j=1; j<g_date_month (upper)+1; j++)
	  {
	    if (j==g_date_month (upper)) /*** Dernier mois - Last month */
	      day = day + g_date_day (upper);
	    else
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Anne intermdiaire - Intermediate year */
      if ((i!=g_date_year (lower)) && (i!=g_date_year (upper)))
	for (j=1; j<13; j++)
	  day = day + g_date_days_in_month (j, i);
    }
  return day;
}

/**
 * td_date_add_days:
 * @date: date
 * @days: days quantity for addition
 * 
 * fr: Retourne une nouvelle date resultant de l'addition
 *
 * en: Return a new date result the addition
 * 
 * Return value: date
 **/

GDate *td_date_add_days (GDate *date, int days)
{
  GDate *ret;
  ret = g_date_new_dmy (g_date_day (date), g_date_month (date), g_date_year (date));
  g_date_add_days (ret, days);
  return ret;
}

/**
 * td_date_add_months:
 * @date: date
 * @months: months quantity for addition
 * 
 * fr: Retourne une nouvelle date resultant de l'addition
 *
 * en: Return a new date result the addition
 * 
 * Return value: date
 **/

GDate *td_date_add_months (GDate *date, int months)
{
  GDate *ret;
  ret = g_date_new_dmy (g_date_day (date), g_date_month (date), g_date_year (date));
  g_date_add_months (ret, months);
  return ret;
}

/**
 * td_date_add_years:
 * @date: date
 * @years: months quantity for addition
 * 
 * fr: Retourne une nouvelle date resultant de l'addition
 *
 * en: Return a new date result the addition
 * 
 * Return value: date
 **/

GDate *td_date_add_years (GDate *date, int years)
{
  GDate *ret;
  ret = g_date_new_dmy (g_date_day (date), g_date_month (date), g_date_year (date));
  g_date_add_years (ret, years);
  return ret;
}

/*****************************************************************************/
/*** Flottant - Float */
/*****************************************************************************/

/**
 * td_float_round:
 * @number: float
 * 
 * fr: Retourne l'arrondi par excs du rel 
 *
 * en: Return the round up of float
 * 
 * Return value: integer
 **/

int td_float_round (float number)
{
  int ret;
  ret = number;
  if (number-ret > 0)
    ret++;
  return ret;
}
