/*****************************************************************************/
/*** database.h : Liens vers les greffons d'interface de base de donnes
 *** database.h : Links to plug-ins for database interface
 * 
 *
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <dlfcn.h>
#include <toutdoux.h>


/*****************************************************************************/
/*** Cration des tables - Table creating */
/*****************************************************************************/

/**
 * td_database_table_create:
 * @table: table
 * 
 * fr: Cr la table
 *
 * en: Creates table
 * 
 * Return value: FALSE on error
 **/

gboolean td_database_table_create (GtkObject *table)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_table_create");
  return symbol (table);
}

/**
 * td_database_base_create:
 * @base: base
 * 
 * fr: Cr les tables de la base
 *
 * en: Creates base's tables
 * 
 * Return value: FALSE on error
 **/

gboolean td_database_base_create (GtkObject *base)
{
  int i;
  for (i=0; i < g_list_length (TD_DB_BASE (base)->table); i++)
    if (!td_database_table_create (g_list_nth_data (TD_DB_BASE (TD_APP (APP)->base)->table, i)))
      return FALSE;
  return TRUE;
}

/**
 * td_database_table_exist:
 * @table: table
 * 
 * fr: Retourne TRUE si la table existe
 *
 * en: Return TRUE if table exist
 * 
 * Return value: boolean
 **/

gboolean td_database_table_exist (gchar *table)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_table_exist");
  return symbol (table);
}

/*****************************************************************************/
/*** Destruction des tables - Table destroying */
/*****************************************************************************/

/**
 * td_database_table_drop:
 * @table: table
 * 
 * fr: Dtruit la table
 *
 * en: Destroys table
 * 
 * Return value: FALSE on error
 **/

gboolean td_database_table_drop (GtkObject *table)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_table_drop");
  return symbol (table);
}

/**
 * td_database_base_drop:
 * 
 * fr: Dtruit les tables de la base
 *
 * en: Destroys base's tables
 * 
 * Return value: FALSE on error
 **/

gboolean td_database_base_drop (void)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_base_drop");
  return symbol ();
}

/*****************************************************************************/
/*** Remplissage de tables - Tables filling */
/*****************************************************************************/

/**
 * td_database_table_copy_in:
 * @datatable: datatable
 * 
 * fr: Peuple la base avec la table de donnes
 *
 * en: Populate the database with datatable
 * 
 * Return value: FALSE on error
 **/

gboolean td_database_table_copy_in (GtkObject *datatable)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_table_copy_in");
  return symbol (datatable);
}

/*****************************************************************************/
/*** Requete - Query */
/*****************************************************************************/

/**
 * td_database_command:
 * @query: query
 * 
 * fr: Execute la requte commande (sans rsultat)
 *
 * en: Execute the command (without result) query
 * 
 * Return value: FALSE on error
 **/

gboolean td_database_command (gchar* query)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_command");
  return symbol (query);
}

/*** FIXME: hors service - not works */
gboolean td_database_command_asyn (gchar* query)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_command_asyn");
  return symbol (query);
}

/**
 * td_database_insert:
 * @query: query
 * @table: table
 * 
 * fr: Execute la requte d'insertion
 *
 * en: Execute the insert query
 * 
 * Return value: FALSE on error
 **/

gboolean td_database_insert (gchar* query, gchar* table)
{
  gboolean (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_insert");
  return symbol (query, table);
}

/**
 * td_database_select:
 * @query: query
 * 
 * fr: Execute la requte de slection
 *
 * en: Execute the select query
 * 
 * Return value: array
 **/

GtkObject *td_database_select (gchar* query)
{
  GtkObject *(*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_select");
  return symbol (query);
}

/**
 * td_database_field:
 * @query: query
 * 
 * fr: Retourne la liste des champs de la requte de slection
 *
 * en: Return the field-list of the select query
 * 
 * Return value: list
 **/

GList *td_database_field (gchar* query)
{
  GList *(*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_field");
  return symbol (query);
}

/**
 * td_database_row:
 * @query: query
 * 
 * fr: Retourne la liste des valeur de la premire instance de la requte de slection
 *
 * en: Return the values list of the first row of the select query
 * 
 * Return value: list
 **/

GList *td_database_row (gchar* query)
{
  GList *(*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_row");
  return symbol (query);
}

/**
 * td_database_column:
 * @query: query
 * 
 * fr: Retourne la liste des valeur du premier champs de la requte de slection
 *
 * en: Return the values list of the first field of the select query
 * 
 * Return value: list
 **/

GList *td_database_column (gchar* query)
{
  GList *(*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_column");
  return symbol (query);
}

/**
 * td_database_value:
 * @query: query
 * 
 * fr: Retourne la premier valeur de la requte de slection
 *
 * en: Return the fisrt value of the select query
 * 
 * Return value: list
 **/

gchar *td_database_value (gchar* query)
{
  gchar *(*symbol)();

  /*** Macros SQL interne - Internal SQL macros */
  if (strstr (query, "TD_CURRENT"))
    return td_database_current (query);

  /*** Go ! */
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_value");
  return symbol (query);
}

/**
 * td_database_set_current:
 * @query: query
 * 
 * fr: Place dans le cache (TD_CURRENT) l'instance de la requte de slection
 *
 * en: Put in cache (TD_CURRENT) the row of the select query
 **/

void td_database_set_current (gchar* query)
{
  void (*symbol)();
  (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_set_current");
  symbol (query);
}

/**
 * td_database_current:
 * @query: query
 * 
 * fr: Retoune une valeur du cache (TD_CURRENT) avec une syntaxe SQL
 *
 * en: Return on value of cache (TD_CURRENT) with SQL syntax
 * 
 * Return value: string
 **/

gchar *td_database_current (gchar* query)
{
  int i, j;
  gchar *query2;
  gchar *field;
  gchar *where;
  gchar *membre1;
  gchar *membre2;

  /*** CURRENT = NULL */
  if ((!strcmp (query, "TD_CURRENT (*);")) ||
      ((!strcmp (query, "TD_CURRENT (null);")) && (!CURRENT_ROW)))
    {
      td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
      return "(ok)";
    }
  if ((!strcmp (query, "TD_CURRENT (null);")) && (CURRENT_ROW))
    {
      td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
      return NULL;
    }
  if (!CURRENT_ROW)
    {
      td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
      return NULL;
    }
    
  /*** Requete sans condition - Query without condition */
  if (!strstr (query, "WHERE"))
    {
      field = strpbrk (query, " ");
      field = g_strndup (field+1, strlen (field)-2);
      for (i=0; i< g_list_length (CURRENT_FIELD); i++)
	if (!strcmp (g_list_nth_data (CURRENT_FIELD, i), field))
	  {
	    td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
	    return g_list_nth_data (CURRENT_ROW, i);
	  }
    }

  /*** Requete avec une condition ngative - Query with negative condition */
  if (strstr (query, "WHERE NOT"))
    {
/*        printf ("***** debug: query: %s \n", query); */
      query2 = strpbrk (query, " ");
      query2 = g_strndup (query2+1, strlen (query2)-2);
/*        printf ("***** debug: query2: %s \n", query2); */
      field = g_strdup_printf ("%s", query2);
/*        printf ("***** debug: field1: %s \n", field); */
      where = strtok (field, "WHERE");
/*        printf ("***** debug: field2: %s \n", field); */
/*        printf ("***** debug: where1: %s \n", where); */
      where = strtok (NULL, "NOT");
/*        printf ("***** debug: field3: %s \n", field); */
/*        printf ("***** debug: where2: %s \n", where); */
      where = strtok (NULL, "NOT");
/*        printf ("***** debug: field3+: %s \n", field); */
/*        printf ("***** debug: where2+: %s \n", where); */
      field = g_strndup (field, strlen (field)-1);
      where = g_strndup (where+1, strlen (where)-1);
/*        printf ("***** debug: field4: %s \n", field); */
/*        printf ("***** debug: where3: %s \n", where); */
      membre1 = g_strdup_printf ("%s", where);
      membre2 = strtok (membre1, "=");
      membre2 = strtok (NULL, "=");
      membre1 = g_strndup (membre1+1, strlen (membre1)-2);
      membre2 = g_strndup (membre2+2, strlen (membre2)-4);
/*        printf ("***** debug: membre1: %s \n", membre1); */
/*        printf ("***** debug: membre2: %s \n\n", membre2); */
      for (i=0; i< g_list_length (CURRENT_FIELD); i++)
	if (!strcmp (g_list_nth_data (CURRENT_FIELD, i), field))
	  for (j=0; j< g_list_length (CURRENT_FIELD); j++)
	    if (!strcmp (g_list_nth_data (CURRENT_FIELD, j), membre1))
	      {
		if (strcmp (g_list_nth_data (CURRENT_ROW, j), membre2))
		  {
		    td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
		    return g_list_nth_data (CURRENT_ROW, i);
		  }
		else
		  {
		    td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
		    return NULL;
		  }
	      }
      td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
      return NULL;
    }

  /*** Requete avec une condition positive - Query with positive condition */
  if (strstr (query, "WHERE"))
    {
/*        printf ("***** debug: query: %s \n", query); */
      query2 = strpbrk (query, " ");
      query2 = g_strndup (query2+1, strlen (query2)-2);
/*        printf ("***** debug: query2: %s \n", query2); */
      field = g_strdup_printf ("%s", query2);
/*        printf ("***** debug: field1: %s \n", field); */
      where = strtok (field, "WHERE");
/*        printf ("***** debug: field2: %s \n", field); */
/*        printf ("***** debug: where1: %s \n", where); */
      where = strtok (NULL, "WHERE");
/*        printf ("***** debug: field3: %s \n", field); */
/*        printf ("***** debug: where2: %s \n", where); */
      field = g_strndup (field, strlen (field)-1);
      where = g_strndup (where+1, strlen (where)-1);
/*        printf ("***** debug: field4: %s \n", field); */
/*        printf ("***** debug: where3: %s \n", where); */
      membre1 = g_strdup_printf ("%s", where);
      membre2 = strtok (membre1, "=");
      membre2 = strtok (NULL, "=");
      membre1 = g_strndup (membre1+1, strlen (membre1)-2);
      membre2 = g_strndup (membre2+2, strlen (membre2)-4);
/*        printf ("***** debug: membre1: %s \n", membre1); */
/*        printf ("***** debug: membre2: %s \n\n", membre2); */
      for (i=0; i< g_list_length (CURRENT_FIELD); i++)
	if (!strcmp (g_list_nth_data (CURRENT_FIELD, i), field))
	  for (j=0; j< g_list_length (CURRENT_FIELD); j++)
	    if (!strcmp (g_list_nth_data (CURRENT_FIELD, j), membre1))
	      {
		if (!strcmp (g_list_nth_data (CURRENT_ROW, j), membre2))
		  {
		    td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
		    return g_list_nth_data (CURRENT_ROW, i);
		  }
		else
		  {
		    td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
		    return NULL;
		  }
	      }
      td_palette_message (NULL, g_strdup_printf ("%s '%s'", _("Simulating in local database td_query"), query), "sql");
      return NULL;
    }
  return NULL;
}

/*****************************************************************************/
/*** Chane de caractre - String */
/*****************************************************************************/

/**
 * td_database_strescape
 * @source: original string
 * 
 * fr: Ajoute un '\' par '\' ou ''' ou '"'
 *
 * en: Add a '\' per '\' or ''' or '"'
 * 
 * Return value: string adjusted
 **/

gchar *td_database_strescape (gchar *source)
{
  /*** THANKS: function recoded by
       File: email: patch for hOpla
       Author: Guy Zrcher <gzuercher@raptus.com> */

  gchar *srcpos = (guchar *) source;
  gchar *dest = g_malloc (strlen (source) * 4 + 1);
  gchar *dstpos = dest;
  
  while (*srcpos)
    {
      if ((*srcpos == '\'') || /*** guillemets simples - simple quotes */
	  (*srcpos == '\"') || /*** guillemets doubles - double quotes */
	  (*srcpos == '\\'))  /*** antislashes - backslashes */
	*dstpos++ = '\\';
      
      *dstpos++ = *srcpos;
      srcpos++;
    }
  *dstpos++ = 0;
  return dest;
}

/**
 * td_database_adaptvalue
 * @value: original string
 * @type: type
 * 
 * fr: Ajuste la valeur pour former un requte SQL valide
 *
 * en: Ajust the value for making a valid SQL query
 * 
 * Return value: string adjusted
 **/

gchar *td_database_adaptvalue (gchar *value, gchar *type)
{
  /*** THANKS: function recoded by
       File: email: patch for hOpla
       Author: Guy Zrcher <gzuercher@raptus.com> */
  int day, month, year;

  /*** boolean, intervale */
  if ((strstr (type, "bool")) ||
      (strstr (type, "interval")))
    {
      value = g_strdup_printf ("'%s'", value);
      return (value);
    }

  /*** date */
  if (strstr (type, "date"))
    {
      sscanf (value, "%d-%d-%d", &year, &month, &day);
      value = g_strdup_printf ("'%04d-%02d-%02d'", year, month, day);
      return (value);
    }

  /*** timestamp */
  if (strstr (type, "timestamp"))
    {
      if (strlen (value) == 0)
	value="null";
      else
	value = g_strdup_printf ("'%s'", value);
      return (value);
    }

  /*** char(n), varchar(n), text */
  if ((strstr (type, "char")) ||
      (strstr (type, "text")))
    {
      value = td_database_strescape (value);
      value = g_strdup_printf ("'%s'", value);
      return (value);
   }

  return (value);
} 







