/*****************************************************************************/
/*** gtdk.c : Functions complmentaires aux objects standards gtk
 *** gtdk.c : Additionals functions for standards objects
 *
 *
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gnome.h>

#include "commons.h"
#include "gtdk.h"

#include "td_app.h"
#include "td_app_palette.h"

/*****************************************************************************/
/*** Arbre  collones - Ctree */
/*****************************************************************************/

/**
 * gtdk_ctree_expand:
 * @widget: ctree widget
 * @node: node
 * @scrolledwindow: scrolledwindow of ctree
 * 
 * fr: Lors d'une expansion d'une node, la fentre se dplace afin de la voir compltement
 *
 * en: When node of ctree expanded, scrolledwindow moving for an optimal view
 * 
 * Return value: internal
 **/

gboolean gtdk_ctree_expand (GtkWidget *widget, GtkCTreeNode *node, GtkWidget *scrolledwindow)
{
  GtkCList *clist;
  GtkCTreeNode *node_selection;
  int i, row_debut, row_fin;
  GtkAdjustment *tampon_adjustement;

  /*** fr: FIXME: le dplacement de l'expansion est inhib si l'ascenseur vertical se met en route : un bug de GTK+ => bien les gars ! */
  /*** en: FIXME: the expand moving is destroyed if vscrollbar start : GTK+ Bug => great ! */

  /*** Flag interne - Internal flag */
  if (FLAG_ANTI_AUTOEXPAND)
    return TRUE;

  /*** Ligne - Line */
  clist = GTK_CLIST (widget);
  i = 0;
  node_selection = GTK_CTREE_NODE (clist->row_list);
  while (node_selection && node_selection != node)
    {
      node_selection = GTK_CTREE_NODE_NEXT (node_selection);
      i++;
    }
  row_debut = i;

  /*** Detection du dernier - Last detection */
  if (GTK_CTREE_ROW(node_selection)->children)
    {
      node_selection = GTK_CTREE_ROW(node_selection)->children;
      gtk_ctree_collapse (GTK_CTREE(widget), node_selection);
      i++;
      while (GTK_CTREE_ROW(node_selection)->sibling)
	{
	  node_selection = GTK_CTREE_ROW(node_selection)->sibling;
	  gtk_ctree_collapse (GTK_CTREE(widget), node_selection);
	  i++;
	}
    }
  row_fin = i;

  /*** Scroll */
  tampon_adjustement = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolledwindow));
  for (i=row_debut; i<row_fin+1; i++)
    if (((i+1)*((GTK_CLIST (widget)->row_height)+1))>(tampon_adjustement->value+tampon_adjustement->page_size))
      if (((row_debut)*((GTK_CLIST (widget)->row_height)+1))>(tampon_adjustement->value))
	{
	  /*** FIXME: bibliothques GTK+ pas claires - GTK+ libs not clear */
	  printf ("debug : 'gtk_scrolled_window_set_vadjustment' de 'gtdk_ctree_expand' activ \n");
	  gtk_adjustment_set_value (tampon_adjustement, (tampon_adjustement->value)+((GTK_CLIST(widget)->row_height)+1));
	  gtk_scrolled_window_set_vadjustment (GTK_SCROLLED_WINDOW (scrolledwindow), tampon_adjustement);
	}
  return TRUE;
}

/**
 * gtdk_ctree_clear:
 * @widget: ctree widget
 * 
 * fr: Razage du ctree
 *
 * en: Clear ctree
 **/

void gtdk_ctree_clear (GtkWidget *widget)
{
  GtkCTreeNode *ctree_tampon;
  GtkCTreeNode *ctree_tampon_suivant;
  ctree_tampon = gtk_ctree_node_nth (GTK_CTREE(widget), 0);
  while (ctree_tampon)
    {
      gtk_ctree_collapse (GTK_CTREE (widget), ctree_tampon);
      ctree_tampon_suivant =  GTK_CTREE_NODE_NEXT (ctree_tampon);
      gtk_ctree_remove_node (GTK_CTREE (widget), ctree_tampon);
      ctree_tampon = ctree_tampon_suivant;
    }
}

/**
 * gtdk_ctree_next:
 * @node: node
 * 
 * fr: Retourne la node suivante
 *
 * en: Return next node
 * 
 * Return value: node
 **/

/*** Suivant - Next */
GtkCTreeNode *gtdk_ctree_next (GtkCTreeNode *node)
{
  GtkCTreeNode *node_parent;
  if (GTK_CTREE_ROW (node)->children)
    return (GTK_CTREE_ROW (node)->children);
  if (GTK_CTREE_ROW (node)->sibling)
    return (GTK_CTREE_ROW (node)->sibling);
  do
    {
      node_parent = GTK_CTREE_ROW (node)->parent;
      if (!node_parent)
	node = NULL;
      else
	if (GTK_CTREE_ROW (node_parent)->sibling == NULL)
	  node = node_parent;
	else
	  node = GTK_CTREE_ROW (node_parent)->sibling;
    }
  while ((node == node_parent) && (node));
  return node;
}

/**
 * gtdk_ctree_optimal_column_width:
 * @widget: ctree widget
 * @column: column
 * 
 * fr: Retourne la largeur de colonne pour une vue maximal
 *
 * en: Return width of column for a maximal view of column
 * 
 * Return value: integer
 **/

int gtdk_ctree_optimal_column_width (GtkWidget *widget, int column)
{
  GtkCTreeNode *node_tampon;
  gchar *txt_tampon;
  GdkPixmap *pixmap_tampon;
  GdkBitmap *mask_tampon;
  guint8 spacing_tampon;
  int txt_long;
  int level_long;
  int ret = 0;
  GdkFont *font = NULL;
  font = widget->style->font;
  node_tampon = gtk_ctree_node_nth (GTK_CTREE (widget), 0);
  while (node_tampon)
    {
      gtk_ctree_node_get_pixtext (GTK_CTREE(widget), node_tampon, column, &txt_tampon, &spacing_tampon, &pixmap_tampon, &mask_tampon);
      txt_long = gdk_string_width (font, txt_tampon);
      level_long = (GTK_CTREE_ROW (node_tampon)->level*GTK_CTREE (widget)->tree_indent);
      if (txt_long+level_long > ret)
	ret = txt_long+level_long;
      node_tampon = gtdk_ctree_next (node_tampon);
    }
  return ret;
}

/**
 * gtdk_ctree_invert_expand_recursive:
 * @ctree: ctree widget
 * @node: node
 * 
 * fr: Expansion rcurcive des nodes parents
 *
 * en: Recursively expand parent of node
 **/

void gtdk_ctree_invert_expand_recursive (GtkCTree *ctree, GtkCTreeNode *node)
{
  GtkCTreeNode *node_parent;
  FLAG_ANTI_AUTOEXPAND = TRUE;
  gtk_ctree_expand (ctree, node);
  node_parent = GTK_CTREE_ROW (node)->parent;
  while (node_parent)
    {
      gtk_ctree_expand (ctree, node_parent);
      node_parent = GTK_CTREE_ROW (node_parent)->parent;
    }
  FLAG_ANTI_AUTOEXPAND = FALSE;
}

/**
 * gtdk_ctree_read_node:
 * @ctree: ctree widget
 * @node: node
 * 
 * fr: Retourne la liste des valeurs de la node
 *
 * en: Return the data list of the node
 * 
 * Return value: list
 **/

GList *gtdk_ctree_read_node (GtkCTree *ctree, GtkCTreeNode *node)
{
  GList *ret = NULL;
  gchar *value_tmp;
  guint8 spacing_tmp;
  GdkPixmap *pixmap_tmp;
  GdkBitmap *mask_tmp;
  int i, j;
  gtk_ctree_node_get_pixtext (ctree, node, 0, &value_tmp, &spacing_tmp, &pixmap_tmp, &mask_tmp);
  ret = g_list_append (ret, value_tmp);
  for (i=1; i < GTK_CLIST(ctree)->columns; i++)
    {
      gtk_ctree_node_get_text (ctree, node, i, &value_tmp);
      ret = g_list_append (ret, value_tmp);
    }
  return ret;
}

/*****************************************************************************/
/*** Arbre - Tree */
/*****************************************************************************/

/**
 * gtdk_tree_item_set_label:
 * @tree_item: tree_item
 * @data: label's text
 * 
 * fr: Cr une tiquette justifie  gauche pour un item d'arbre
 *
 * en: Creates label for an item tree
 **/

void gtdk_tree_item_set_label (GtkWidget *tree_item, gchar *data)
{
  GtkWidget *hbox;
  GtkWidget *label;
  GtkWidget *label_end;
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (tree_item), hbox);
  label = gtk_label_new (data);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_widget_show (label);
  gtk_label_set_text (GTK_LABEL (label), data);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 10);
  label_end = gtk_label_new ("");
  gtk_widget_show (label_end);
  gtk_box_pack_start (GTK_BOX (hbox), label_end, TRUE, TRUE, 0);
}

/*****************************************************************************/
/*** Liste  collones - Clist */
/*****************************************************************************/

/**
 * gtdk_clist_correction:
 * @widget: clist widget
 * @row: row
 * @bevent_y: y of click event
 * 
 * fr: Corrige le numro de ligne retourn par un 'gtk_clist_get_selection_info'
 *
 * en: Correct row number generated with 'gtk_clist_get_selection_info'
 * 
 * Return value: integer
 **/

int gtdk_clist_correction (GtkWidget *widget, int row, int bevent_y)
{
  if ((row == 0) && (GTK_CLIST(widget)->rows !=0))
    {
      if (bevent_y <= (GTK_CLIST(widget)->row_height+1))
	return 0;
      else
	return -1;
    }
  if ((row >= (GTK_CLIST(widget)->rows))||(row<0))
    return -1;
  return row;
}

/**
 * gtdk_clist_adjustheight:
 * @widget : clist widget
 * 
 * fr: Ajuste la hauteur d'un 'clist' pour une valeur minimal de 16
 *
 * en: Ajust height of clist with 16 for minimal size
 **/

void gtdk_clist_adjustheight (GtkWidget *widget)
{
  if ((GTK_CLIST (widget)->row_height) < 16)
    gtk_clist_set_row_height (GTK_CLIST (widget), 16);
}

/*****************************************************************************/
/*** Fentre - Window */
/*****************************************************************************/

/**
 * gtdk_window_load_conf:
 * @name : window's name
 * @window : window
 * 
 * fr: Redimensionne une fentre avec la configuration utilisateur
 *
 * en: Resize window with user configuration
 **/

void gtdk_window_load_conf (gchar *name, GtkWidget *window)
{
  gchar *txt_tmp;
  int xfen, yfen;

  /*** x */
  txt_tmp = g_strconcat ("geometry-x/", name, NULL);
  txt_tmp = g_strdup_printf ("%s", gnome_config_get_string (txt_tmp));
  if (!strcmp (txt_tmp, "(null)"))
    xfen = 600;
  else
    xfen = atoi (txt_tmp);

  /*** y */
  txt_tmp = g_strconcat ("geometry-y/", name, NULL);
  txt_tmp = g_strdup_printf ("%s", gnome_config_get_string (txt_tmp));
  if (!strcmp (txt_tmp, "(null)"))
    yfen = 400;
  else
    yfen = atoi (txt_tmp);

  /*** Redimensionne - Resize */
  gtk_widget_set_usize (window, xfen, yfen);
}

/**
 * gtdk_window_save_conf:
 * @name : window's name
 * @window : window
 * 
 * fr: Sauvegarde les dimensions de la fentre dans la configuration utilisateur
 *
 * en: Save size window with user configuration
 **/

void gtdk_window_save_conf (gchar *name, GtkWidget *window)
{
  int xfen, yfen;

  /*** Capture dimensions - Size capture */
  xfen = window->allocation.width;
  yfen = window->allocation.height;

  /*** Ecriture, si les dimmensions sont "correctes" - Write, if size are correct */
  if ((xfen > 2) && (yfen > 2))
    {
      gnome_config_set_string (g_strconcat ("geometry-x/", name, NULL), g_strdup_printf ("%d", xfen));
      gnome_config_set_string (g_strconcat ("geometry-y/", name, NULL), g_strdup_printf ("%d", yfen));
      gnome_config_sync ();
    }
}

/**
 * gtdk_window_save_defaultconf:
 * @name : window's name
 * @xwindow : x size
 * @ywindow : y size
 * 
 * fr: Sauvegarde size comme configuration utilisateur par dfault
 *
 * en: Save size like defaults values with user configuration
 **/

void gtdk_window_save_defaultconf (gchar *name, int xwindow, int ywindow)
{
  gchar *txt_tmp;

  /*** x */
  txt_tmp = g_strconcat ("geometry-x/", name, NULL);
  txt_tmp = g_strdup_printf ("%s", gnome_config_get_string (txt_tmp));
  if (!strcmp (txt_tmp, "(null)"))
    gnome_config_set_string (g_strconcat ("geometry-x/", name, NULL), g_strdup_printf ("%d", xwindow));

  /*** y */
  txt_tmp = g_strconcat ("geometry-y/", name, NULL);
  txt_tmp = g_strdup_printf ("%s", gnome_config_get_string (txt_tmp));
  if (!strcmp (txt_tmp, "(null)"))
    gnome_config_set_string (g_strconcat ("geometry-y/", name, NULL), g_strdup_printf ("%d", ywindow));
  gnome_config_sync ();
}

/*****************************************************************************/
/*** Panneau - Paned */
/*****************************************************************************/

/**
 * gtdk_paned_load_conf:
 * @name : paned's name
 * @paned : widget
 * 
 * fr: Redimensionne une sparation avec la configuration utilisateur
 *
 * en: Resize paned with user configuration
 **/

void gtdk_paned_load_conf (gchar *name, GtkWidget *paned)
{
  gchar *txt_tmp;
  int position;

  /*** Position */
  txt_tmp = g_strconcat ("geometry-paned/", name, NULL);
  txt_tmp = g_strdup_printf ("%s", gnome_config_get_string (txt_tmp));
  if (!strcmp (txt_tmp, "(null)"))
    position = 100;
  else
    position = atoi (txt_tmp);

  /*** Redimensionne - Resize */
  gtk_paned_set_position (GTK_PANED (paned), position);
}

/**
 * gtdk_paned_save_conf:
 * @name : paned's name
 * @paned : widget
 * 
 * fr: Sauvegarde les dimensions de la sparation dans la configuration utilisateur
 *
 * en: Save size paned with user configuration
 **/

void gtdk_paned_save_conf (gchar *name, GtkWidget *paned)
{
  int position;

  /*** Position */
  if (GTK_IS_HPANED (paned))
    position = GTK_PANED (paned)->child1->allocation.width;
  else
    position = GTK_PANED (paned)->child1->allocation.height;

  /*** Ecriture, si les dimmensions sont "correctes" - Write, if size are correct */
  if (position > 2)
    {
      gnome_config_set_string (g_strconcat ("geometry-paned/", name, NULL), g_strdup_printf ("%d", position));
      gnome_config_sync ();
    }
}

/**
 * gtdk_paned_save_defaultconf:
 * @name : paned's name
 * @position : position
 * 
 * fr: Sauvegarde la position comme configuration utilisateur par dfault
 *
 * en: Save position like default value with user configuration
 **/

void gtdk_paned_save_defaultconf (gchar *name, int position)
{
  gchar *txt_tmp;
  txt_tmp = g_strconcat ("geometry-paned/", name, NULL);
  txt_tmp = g_strdup_printf ("%s", gnome_config_get_string (txt_tmp));
  if (!strcmp (txt_tmp, "(null)"))
    gnome_config_set_string (g_strconcat ("geometry-paned/", name, NULL), g_strdup_printf ("%d", position));
  gnome_config_sync ();
}

/*****************************************************************************/
/*** Bote - Box */
/*****************************************************************************/

/**
 * gtdk_vbox_title:
 * @title : title
 * @vbox : vbox widget
 * 
 * fr: Cr et affiche un titre dans un 'vbox'
 *
 * en: Creates and display a title frame in 'vbox'
 **/

void gtdk_vbox_title (gchar *title, GtkWidget *vbox)
{
  GtkWidget *viewport;
  GtkWidget *label;
  viewport = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport);
  gtk_box_pack_start (GTK_BOX (vbox), viewport, FALSE, FALSE, 0);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_OUT);
  label = gtk_label_new (title);
  gtk_widget_show (label);
  gtk_container_add (GTK_CONTAINER (viewport), label);
}

/*****************************************************************************/
/*** Entre+selecteur - Entry+selector */
/*****************************************************************************/

/**
 * gtdk_entry_file_selector:
 * @title: file selector's title
 * @file: default file for file selector
 * @extension: extension of file
 * 
 * fr: Cr et affiche un 'entry' avec un bouton pour selecteur de fichier
 *
 * en: Creates and display a 'entry' with button for file selector
 * 
 * Return value: entry + button
 **/

GtkWidget *gtdk_entry_file_selector (gchar *title, gchar *file , gchar *extension)
{
  GList *data = NULL;
  GtkWidget *ret;
  GtkWidget *entry;
  GtkWidget *toggle_button;
  GtkWidget *arrow;
  if (!file)
    file = g_get_home_dir ();
  ret = gtk_hbox_new (FALSE, 0);
  entry = gtk_entry_new ();
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (ret), entry, TRUE, TRUE, 0);
  toggle_button = gtk_toggle_button_new();
  gtk_widget_show (toggle_button);
  gtk_widget_set_usize (toggle_button, 17, 16);
  arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
  gtk_widget_show (arrow);
  gtk_misc_set_padding (GTK_MISC (arrow), 1, 0);
  gtk_container_add (GTK_CONTAINER (toggle_button), arrow);
  gtk_box_pack_start (GTK_BOX (ret), toggle_button, FALSE, FALSE, 0);
  data = g_list_append (data, entry);
  data = g_list_append (data, title);
  data = g_list_append (data, file);
  if (extension)
    data = g_list_append (data, extension);
  else
    data = g_list_append (data, "(null)");
  gtk_signal_connect (GTK_OBJECT (toggle_button), "clicked", GTK_SIGNAL_FUNC (gtdk_entry_file_selector_clicked), data);
  return ret;
}

/**
 * gtdk_entry_file_selector_clicked:
 * @button: button clicked for file selector creation
 * @data: entry + file selector tile + default file location
 * 
 * fr: Cr l'interface du selecteur de fichier
 *
 * en: Creates the interface of file selector
 **/

void gtdk_entry_file_selector_clicked (GtkButton *button, GList *data)
{
  gchar *txt_tmp;
  if (GTK_TOGGLE_BUTTON (button)->active)
    {
      if (!strcmp ("(null)", g_list_nth_data (data, 3)))
	txt_tmp = td_fileselector_create_window ((gchar*) g_list_nth_data (data, 1), (gchar*) g_list_nth_data (data, 2), NULL, TRUE);
      else
	txt_tmp = td_fileselector_create_window ((gchar*) g_list_nth_data (data, 1), (gchar*) g_list_nth_data (data, 2), (gchar*) g_list_nth_data (data, 3), TRUE);
      if (txt_tmp)
	gtk_entry_set_text (GTK_ENTRY (g_list_nth_data (data, 0)), txt_tmp);
      gtk_button_clicked (GTK_BUTTON (button));
    }
  else
    txt_tmp = td_fileselector_create_window (NULL, NULL, NULL, FALSE);
}

/**
 * gtdk_entry_date_selector:
 * @current: current date
 * 
 * fr: Cr et affiche un 'entry' avec un bouton pour calendrier
 *
 * en: Creates and display a 'entry' with button for calendar
 * 
 * Return value: entry + button
 **/

GtkWidget *gtdk_entry_date_selector (gchar *current)
{
  GtkWidget *ret;
  GtkWidget *entry;
  GtkWidget *toggle_button;
  GtkWidget *arrow;
  ret = gtk_hbox_new (FALSE, 0);
  entry = gtk_entry_new ();
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (ret), entry, TRUE, TRUE, 0);
  toggle_button = gtk_toggle_button_new();
  gtk_widget_show (toggle_button);
  gtk_widget_set_usize (toggle_button, 17, 16);
  arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
  gtk_widget_show (arrow);
  gtk_misc_set_padding (GTK_MISC (arrow), 1, 0);
  gtk_container_add (GTK_CONTAINER (toggle_button), arrow);
  gtk_box_pack_start (GTK_BOX (ret), toggle_button, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (toggle_button), "clicked", GTK_SIGNAL_FUNC (gtdk_entry_date_selector_clicked), entry);
  return ret;
}

/**
 * gtdk_entry_date_selector_clicked:
 * @widget: button clicked for calendar creation
 * @entry: entry
 * 
 * fr: Cr l'interface du calendier
 *
 * en: Creates the interface of calendar
 **/

void gtdk_entry_date_selector_clicked (GtkWidget *widget, GtkWidget *entry)
{
  GList *data = NULL;
  GtkWidget *calendar;
  double x, y;
  GDate *date_tmp;
  if (WINDOW_DATE_SELECTION)
    {
      gtk_widget_destroy (WINDOW_DATE_SELECTION);
      WINDOW_DATE_SELECTION = NULL;
    }
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      WINDOW_DATE_SELECTION = gtk_window_new (GTK_WINDOW_POPUP);
      gtk_window_set_policy (GTK_WINDOW (WINDOW_DATE_SELECTION), FALSE, FALSE, TRUE);
      gtk_window_set_position (GTK_WINDOW (WINDOW_DATE_SELECTION), GTK_WIN_POS_MOUSE);
      calendar = gtk_calendar_new ();
      gtk_widget_show (calendar);
      gtk_container_add (GTK_CONTAINER (WINDOW_DATE_SELECTION), calendar);
      gtk_calendar_display_options (GTK_CALENDAR (calendar), GTK_CALENDAR_SHOW_HEADING | GTK_CALENDAR_SHOW_DAY_NAMES | GTK_CALENDAR_SHOW_WEEK_NUMBERS | GTK_CALENDAR_WEEK_START_MONDAY);
      gtdk_calendar_select_date (GTK_CALENDAR (calendar), td_date_parse (gtk_entry_get_text (GTK_ENTRY (entry)), "year-month-day"));
      gtk_widget_show (WINDOW_DATE_SELECTION);
      data = g_list_append (data, widget);
      data = g_list_append (data, entry);
      gtk_signal_connect (GTK_OBJECT (calendar), "day_selected", GTK_SIGNAL_FUNC (gtdk_entry_date_selector_day_selected), data);
    }
}

/**
 * gtdk_entry_date_selector_day_selected:
 * @calendar: calendar widget
 * @data: button + entry
 * 
 * fr: Evnement 'day_selected' du calendrier
 *
 * en: Event 'day_selected' of calendar
 **/

void gtdk_entry_date_selector_day_selected (GtkCalendar *calendar, GList *data)
{
  gchar *txt_tmp;
  gchar *txt_tmp2;
  txt_tmp = gtk_entry_get_text (GTK_ENTRY (g_list_nth_data (data, 1)));
  txt_tmp2 = td_date_print (gtdk_calendar_get_date (calendar), "year-month-day");
  if (strcmp (txt_tmp, txt_tmp2))
    gtk_entry_set_text (GTK_ENTRY (g_list_nth_data (data, 1)), txt_tmp2);
  else
    {
      gtk_button_clicked (GTK_BUTTON (g_list_nth_data (data, 0)));
      if (WINDOW_DATE_SELECTION)
	{
	  gtk_widget_destroy (WINDOW_DATE_SELECTION);
	  WINDOW_DATE_SELECTION = NULL;
	}
    }
}

/*****************************************************************************/
/*** Menu option - Option menu */
/*****************************************************************************/

/**
 * gtdk_option_menu_set_popdown_strings:
 * @widget: option menu
 * @data: list
 * @widget_dest: entry widget for showing item selected
 * 
 * fr: Place une liste dans un menu  options
 *
 * en: Put list in option menu
 **/

void gtdk_option_menu_set_popdown_strings (GtkWidget *widget, GList *data, GtkWidget *widget_dest)
{
  int i;
  GList *menuitem = NULL;
  GtkWidget *menu;
  GList *data_tmp;
  GList *data_tmp2 = NULL;
  for (i=0; i<g_list_length (data); i++)
    if (strlen (g_strstrip (g_list_nth_data (data, i))))
      data_tmp2= g_list_append (data_tmp2, g_list_nth_data (data, i));
  menu = gtk_menu_new ();
  for (i=0; i<g_list_length (data_tmp2); i++)
    {
      data_tmp = NULL;
      data_tmp = g_list_append (data_tmp, g_list_nth_data (data_tmp2, i));
      data_tmp = g_list_append (data_tmp, widget_dest);
      if (!strcmp ("(separator)", g_list_nth_data (data_tmp2, i)))
	menuitem = g_list_append (menuitem, gtk_menu_item_new ());
      else
	menuitem = g_list_append (menuitem, gtk_menu_item_new_with_label ((gchar*) g_list_nth_data (data_tmp2, i)));
      gtk_widget_show (GTK_WIDGET (g_list_nth_data (menuitem, i)));
      gtk_signal_connect (GTK_OBJECT (g_list_nth_data (menuitem, i)), "activate",
			  GTK_SIGNAL_FUNC (gtdk_option_menu_activate), data_tmp);
      gtk_menu_append (GTK_MENU (menu), GTK_WIDGET (g_list_nth_data (menuitem, i)));
    }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (widget), menu);
}

/**
 * gtdk_option_menu_activate:
 * @menuitem: item menu
 * @data: value + entry
 *
 * fr: Evnement 'activate' de l'item de menu
 *
 * en: Event 'activate' of item menu
 **/

void gtdk_option_menu_activate (GtkWidget *menuitem, GList *data)
{
  if (strcmp ("(separator)", g_list_nth_data (data, 0)))
    { 
      /*** gtk_entry */
      if (GTK_IS_ENTRY (g_list_nth_data (data, 1)))
	gtk_entry_set_text (GTK_ENTRY (g_list_nth_data (data, 1)), g_list_nth_data (data, 0));

      /*** gtk_combo */
      if (GTK_IS_COMBO (g_list_nth_data (data, 1)))
	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (g_list_nth_data (data, 1))->entry), g_list_nth_data (data, 0));

      /*** gtk_spin_button */
      if (GTK_IS_SPIN_BUTTON (g_list_nth_data (data, 1)))
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (g_list_nth_data (data, 1)), atoi (g_list_nth_data (data, 0)));

      /*** FIXME: la suite arrive => +tards - next arrives => this night */
    }
}

/**
 * gtdk_entry_option_menu_new:
 * 
 * fr: Cr et affiche une entre avec un menu  options
 *
 * en: Creates and display entry with option menu
 * 
 * Return value: entry + option menu
 **/

GtkWidget *gtdk_entry_option_menu_new (void)
{
  GtkWidget *ret;
  GtkWidget *entry;
  GtkWidget *option_menu;
  ret = gtk_hbox_new (FALSE, 0);
  entry = gtk_entry_new ();
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (ret), entry, TRUE, TRUE, 0);
  option_menu = gtk_option_menu_new ();
  gtk_widget_show (option_menu);
  gtk_box_pack_start (GTK_BOX (ret), option_menu, FALSE, FALSE, 0);
  return ret;
}

/**
 * gtdk_entry_option_menu_set_popdown_strings:
 * @widget: option menu
 * @data: list
 * 
 * fr: Place une liste dans une entre avec un menu  options
 *
 * en: Put list in entry with option menu
 **/

void gtdk_entry_option_menu_set_popdown_strings (GtkWidget *widget, GList *data)
{
  gtdk_option_menu_set_popdown_strings (g_list_nth_data (gtk_container_children (GTK_CONTAINER (widget)), 1), data, 
					g_list_nth_data (gtk_container_children (GTK_CONTAINER (widget)), 0));
}

/*****************************************************************************/
/*** Item de menu - Menu item */
/*****************************************************************************/

/**
 * gtdk_menu_item_new:
 * @text: item menu 's text
 * @dir: item menu 's directory of pixmap
 * @file: item menu 's file of pixmap
 * 
 * fr: Cr un item de menu incluant une image
 *
 * en: Creates menu item including pixmap
 * 
 * Return value: menu item
 **/

GtkWidget *gtdk_menu_item_new (gchar *text, gchar *dir, gchar *file)
{
  return (gtdk_menu_item_accel_new (text, dir, file, NULL, 0, 0));
}

/**
 * gtdk_menu_item_accel_new:
 * @text: text
 * @dir: directory of pixmap (NULL => default directory (TD_PACKAGE_DATA_DIR/share/pixmaps/TD_PACKAGE_DIR/))
 * @file: file of pixmap (NULL => empty pixmap (TD_PACKAGE_DATA_DIR/share/pixmaps/TD_PACKAGE_DIR/list_empty.xpm))
 * @accel_group: accelerator group
 * @accel_key: accelerator key
 * @accel_mods: accelerator modifier
 * 
 * fr: Cr un item de menu incluant une image et un accelerateur
 *
 * en: Creates menu item including pixmap and accelerator
 * 
 * Return value: menu item
 **/

GtkWidget *gtdk_menu_item_accel_new (gchar *text, gchar *dir, gchar *file, GtkAccelGroup *accel_group, guint accel_key, guint accel_mods)
{
  GtkWidget *ret;
  GtkWidget *label;
  GtkWidget *label_end;
  GtkWidget *label_accel;
  GtkWidget *img;
  GtkWidget *hbox;

  /*** Init */
  if (!text)
    {
      ret = gtk_menu_item_new ();
      gtk_widget_show (ret);
      return ret;
    }
  if (!dir)
    dir = g_strconcat (TD_PACKAGE_DATA_DIR, "/share/pixmaps/", TD_PACKAGE_DIR, "/", NULL);
  if (!file)
    file = "list_empty.xpm";
  file = g_strconcat (dir, file, NULL);

  /*** Bouton - Button */
  ret = gtk_menu_item_new ();
  gtk_widget_show (ret);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (ret), hbox);
  img = gnome_pixmap_new_from_file (file);
  gtk_widget_show (img);
  gtk_box_pack_start (GTK_BOX (hbox), img, FALSE, TRUE, 0);

  /*** Etiquette - Label */
  label = gtk_label_new (text);
  gtk_misc_set_padding (GTK_MISC (label), 7, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
  gtk_widget_show (label);
  label_end = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX (hbox), label_end, TRUE, TRUE, 0);
  gtk_widget_show (label_end);

  /*** Accelerateur - Accelerator */
  label_accel = gtk_label_new ("");
  gtk_misc_set_padding (GTK_MISC (label_accel), 10, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label_accel, FALSE, FALSE, 0);
  gtk_widget_show (label_accel);
  gtk_signal_connect (GTK_OBJECT (ret), "add_accelerator", GTK_SIGNAL_FUNC (gtdk_menu_item_add_accelerator), label_accel);
  gtk_signal_connect (GTK_OBJECT (ret), "remove_accelerator", GTK_SIGNAL_FUNC (gtdk_menu_item_remove_accelerator), label_accel);
  if (accel_group)
    gtk_widget_add_accelerator (ret, "activate", accel_group, accel_key, accel_mods, GTK_ACCEL_VISIBLE);
  return ret;
}

/**
 * gtdk_menu_item_add_accelerator:
 * @widget: item menu
 * @accel_signal_id: accelerator identifier
 * @accel_group: accelerator group
 * @accel_key: accelerator key
 * @accel_mods: acceleration modifier
 * @accel_flags: accelerator flags
 * @label_accel: label
 * 
 * fr: Ajoute l'accelerateur  l'item de menu
 *
 * en: Adds the accelerator to item menu
 **/

void gtdk_menu_item_add_accelerator (GtkWidget *widget, guint accel_signal_id, GtkAccelGroup *accel_group, guint accel_key, GdkModifierType accel_mods, GtkAccelFlags accel_flags, GtkWidget *label_accel)
{
  gchar* txt_tmp = "";
  int i = 0;

  /*** Nom du modificateur - Modifier's name */
  if (strstr (gtk_accelerator_name (accel_key, accel_mods), "<Shift>"))
    {
      txt_tmp = g_strconcat (txt_tmp, "Shft+", NULL);
      i = i + 7;
    }
  if (strstr (gtk_accelerator_name (accel_key, accel_mods), "<Control>"))
    {
      txt_tmp = g_strconcat (txt_tmp, "Ctl+", NULL);
      i = i + 9;
    }
  if (strstr (gtk_accelerator_name (accel_key, accel_mods), "<Alt>"))
    {
      txt_tmp = g_strconcat (txt_tmp, "Alt+", NULL);
      i = i + 5;
    }

  /*** Texte - Text */
  txt_tmp = g_strconcat (txt_tmp, gtk_accelerator_name (accel_key, accel_mods) + i, NULL);
  gtk_label_set_text (GTK_LABEL (label_accel), txt_tmp);
}

/**
 * gtdk_menu_item_remove_accelerator:
 * @widget: item menu
 * @accel_group: accelerator group
 * @accel_key: accelerator key
 * @accel_mods: acceleration modifier
 * @label_accel: label
 * 
 * fr: Supprimer l'accelerateur de l'item de menu
 *
 * en: Removes the accelerator of item menu
 **/

void gtdk_menu_item_remove_accelerator (GtkWidget *widget, GtkAccelGroup *accel_group, guint accel_key, GdkModifierType accel_mods, GtkWidget *label_accel)
{
  gtk_label_set_text (GTK_LABEL (label_accel), (""));
}

/**
 * gtdk_menu_item_set_submenu:
 * @menuitem: item menu
 * @menu: sub menu
 * 
 * fr: Cr un sous-menu 
 *
 * en: Creates a submenu
 **/

void gtdk_menu_item_set_submenu (GtkMenuItem *menuitem, GtkWidget *menu)
{
  GtkWidget *tearoff;
  gtk_menu_item_set_submenu (menuitem, menu);
  tearoff = gtk_tearoff_menu_item_new ();
  gtk_widget_show (tearoff);
  gtk_menu_append (GTK_MENU (menu), tearoff);
}

/*****************************************************************************/
/*** Types spcifiques de GDK/GTK+ - Specific types of GDK/GTK+ */
/*****************************************************************************/

/**
 * gtdk_anchor_parse:
 * @value: string
 * 
 * fr: Retourne la 'GtkAnchorType' correspondant  la chane de caractres
 *
 * en: Return the 'GtkAnchorType' corresponding the string
 * 
 * Return value: anchor
 **/

GtkAnchorType gtdk_anchor_parse (gchar *value)
{
  if (!strcmp (value, "center"))
    return GTK_ANCHOR_CENTER;
  if (strstr (value, "north"))
    {
      if (!strcmp (value, "north"))
	return GTK_ANCHOR_NORTH;
      if (!strcmp (value, "north west"))
	return GTK_ANCHOR_NORTH_WEST;
      if (!strcmp (value, "north east"))
	return GTK_ANCHOR_NORTH_EAST;
    }
  if (strstr (value, "south"))
    {
      if (!strcmp (value, "south"))
	return GTK_ANCHOR_SOUTH;
      if (!strcmp (value, "south west"))
	return GTK_ANCHOR_SOUTH_WEST;
      if (!strcmp (value, "south east"))
	return GTK_ANCHOR_SOUTH_EAST;
    }
  if (!strcmp (value, "west"))
    return GTK_ANCHOR_WEST;
  if (!strcmp (value, "east"))
    return GTK_ANCHOR_EAST;
  return GTK_ANCHOR_CENTER;
}

/**
 * gtdk_justification_parse:
 * @value: string
 * 
 * fr: Retourne la 'GtkJustification' correspondant  la chane de caractres
 *
 * en: Return the 'GtkJustification' corresponding the string
 * 
 * Return value: justification
 **/

GtkJustification gtdk_justification_parse (gchar *value)
{
  if (!strcmp (value, "left"))
    return GTK_JUSTIFY_LEFT;
  if (!strcmp (value, "right"))
    return GTK_JUSTIFY_RIGHT;
  if (!strcmp (value, "center"))
    return GTK_JUSTIFY_CENTER;
  if (!strcmp (value, "fill"))
    return GTK_JUSTIFY_FILL;
  return GTK_JUSTIFY_LEFT;
}

/**
 * gtdk_linestyle_parse:
 * @value: string
 * 
 * fr: Retourne la 'GdkLineStyle' correspondant  la chane de caractres
 *
 * en: Return the 'GdkLineStyle' corresponding the string
 * 
 * Return value: line style
 **/

GdkLineStyle gtdk_linestyle_parse (gchar *value)
{
  if (!strcmp (value, "solid"))
    return GDK_LINE_SOLID;
  if (!strcmp (value, "dash"))
    return GDK_LINE_ON_OFF_DASH;
  if (!strcmp (value, "dash double"))
    return GDK_LINE_DOUBLE_DASH;
  return GDK_LINE_SOLID;
}

/**
 * gtdk_cap_parse:
 * @value: string
 * 
 * fr: Retourne la 'GdkCapStyle' correspondant  la chane de caractres
 *
 * en: Return the 'GdkCapStyle' corresponding the string
 * 
 * Return value: cap style
 **/

GdkCapStyle gtdk_cap_parse (gchar *value)
{
  if (!strcmp (value, "butt"))
    return GDK_CAP_BUTT;
  if (!strcmp (value, "not last"))
    return GDK_CAP_NOT_LAST;
  if (!strcmp (value, "round"))
    return GDK_CAP_ROUND;
  if (!strcmp (value, "projecting"))
    return GDK_CAP_PROJECTING;
  return GDK_CAP_BUTT;
}

/**
 * gtdk_join_parse:
 * @value: string
 * 
 * fr: Retourne la 'GdkJoinStyle' correspondant  la chane de caractres
 *
 * en: Return the 'GdkJoinStyle' corresponding the string
 * 
 * Return value: join style
 **/

GdkJoinStyle gtdk_join_parse (gchar *value)
{
  if (!strcmp (value, "miter"))
    return GDK_JOIN_MITER;
  if (!strcmp (value, "round"))
    return GDK_JOIN_ROUND;
  if (!strcmp (value, "bevel"))
    return GDK_JOIN_BEVEL;
  return GDK_JOIN_MITER;
}

/**
 * gtdk_color_invert:
 * @color: color
 * 
 * fr: Retourne la couleur oppose
 *
 * en: Return the opposite color
 * 
 * Return value: color
 **/

GdkColor gtdk_color_invert (GdkColor color)
{
  color.red = 65535-color.red;
  color.green = 65535-color.green;
  color.blue = 65535-color.blue;
  return color;
}

/**
 * gtdk_color_parse:
 * @value: element[state]
 * 
 * fr: Retourne une couleur du thme courant
 *
 * en: Return a color of current theme
 * 
 * Return value: color
 **/

GdkColor gtdk_color_parse (gchar *value)
{
  GdkColor color;

  /*** Thme GTK+ - Theme GTK+ */
  if (strstr (value, "["))
    {

      /*** NORMAL */
      if (strstr (value, "normal"))
	{
	  if (!strcmp (value, "fg[normal]"))
	    return (TD_APP_PALETTE (TD_APP (APP)->palette)->window->style->fg[GTK_STATE_NORMAL]);
	  if (!strcmp (value, "bg[normal]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->bg[GTK_STATE_NORMAL]);
	  if (!strcmp (value, "base[normal]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->base[GTK_STATE_NORMAL]);
	  if (!strcmp (value, "text[normal]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->text[GTK_STATE_NORMAL]);
	  if (!strcmp (value, "light[normal]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->light[GTK_STATE_NORMAL]);
	  if (!strcmp (value, "dark[normal]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->dark[GTK_STATE_NORMAL]);
	  if (!strcmp (value, "mid[normal]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->mid[GTK_STATE_NORMAL]);
	}
      
      /*** INSENSITIVE */
      if (strstr (value, "insensitive"))
	{
	  if (!strcmp (value, "fg[insensitive]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->fg[GTK_STATE_INSENSITIVE]);
	  if (!strcmp (value, "bg[insensitive]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->bg[GTK_STATE_INSENSITIVE]);
	  if (!strcmp (value, "base[insensitive]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->base[GTK_STATE_INSENSITIVE]);
	  if (!strcmp (value, "text[insensitive]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->text[GTK_STATE_INSENSITIVE]);
	  if (!strcmp (value, "light[insensitive]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->light[GTK_STATE_INSENSITIVE]);
	  if (!strcmp (value, "dark[insensitive]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->dark[GTK_STATE_INSENSITIVE]);
	  if (!strcmp (value, "mid[insensitive]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->mid[GTK_STATE_INSENSITIVE]);
	}

      /*** PRELIGHT */
      if (strstr (value, "prelight"))
	{
	  if (!strcmp (value, "fg[prelight]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->fg[GTK_STATE_PRELIGHT]);
	  if (!strcmp (value, "bg[prelight]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->bg[GTK_STATE_PRELIGHT]);
	  if (!strcmp (value, "base[prelight]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->base[GTK_STATE_PRELIGHT]);
	  if (!strcmp (value, "text[prelight]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->text[GTK_STATE_PRELIGHT]);
	  if (!strcmp (value, "light[prelight]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->light[GTK_STATE_PRELIGHT]);
	  if (!strcmp (value, "dark[prelight]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->dark[GTK_STATE_PRELIGHT]);
	  if (!strcmp (value, "mid[prelight]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->mid[GTK_STATE_PRELIGHT]);
	}

      /*** ACTIVE */
      if (strstr (value, "active"))
	{
	  if (!strcmp (value, "fg[active]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->fg[GTK_STATE_ACTIVE]);
	  if (!strcmp (value, "bg[active]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->bg[GTK_STATE_ACTIVE]);
	  if (!strcmp (value, "base[active]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->base[GTK_STATE_ACTIVE]);
	  if (!strcmp (value, "text[active]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->text[GTK_STATE_ACTIVE]);
	  if (!strcmp (value, "light[active]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->light[GTK_STATE_ACTIVE]);
	  if (!strcmp (value, "dark[active]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->dark[GTK_STATE_ACTIVE]);
	  if (!strcmp (value, "mid[active]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->mid[GTK_STATE_ACTIVE]);
	}

      /*** SELECTED */
      if (strstr (value, "selected"))
	{
	  if (!strcmp (value, "fg[selected]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->fg[GTK_STATE_SELECTED]);
	  if (!strcmp (value, "bg[selected]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->bg[GTK_STATE_SELECTED]);
	  if (!strcmp (value, "base[selected]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->base[GTK_STATE_SELECTED]);
	  if (!strcmp (value, "text[selected]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->text[GTK_STATE_SELECTED]);
	  if (!strcmp (value, "light[selected]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->light[GTK_STATE_SELECTED]);
	  if (!strcmp (value, "dark[selected]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->dark[GTK_STATE_SELECTED]);
	  if (!strcmp (value, "mid[selected]"))
	    return (TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->mid[GTK_STATE_SELECTED]);
	}
    }
  gdk_color_parse (value, &color);
  return color;
}

/**
 * gtdk_cursor:
 * @cursor_source: xbm image
 * @cursor_mask: xbm image
 * 
 * fr: Retourne le curseur
 *
 * en: Return the cursor
 * 
 * Return value: cursor
 **/

GdkCursor *gtdk_cursor (const gchar *cursor_source, const gchar *cursor_mask)
{
  GdkCursor *cursor;
  GdkPixmap *cursor_source_pixmap, *cursor_mask_pixmap;
  GdkColor cursor_fg = { 0, 0, 0, 0 };
  GdkColor cursor_bg = { 0, 65535, 65535, 65535 };
  cursor_source_pixmap = gdk_bitmap_create_from_data (NULL, cursor_source, 32, 32);
  cursor_mask_pixmap = gdk_bitmap_create_from_data (NULL, cursor_mask,	32, 32);
  cursor = gdk_cursor_new_from_pixmap (cursor_source_pixmap, cursor_mask_pixmap, &cursor_fg, &cursor_bg, 3, 2);
  gdk_pixmap_unref (cursor_source_pixmap);
  gdk_pixmap_unref (cursor_mask_pixmap);
  return cursor;
}

/**
 * gtdk_widget_get_default_font:
 * 
 * fr: Retourne la fonte du thme courant
 *
 * en: Return the font of the current theme
 * 
 * Return value: font
 **/

GdkFont *gtdk_widget_get_default_font (void)
{
  return TD_APP_PALETTE (TD_APP(APP)->palette)->window->style->font;
}

/*****************************************************************************/
/*** Cavenas - Canvas */
/*****************************************************************************/

/**
 * gtdk_canvas_clear:
 * @canvas: canvas widget
 * 
 * fr: Razage du cavenas
 *
 * en: Clear canvas
 **/

void gtdk_canvas_clear (GnomeCanvas *canvas)
{
  GnomeCanvasGroup *root = gnome_canvas_root (canvas);
  GnomeCanvasItem *item;
  item = g_list_nth_data (root->item_list, 0);
  while (item)
    {
      gtk_object_destroy (GTK_OBJECT (item));
      item = g_list_nth_data (root->item_list, 0);
    }
}

/**
 * gtdk_canvas_link:
 * @item: canvas item
 * @line: canvas link
 * @arrow: canvas link's arrow
 * 
 * fr: Affiche un item lie
 *
 * en: Displays a linked item
 **/

void gtdk_canvas_link (GnomeCanvasItem *item, GnomeCanvasItem *line, GnomeCanvasItem *arrow)
{
  double x, y;
  double line_x1, line_y1, line_x2, line_y2, item_x1, item_y1, item_x2, item_y2, item_ratio, line_ratio;
  item_x1 = GNOME_CANVAS_RE (item)->x1;
  item_y1 = GNOME_CANVAS_RE (item)->y1;
  item_x2 = GNOME_CANVAS_RE (item)->x2;
  item_y2 = GNOME_CANVAS_RE (item)->y2;
  line_x1 = GNOME_CANVAS_LINE (line)->coords[0];
  line_y1 = GNOME_CANVAS_LINE (line)->coords[1];
  line_x2 = GNOME_CANVAS_LINE (line)->coords[2];
  line_y2 = GNOME_CANVAS_LINE (line)->coords[3];

  /*** Valeur - Value */
  if (!abs(item_x2-item_x1))
    item_ratio=0;
  else
    item_ratio = abs(item_x2-item_x1)/abs(item_y2-item_y1);
  if (!abs(line_y2-line_y1))
    line_ratio=0;
  else
    line_ratio = abs(line_x2-line_x1)/abs(line_y2-line_y1);
  if (line_ratio>=item_ratio)
    {
      if (line_x2>line_x1)
	{

	  /*** Gauche - Left */
	  x = item_x2;
	  y = 0;
	  gnome_canvas_item_i2w (item, &x, &y);
	  y = line_y1-(-item_x2*((line_y1-line_y2)/(line_x1-line_x2)));
	}
      else
	{

	  /*** Droit - Right */
	  x = item_x1;
	  y = 0;
	  gnome_canvas_item_i2w (item, &x, &y);
	  y = line_y1-(-item_x1*((line_y1-line_y2)/(line_x1-line_x2)));
	}
    }
  else
    {
      if (line_y2>line_y1)
	{

	  /*** Haut - Up */
	  x = 0;
	  y = item_y2;
	  gnome_canvas_item_i2w (item, &x, &y);
	  x = line_x1-(-item_y2*((line_x1-line_x2)/(line_y1-line_y2)));
	}
      else
	{

	  /*** Bas - Down */
	  x = 0;
	  y = item_y1;
	  gnome_canvas_item_i2w (item, &x, &y);
	  x = line_x1-(-item_y1*((line_x1-line_x2)/(line_y1-line_y2)));
	}
    }

  /*** Dessin - Drawing */
  gnome_canvas_item_set (arrow, 
			 "x1", (double) x-4, 
			 "y1", (double) y-4, 
			 "x2", (double) x+4, 
			 "y2", (double) y+4, NULL);
}

/**
 * gtdk_canvas_link_xy:
 * @item_x1w: x1 of canvas item
 * @item_y1w: y1 of canvas item
 * @item_x2w: x2 of canvas item
 * @item_y2w: y2 of canvas item
 * @line_x1: x1 of canvas link
 * @line_y1: y1 of canvas link
 * @line_x2: x2 of canvas link
 * @line_y2: y2 of canvas link
 * @arrow: canvas link
 * 
 * fr: Affiche un lien lie  partir de coordonnes
 *
 * en: Displays a linked item by coordinates
 **/

void gtdk_canvas_link_xy (double item_x1w, double item_y1w, double item_x2w, double item_y2w,
			  double line_x1, double line_y1, double line_x2, double line_y2,
			  GnomeCanvasItem *arrow)
{
  double item_x1, item_y1, item_x2, item_y2;
  double item_ratio, line_ratio, x, y;
  item_x1 = item_x1w-line_x1;
  item_y1 = item_y1w-line_y1;
  item_x2 = item_x2w-line_x1;
  item_y2 = item_y2w-line_y1;

  /*** Valeur - Value */
  if (!abs(item_x2-item_x1))
    item_ratio=0;
  else
    item_ratio = abs(item_x2-item_x1)/abs(item_y2-item_y1);
  if (!abs(line_y2-line_y1))
    line_ratio=0;
  else
    line_ratio = abs(line_x2-line_x1)/abs(line_y2-line_y1);
  if (line_ratio>=item_ratio)
    {
      if (line_x2>line_x1)
	{

	  /*** Gauche - Left */
	  x = item_x2w;
	  y = line_y1-(-item_x2*((line_y1-line_y2)/(line_x1-line_x2)));
	}
      else
	{

	  /*** Droit - Right */
	  x = item_x1w;
	  y = line_y1-(-item_x1*((line_y1-line_y2)/(line_x1-line_x2)));
	}
    }
  else
    {
      if (line_y2>line_y1)
	{

	  /*** Haut - Up */
	  x = line_x1-(-item_y2*((line_x1-line_x2)/(line_y1-line_y2)));
	  y = item_y2w;
	}
      else
	{

	  /*** Bas - Down */
	  x = line_x1-(-item_y1*((line_x1-line_x2)/(line_y1-line_y2)));
	  y = item_y1w;
	}
    }

  /*** Dessin - Drawing */
  gnome_canvas_item_set (arrow, 
			 "x1", (double) x-4, 
			 "y1", (double) y-4, 
			 "x2", (double) x+4, 
			 "y2", (double) y+4, NULL);
}

/**
 * gtdk_canvas_w2c_x:
 * @x: x coordinate
 * 
 * fr: converti des coordonnes globales en coordonnes du cavenas
 *
 * en: convert world coordinates into canvas pixel coordinates
 * 
 * Return value: x coordinate
 **/

double gtdk_canvas_w2c_x (double x)
{
  return x-4; /*** Bordure - Border */
}

/**
 * gtdk_canvas_w2i_x:
 * @item: canvas item
 * @x: x coordinate
 * 
 * fr: converti des coordonnes globales en coordonnes de l'item
 *
 * en: convert world coordinates into item pixel coordinates
 * 
 * Return value: x coordinate
 **/

double gtdk_canvas_w2i_x (GnomeCanvasItem *item, double x)
{
  double x1, y1;
  x1 = x;
  y1 = 0;
  gnome_canvas_item_w2i (item, &x1, &y1);
  return x1;
}

/**
 * gtdk_canvas_c2w_x:
 * @canvas: canvas
 * @x: x coordinate
 * 
 * fr: converti des coordonnes du cavenas en coordonnes globales
 *
 * en: convert canvas pixel coordinates into world coordinates
 * 
 * Return value: x coordinate
 **/

double gtdk_canvas_c2w_x (GnomeCanvas *canvas, double x)
{
  double x1, y1;
  gnome_canvas_c2w (canvas, x, 0, &x1, &y1);
  return x1;
}

/**
 * gtdk_canvas_i2w_x:
 * @item: item
 * @x: x coordinate
 * 
 * fr: converti des coordonnes de l'item en coordonnes globales
 *
 * en: convert item pixel coordinates into world coordinates
 * 
 * Return value: x coordinate
 **/

double gtdk_canvas_i2w_x (GnomeCanvasItem *item, double x)
{
  double x1, y1;
  x1 = x;
  y1 = 0;
  gnome_canvas_item_i2w (item, &x1, &y1);
  return x1+4; /*** Bordure - Border */
}

/*****************************************************************************/
/*** Calendrier - Calendar */
/*****************************************************************************/

/**
 * gtdk_calendar_select_date:
 * @calendar: calendar
 * @date: x date
 * 
 * fr: slection de la date dans le calendrier
 *
 * en: select the date in the calendar
 **/

void gtdk_calendar_select_date (GtkCalendar *calendar, GDate *date)
{
  gtk_calendar_select_month (calendar, g_date_month (date)-1, g_date_year (date));
  gtk_calendar_select_day (calendar, g_date_day (date));
}

/**
 * gtdk_calendar_get_date:
 * @calendar: calendar
 * 
 * fr: Retourne la date courant du calendrier
 *
 * en: Return the current date od calendar
 * 
 * Return value: date
 **/

GDate *gtdk_calendar_get_date (GtkCalendar *calendar)
{
  int year, month, day;
  gtk_calendar_get_date (calendar, &year, &month, &day);
  return (g_date_new_dmy (day, month+1, year));
}

/*****************************************************************************/
/*** Barre d'outils - Toolbar */
/*****************************************************************************/

/**
 * gtdk_toolbar_append_button:
 * @toolbar: toolbar
 * @text: label
 * @icon_file: file location of the icon
 * 
 * fr: Ajoute un boutton  la barre d'outils
 *
 * en: Add a button to the toolbar
 * 
 * Return value: button
 **/

GtkWidget *gtdk_toolbar_append_button (GtkToolbar *toolbar, gchar *text, gchar *icon_file)
{
  GtkWidget *ret;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  /* gdk_pixbuf_render_pixmap_and_mask (gdk_pixbuf_scale_simple (gdk_pixbuf_new_from_file (icon_file), 24, 24, GDK_INTERP_NEAREST), &pixmap, &mask, 127); */
  gdk_pixbuf_render_pixmap_and_mask (gdk_pixbuf_new_from_file (icon_file), &pixmap, &mask, 127);
  ret = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_BUTTON, NULL, text, text, NULL, gtk_pixmap_new (pixmap, mask), NULL, NULL);
  gtk_widget_show (ret);
  gdk_pixmap_unref (pixmap);
  gdk_bitmap_unref (mask);
  return ret;
}

/**
 * gtdk_toolbar_append_button_stock:
 * @toolbar: toolbar
 * @text: label
 * @window: window of toolbar
 * @icon: xpm image
 * 
 * fr: Ajoute un boutton  la barre d'outils
 *
 * en: Add a button to the toolbar
 * 
 * Return value: button
 **/

GtkWidget *gtdk_toolbar_append_button_stock (GtkToolbar *toolbar, gchar *text, GtkWidget *window, gchar *icon)
{
  GtkWidget *ret;
  ret = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_BUTTON, NULL, text, text, NULL, gnome_stock_pixmap_widget (window, icon), NULL, NULL);
  gtk_widget_show (ret);
  return ret;
}

/**
 * gtdk_toolbar_append_togglebutton:
 * @toolbar: toolbar
 * @text: label
 * @icon_file: file location of the icon
 * 
 * fr: Ajoute un boutton poussoir  la barre d'outils
 *
 * en: Add a togglebutton to the toolbar
 * 
 * Return value: button
 **/

GtkWidget *gtdk_toolbar_append_togglebutton (GtkToolbar *toolbar, gchar *text, gchar *icon_file)
{
  GtkWidget *ret;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  gdk_pixbuf_render_pixmap_and_mask (gdk_pixbuf_new_from_file (icon_file), &pixmap, &mask, 127);
  ret = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, text, text, NULL, gtk_pixmap_new (pixmap, mask), NULL, NULL);
  gtk_widget_show (ret);
  gdk_pixmap_unref (pixmap);
  gdk_bitmap_unref (mask);
  return ret;
}

/**
 * gtdk_toolbar_append_togglebutton_stock:
 * @toolbar: toolbar
 * @text: label
 * @window: window of toolbar
 * @icon: xpm image
 * 
 * fr: Ajoute un boutton poussoir  la barre d'outils
 *
 * en: Add a togglebutton to the toolbar
 * 
 * Return value: button
 **/

GtkWidget *gtdk_toolbar_append_togglebutton_stock (GtkToolbar *toolbar, gchar *text, GtkWidget *window, gchar *icon)
{
  GtkWidget *ret;
  ret = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, text, text, NULL, gnome_stock_pixmap_widget (window, icon), NULL, NULL);
  gtk_widget_show (ret);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (ret), FALSE);
  return ret;
}

/**
 * gtdk_toolbar_append_radiobutton:
 * @toolbar: toolbar
 * @sibling: radiobutton sibling
 * @text: label
 * @icon_file: file location of the icon
 * 
 * fr: Ajoute un boutton radio  la barre d'outils
 *
 * en: Add a radiobutton to the toolbar
 * 
 * Return value: button
 **/

GtkWidget *gtdk_toolbar_append_radiobutton (GtkToolbar *toolbar, GtkWidget *sibling, gchar *text, gchar *icon_file)
{
  GtkWidget *ret;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  gdk_pixbuf_render_pixmap_and_mask (gdk_pixbuf_new_from_file (icon_file), &pixmap, &mask, 127);
  ret = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_RADIOBUTTON, sibling, text, text, NULL, gtk_pixmap_new (pixmap, mask), NULL, NULL);
  gtk_widget_show (ret);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (ret), FALSE);
  gdk_pixmap_unref (pixmap);
  gdk_bitmap_unref (mask);
  return ret;
}

/**
 * gtdk_toolbar_append_radiobutton_stock:
 * @toolbar: toolbar
 * @sibling: radiobutton sibling
 * @text: label
 * @window: window of toolbar
 * @icon: xpm image
 * 
 * fr: Ajoute un boutton radio  la barre d'outils
 *
 * en: Add a radiobutton to the toolbar
 * 
 * Return value: button
 **/

GtkWidget *gtdk_toolbar_append_radiobutton_stock (GtkToolbar *toolbar, GtkWidget *sibling, gchar *text, GtkWidget *window, gchar *icon)
{
  GtkWidget *ret;
  ret = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_RADIOBUTTON, sibling, text, text, NULL, gnome_stock_pixmap_widget (window, icon), NULL, NULL);
  gtk_widget_show (ret);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (ret), FALSE);
  return ret;
}
