/*****************************************************************************/
/*** td_app.c : Objet Gtk+
 *** td_app.c : Gtk+ object
 *
 *
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <dlfcn.h>

#include "commons.h"
#include "gtdk.h"
#include "database.h"

#include "td_app.h"
#include "td_app_palette.h"
#include "td_app_editprop.h"

#include "td_mod.h"

#include "td_db_connect.h"

/*****************************************************************************/
/*** Arguments */
/*****************************************************************************/
static GtkObjectClass *parent_class = NULL;

enum {
  ARG_0,
  ARG_MODIFIED,
  ARG_MODE_BUILD,
  ARG_MODE_DEBUG,
  ARG_MODE_NOCOLOR,
};

static void td_app_set_modified (TdApp *app, gboolean modified)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->modified != modified)
    app->modified = modified;
}

static void td_app_set_mode_build (TdApp *app, gboolean mode_build)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->mode_build != mode_build)
    app->mode_build = mode_build;
}

static void td_app_set_mode_debug (TdApp *app, gboolean mode_debug)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->mode_debug != mode_debug)
    app->mode_debug = mode_debug;
}

static void td_app_set_mode_nocolor (TdApp *app, gboolean mode_nocolor)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->mode_nocolor != mode_nocolor)
    app->mode_nocolor = mode_nocolor;
}

static void td_app_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
{
  TdApp *app;
  app = TD_APP (object);
  switch (arg_id)
    {
    case ARG_MODIFIED:
      td_app_set_modified (app, GTK_VALUE_BOOL (*arg));
      break;
    case ARG_MODE_BUILD:
      td_app_set_mode_build (app, GTK_VALUE_BOOL (*arg));
      break;
    case ARG_MODE_DEBUG:
      td_app_set_mode_debug (app, GTK_VALUE_BOOL (*arg));
      break;
    case ARG_MODE_NOCOLOR:
      td_app_set_mode_nocolor (app, GTK_VALUE_BOOL (*arg));
      break;
    default:
      break;
    }
}

static void td_app_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
{
  TdApp *app;
  app = TD_APP (object);
   switch (arg_id)
    {
    case ARG_MODIFIED:
      GTK_VALUE_BOOL (*arg) = app->modified;
      break;
    case ARG_MODE_BUILD:
      GTK_VALUE_BOOL (*arg) = app->mode_build;
      break;
    case ARG_MODE_DEBUG:
      GTK_VALUE_BOOL (*arg) = app->mode_debug;
      break;
    case ARG_MODE_NOCOLOR:
      GTK_VALUE_BOOL (*arg) = app->mode_nocolor;
    default:
      arg->type = GTK_TYPE_INVALID;
      break;
    }
}

/*****************************************************************************/
/*** Initialisation */
/*****************************************************************************/
static void td_app_init (TdApp *app)
{
  app->palette = NULL;
  app->editprop = NULL;

  app->modified = FALSE;
  app->mode_build = FALSE;
  app->mode_debug = FALSE;
  app->mode_nocolor = FALSE;
  app->admin_field = NULL;

  app->accel_group = NULL;
  app->menu_main = NULL;
  app->menu_gnomeui = NULL;
  app->menu_palette = NULL;
  app->menu_favorites = NULL;
  app->menu_templates = NULL;
  app->menu_commands = NULL;
  app->menu_history = NULL;

  app->connect = NULL;
  app->base = NULL;
  app->about = NULL;
  app->presto_browser = NULL;
  app->custom_browser = NULL;
  app->mod_core = NULL;
  app->mod_data = NULL;
  app->menu_mod = NULL;
}

static void td_app_class_init (TdAppClass *klass)
{
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) klass;
  parent_class = gtk_type_class (gtk_object_get_type ());
  gtk_object_add_arg_type ("TdApp::modified", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_MODIFIED);
  gtk_object_add_arg_type ("TdApp::mode_build", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_MODE_BUILD);
  gtk_object_add_arg_type ("TdApp::mode_debug", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_MODE_DEBUG);
  gtk_object_add_arg_type ("TdApp::mode_nocolor", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_MODE_NOCOLOR);
  object_class->set_arg = td_app_set_arg;
  object_class->get_arg = td_app_get_arg;
  klass->set_palette = td_app_set_palette;
  klass->set_editprop = td_app_set_editprop;
  klass->add_admin_field = td_app_add_admin_field;
  klass->set_accel_group = td_app_set_accel_group;
  klass->set_menu_main = td_app_set_menu_main;
  klass->set_menu_palette = td_app_set_menu_palette;
  klass->set_menu_favorites = td_app_set_menu_favorites;
  klass->set_menu_templates = td_app_set_menu_templates;
  klass->set_menu_commands = td_app_set_menu_commands;
  klass->set_menu_history = td_app_set_menu_history;
  klass->set_current = td_app_set_current;
  klass->set_modified = td_app_set_modified;
  klass->set_about = td_app_set_about;
  klass->add_menu_mod = td_app_add_menu_mod;
  klass->create = td_app_create;
  klass->quit = td_app_quit;
}

GtkType td_app_get_type (void)
{
  static GtkType app_type = 0;
  if (!app_type)
    {
      static const GtkTypeInfo app_info =
      {
  	"TdApp", sizeof (TdApp), sizeof (TdAppClass),
  	(GtkClassInitFunc) td_app_class_init,
	(GtkObjectInitFunc) td_app_init,
	NULL, NULL, (GtkClassInitFunc) NULL,
      };
      app_type = gtk_type_unique (GTK_TYPE_OBJECT, &app_info);
    }
  return app_type;
}

/**
 * td_app_new:
 * 
 * fr: Cr une nouvelle application
 *
 * en: Creates a new application
 * 
 * Return value: application
 **/

GtkObject *td_app_new (void)
{
  return GTK_OBJECT (gtk_type_new (td_app_get_type ()));
}

/**
 * td_app_quit:
 * @app: application
 * 
 * fr: Dtruit l'application
 *
 * en: Destroys the application
 **/

void td_app_quit (TdApp *app)
{
  void (*symbol)() = NULL;
  int i;
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));

  /*** Fermeture du fichier en cours - Closing the current file */
  td_palette_message (NULL, _("Stopping"), "begin");
  td_palette_message (g_strdup_printf (("%s..."), _("Stopping")), NULL, "message");
  if (td_db_connect_close (NULL, NULL))
    {
      /*** Historique - History */
      (gpointer*) symbol = td_dlsym (PLUGINS_CORE_BOOKMARKS, NULL, "libbookmarks.so", "plugins_save_history");
      symbol ();

      /*** Modules noyaux - Core plug-ins */
      FLAG_NO_REFRESH_PALLETTE_MESSAGE = TRUE;
      for (i=0; i<g_list_length (app->mod_core); i++)
	{
	  symbol = dlsym (PLUGINS_CORE[i], "plugins_destroy");
	  if (dlerror() != NULL)
	    td_palette_message (_("Executing plug-in"), g_strdup_printf (_("Executing plug-in : file '%s' : function '%s' not find"), g_list_nth_data (app->mod_core, i), "plugins_destroy"), "failed");
	  else
	    {
	      symbol (g_list_nth_data (app->mod_core, i));
	      td_palette_message (NULL, g_strdup_printf (_("Executing plug-in : file '%s' : function '%s'"), g_list_nth_data (app->mod_core, i), "plugins_destroy"), "plugins");
	    }
	}
      (gpointer*) symbol = dlsym (PLUGINS_CORE_DATABASE, "plugins_destroy");
      symbol ();
      FLAG_NO_REFRESH_PALLETTE_MESSAGE = FALSE;
      td_palette_message (NULL, _("Stopping"), "end");

      /*** Fentres de base - Basis windows */
      td_app_palette_destroy (TD_APP_PALETTE (TD_APP (app)->palette));
      td_app_editprop_destroy (TD_APP_EDITPROP (TD_APP (app)->editprop));

      /*** Application */
      /* gtk_object_destroy (app->accel_group); */
      /* gtk_widget_destroy (app->menu_main); */
      /* gtk_widget_destroy (app->menu_palette); */
      /* gtk_widget_destroy (app->menu_favorites); */
      /* gtk_widget_destroy (app->menu_templates); */
      /* gtk_widget_destroy (app->menu_commands); */
      /* gtk_widget_destroy (app->menu_history); */
      g_list_free (app->admin_field);
      g_list_free (app->mod_core);
      g_list_free (app->mod_data);
      g_list_free (app->menu_mod);
      gtk_object_destroy (app->palette);
      gtk_object_destroy (app->editprop);
      gtk_object_destroy (app->about);
      if (app->presto_browser)
	gtk_object_destroy (app->presto_browser);
      if (app->custom_browser)
	gtk_object_destroy (app->custom_browser);
      gtk_object_destroy (GTK_OBJECT (app));
      gtk_exit (0);
    }
  td_palette_message (NULL, _("Stopping"), "end");
}

/*****************************************************************************/
/*** Affectations - Allocations */
/*****************************************************************************/

/**
 * td_app_set_palette:
 * @app: application
 * @palette: palette
 * 
 * fr: Affecte la palette  l'application
 *
 * en: Allocates the palette to application
 **/

void td_app_set_palette (TdApp *app, GtkObject *palette)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->palette != palette)
    app->palette = palette;
}

/**
 * td_app_set_editprop:
 * @app: application
 * @editprop: bench
 * 
 * fr: Affecte l'tabli  l'application
 *
 * en: Allocates the bench to application
 **/

void td_app_set_editprop (TdApp *app, GtkObject *editprop)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->editprop != editprop)
    app->editprop = editprop;
}

/**
 * td_app_add_admin_field:
 * @app: application
 * @field: field
 * 
 * fr: Ajoute un champs d'administration  l'application
 *
 * en: Add a admintration field to application
 **/

void td_app_add_admin_field (TdApp *app, GtkObject *field)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  app->admin_field = g_list_append (app->admin_field, field);
}

/**
 * td_app_set_accel_group:
 * @app: application
 * @accel_group: accelerators group
 * 
 * fr: Affecte le groupe d'acclrateurs  l'application
 *
 * en: Allocates the accelerators group to application
 **/

void td_app_set_accel_group (TdApp *app, GtkAccelGroup *accel_group)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->accel_group != accel_group)
    app->accel_group = accel_group;
}

/**
 * td_app_set_menu_main:
 * @app: application
 * @menu_gnomeui: menu
 * 
 * fr: Affecte le menu comme menu principal  l'application
 *
 * en: Allocates the menu like main menu to application
 **/

void td_app_set_menu_main (TdApp *app, GnomeUIInfo *menu_gnomeui)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->menu_gnomeui != menu_gnomeui)
    {
      app->menu_gnomeui = menu_gnomeui;
      app->menu_main = gnome_popup_menu_new (menu_gnomeui);
    }
}

/**
 * td_app_set_menu_palette:
 * @app: application
 * @menu_palette: menu
 * 
 * fr: Affecte le menu comme menu de palette  l'application
 *
 * en: Allocates the menu like palette menu to application
 **/

void td_app_set_menu_palette (TdApp *app, GtkWidget *menu_palette)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->menu_palette != menu_palette)
    app->menu_palette = menu_palette;
}

/**
 * td_app_set_menu_favorites:
 * @app: application
 * @menu_favorites: menu
 * 
 * fr: Affecte le menu comme menu des favoris  l'application
 *
 * en: Allocates the menu like favorites menu to application
 **/

void td_app_set_menu_favorites (TdApp *app, GtkWidget *menu_favorites)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->menu_favorites != menu_favorites)
    app->menu_favorites = menu_favorites;
}

/**
 * td_app_set_menu_templates:
 * @app: application
 * @menu_templates: menu
 * 
 * fr: Affecte le menu comme menu des gabarits  l'application
 *
 * en: Allocates the menu like templates menu to application
 **/

void td_app_set_menu_templates (TdApp *app, GtkWidget *menu_templates)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->menu_templates != menu_templates)
    app->menu_templates = menu_templates;
}

/**
 * td_app_set_menu_commands:
 * @app: application
 * @menu_commands: menu
 * 
 * fr: Affecte le menu comme menu des commandes  l'application
 *
 * en: Allocates the menu like commands menu to application
 **/

void td_app_set_menu_commands (TdApp *app, GtkWidget *menu_commands)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->menu_commands != menu_commands)
    app->menu_commands = menu_commands;
}

/**
 * td_app_set_menu_history:
 * @app: application
 * @menu_history: menu
 * 
 * fr: Affecte le menu comme menu de l'historique  l'application
 *
 * en: Allocates the menu like history menu to application
 **/

void td_app_set_menu_history (TdApp *app, GtkWidget *menu_history)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->menu_history != menu_history)
    app->menu_history = menu_history;
}

/**
 * td_app_set_current:
 * @app: application
 * @connect: connection
 * @base: base
 * 
 * fr: Affecte la connection et la base  l'application
 *
 * en: Allocates the connection and database to application
 **/

void td_app_set_current (TdApp *app, GtkObject *connect, GtkObject *base)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->connect != connect)
    app->connect = connect;
  if (app->base != base)
    app->base = base;
}

/**
 * td_app_set_about:
 * @app: application
 * @about: description
 * 
 * fr: Affecte la description  l'application
 *
 * en: Allocates the description to application
 **/

void td_app_set_about (TdApp *app, GtkObject *about)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (app->about != about)
    app->about = about;
}

/**
 * td_app_add_mod_core:
 * @app: application
 * @dir: directory location of plug-in
 * @file: file of plug-in
 * 
 * fr: Ajoute un module noyau  l'application
 *
 * en: Add a core module to application
 * 
 * Return value: FALSE on error
 **/

gboolean td_app_add_mod_core (TdApp *app, gchar *dir, gchar *file)
{
  int mod_core_id;
  gchar *mod_core;
  void (*symbol)();

  /*** Go ! */
  if (!dir)
    dir = TD_PACKAGE_PLUGINS_DIR;
  if (!app)
    app = TD_APP(APP);
  mod_core = g_strdup_printf ("%s%s", dir, file);
  mod_core_id = g_list_length (app->mod_core);

  /*** Chargement - Loading */
  app->mod_core = g_list_append (app->mod_core, mod_core);
  PLUGINS_CORE[mod_core_id] = dlopen (mod_core, RTLD_LAZY);
  if (dlerror() !=NULL)
    {
      td_palette_message (_("Loading plug-in"), g_strdup_printf (_("Loading plug-in : file '%s' unreadable"), mod_core), "failed");
      return FALSE;
    }

  /*** Initialisation */
  symbol = dlsym (PLUGINS_CORE[mod_core_id], "plugins_init");
  if (dlerror() != NULL)
    {
      td_palette_message (_("Executing plug-in"), g_strdup_printf (_("Executing plug-in : file '%s' : function 'plugins_init' not find"), mod_core), "failed");
      return FALSE;
    }
  td_palette_message (NULL, g_strdup_printf (_("Executing plug-in : file '%s' : function 'plugins_init'"), mod_core), "plugins");
  symbol ();
  return TRUE;
}

/**
 * td_app_add_mod_data:
 * @app: application
 * @dir: directory location of plug-in
 * @file: file of plug-in
 * 
 * fr: Ajoute un module de donne  l'application
 *
 * en: Add a data module to application
 * 
 * Return value: FALSE on error
 **/

gboolean td_app_add_mod_data (TdApp *app, gchar *dir, gchar *file)
{
  int mod_data_id;
  gchar *mod_data;
  GtkObject *(*symbol)();

  /*** Go ! */
  if (!dir)
    dir = TD_PACKAGE_PLUGINS_DIR;
  if (!app)
    app = TD_APP (APP);
  mod_data = g_strdup_printf ("%s%s", dir, file);
  mod_data_id = g_list_length (app->mod_data);

  /*** Chargement - Loading */
  PLUGINS_DATA[mod_data_id] = dlopen (mod_data, RTLD_LAZY);
  if (dlerror() !=NULL)
    {
      td_palette_message (_("Loading plug-in"), g_strdup_printf (_("Loading plug-in : file '%s' unreadable"), mod_data), "failed");
      return FALSE;
    }

  /*** Initialisation */
  symbol = dlsym (PLUGINS_DATA[mod_data_id], "plugins_init");
  if (dlerror() != NULL)
    {
      td_palette_message (_("Executing plug-in"), g_strdup_printf (_("Executing plug-in : file '%s' : function 'plugins_init' not find"), mod_data), "failed");
      return FALSE;
    }
  td_palette_message (NULL, g_strdup_printf (_("Executing plug-in : file '%s' : function 'plugins_init'"), mod_data), "plugins");
  app->mod_data = g_list_append (app->mod_data, symbol (mod_data_id, NULL));
  return TRUE;
}

/**
 * td_app_add_menu_mod:
 * @app: application
 * @menu_mod: module menu
 * 
 * fr: Ajoute un menu  module  l'application
 *
 * en: Add a module menu to application
 **/

void td_app_add_menu_mod (TdApp *app, GtkObject *menu_mod)
{
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  app->menu_mod = g_list_append (app->menu_mod, menu_mod);
}

/*****************************************************************************/
/*** Commandes - Commands */
/*****************************************************************************/

/**
 * td_app_create:
 * @app: td_app object
 * 
 * fr: Cr l'interface de l'application
 *
 * en: Creates the interface of application
 **/

void td_app_create (TdApp *app)
{
  GtkObject *connect;
  void *module_tmp;
  void (*symbol_tmp)();;
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));

  /*** Connexion - Connection */
  connect = td_db_connect_new();
  gtk_object_set (GTK_OBJECT (connect),
		  "name", _("New"),
		  "type", "empty", NULL);
  td_app_set_current (TD_APP (app), connect, NULL);

  /*** Fentres de base - Basis windows */
  td_app_palette_create (TD_APP_PALETTE (TD_APP (app)->palette));
  td_app_editprop_create (TD_APP_EDITPROP (TD_APP (app)->editprop));
}

/**
 * td_app_custom_show:
 * @app: td_app object
 * 
 * fr: Cr et affiche l'interface de personnalisation de l'application
 *
 * en: Creates and display customize interface of application
 **/

void td_app_custom_show (TdApp *app)
{
  void (*symbol)();
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (!app->custom_browser)
    {
      app->custom_browser = td_mod_new ();
      gtk_object_set (GTK_OBJECT (app->custom_browser), "name", "app", "name_intl", "(null)", NULL);
      (gpointer*) symbol = td_app_mod_core_symbol (NULL, NULL, "libhelp_customize.so", "plugins_create");
      symbol (app->custom_browser);
    }
  td_mod_show (TD_MOD (TD_MOD (app->custom_browser)->custom_browser));
}

/**
 * td_app_presto_show:
 * @app: td_app object
 * 
 * fr: Cr et affiche l'interface presto de l'application
 *
 * en: Creates and display presto interface of application
 **/

void td_app_presto_show (TdApp *app)
{
  void (*symbol)();
  g_return_if_fail (app != NULL);
  g_return_if_fail (TD_IS_APP (app));
  if (!app->presto_browser)
    {
      app->presto_browser = td_mod_new ();
      gtk_object_set (GTK_OBJECT (app->presto_browser), "name", "app", "name_intl", "(null)", NULL);
      (gpointer*) symbol = td_app_mod_core_symbol (NULL, NULL, "libhelp_doc.so", "plugins_create");
      symbol (app->presto_browser);
    }
  td_mod_show (TD_MOD (TD_MOD (app->presto_browser)->presto_browser));
}

/*** FIXME: hors service - not works */
gboolean td_palette_shell_activity (gchar *command, gchar *message)
{
/*    int pid; */
/*    gchar *txt_tampon; */

  /*** Go ! */ 
/*    pid = gnome_execute_shell (NULL, command); */
/*    if (pid == -1) */
/*      { */
/*        td_palette_message (_("Executing command"), g_strconcat (_("Executing command"), " '", command, "'", NULL), "failed"); */
/*        return FALSE; */
/*      } */
/*    while  */
/*    td_palette_message (NULL, g_strconcat (_("Executing command"), " '", command, "'", NULL), "command"); */
  return TRUE;
}

/**
 * td_palette_message:
 * @chtit_message: short text for palette's statusbar
 * @message: complete text
 * @type: type of message 
 * 
 * type: done, tested, plugins, command, failed, stop, begin, end, message, yes, no
 * 
 * fr: Affiche un message sur la palette et dans la fentre des messages
 *
 * en: Displays message for palette's statusbar and message window
 * 
 * Return value: TRUE
 **/

gboolean td_palette_message (gchar *chtit_message, gchar *message, gchar *type)
{
  /*** FIXME: changer en td_message => +tards - Changing to td_message => this night */
  void (*symbol)();
  GtkCTreeNode *node;
  GtkCTreeNode *node_parent;
  gchar *txt_tampon = NULL;
  gchar *txt[2];
  GdkColor transparent;
  gchar *type_lang = NULL;
  gchar *type_color = NULL;

  /*** Shunt */
  if ((APP) && (!TD_APP (APP)->mode_debug) && ((!strcmp (type, "plugins")) || (!strcmp (type, "sql"))))
    return TRUE;

  /*** Adaptation */
  if (!message)
    message = g_strdup_printf ("%s", chtit_message);

  /*** Ecriture dans la palette - Writting in the palette */
  if ((!FLAG_NO_REFRESH_PALLETTE_MESSAGE) && (APP))
    if (TD_APP_PALETTE (TD_APP (APP)->palette)->status)
      {
	if (!strcmp (type, "end"))
	  gtk_entry_set_text (GTK_ENTRY (TD_APP_PALETTE (TD_APP (APP)->palette)->status), " ");
	if (chtit_message)
	  {
	    if (!strcmp (type, "message"))
	      txt_tampon = g_strdup_printf ("%s", chtit_message);
	    else
	      txt_tampon = g_strdup_printf ("%s : %s", chtit_message, type);
	    gtk_entry_set_text (GTK_ENTRY (TD_APP_PALETTE (TD_APP (APP)->palette)->status), txt_tampon);
	  }
      }

  /*** Console */
  if (!strcmp (type, "done"))
    {
      type_lang = g_strdup_printf ("%s", _("done"));
      type_color = "\033[32m\033[1m";
    }
  if (!strcmp (type, "tested"))
    {
      type_lang = g_strdup_printf ("%s", _("tested"));
      type_color = "\033[32m\033[1m";
    }
  if (!strcmp (type, "plugins"))
    {
      type_lang = g_strdup_printf ("%s", _("plugins"));
      type_color = "\033[32m\033[1m";
    }
  if (!strcmp (type, "command"))
    {
      type_lang = g_strdup_printf ("%s", _("command"));
      type_color = "\033[32m\033[1m";
    }
  if (!strcmp (type, "sql"))
    {
      type_lang = g_strdup_printf ("%s", _("sql"));
      type_color = "\033[32m\033[1m";
    }

  if (!strcmp (type, "failed"))
    {
      type_lang = g_strdup_printf ("%s", _("failed"));
      type_color = "\033[31m\033[1m";
    }
  if (!strcmp (type, "stop"))
    {
      type_lang = g_strdup_printf ("%s", _("stop"));
      type_color = "\033[31m\033[1m";
    }
  if (!strcmp (type, "begin"))
    {
      type_lang = g_strdup_printf ("%s", _("begin"));
      type_color = "\033[0m";
    }
  if (!strcmp (type, "end"))
    {
      type_lang = g_strdup_printf ("%s", _("end"));
      type_color = "\033[0m";
    }
  if (!strcmp (type, "message"))
    {
      type_lang = g_strdup_printf ("%s", _("message"));
      type_color = "\033[0m";
    }
  if (!strcmp (type, "yes"))
    {
      type_lang = g_strdup_printf ("%s", _("yes"));
      type_color = "\033[36m\033[1m";
    }
  if (!strcmp (type, "no"))
    {
      type_lang = g_strdup_printf ("%s", _("no"));
      type_color = "\033[34m\033[1m";
    }
  if (!strcmp (type, "(null)"))
    {
      type_lang = ("(null)"); 
      type_color = "\033[0m";
    }
  if (APP)
    {
      if (!TD_APP (APP)->mode_nocolor)
	printf ("%s%s : %s\033[0m\n", type_color, message, type_lang);
      else
	printf ("%s : %s\n", message, type_lang);
    }
  else
    printf ("%s : %s\n", message, type_lang);

  /*** Arbre - Tree */
  if (PLUGINS_CORE[0])
    {
      (gpointer*) symbol = dlsym (PLUGINS_CORE[0], "plugins_new_message");
      symbol (message, type);
    }
  else
    {
      START_MSG = g_list_append (START_MSG, message);
      START_MSG = g_list_append (START_MSG, type);
    }

  /*** Bouton d'arrt d'urgence - Big red button */
  if (type)
    if (!strcmp (type, "stop"))
      gtk_exit (1);

  /*** Rafrachissement GTK+ - Refreshing GTK+ */
  while (gtk_events_pending())
    gtk_main_iteration();
  return TRUE;
}

/**
 * td_app_mod_core_symbol:
 * @app: td_app object (NULL => default app (APP))
 * @dir: directory location of plug-in (NULL=> default directory (TD_PACKAGE_PLUGINS_DIR))
 * @file: file location of plug-in
 * @symbol: symbol called
 * 
 * fr: Retourne le pointer du symbole du greffon charg
 *
 * en: Return pointer of symbol's plug-in loaded
 * 
 * Return value: pointer
 **/

gpointer *td_app_mod_core_symbol (TdApp *app, gchar *dir, gchar *file, gchar *symbol)
{
  gpointer *ret;
  gboolean bool_tmp;
  int i;
  gchar *name;

  /*** Go ! */
  if (!dir)
    dir = TD_PACKAGE_PLUGINS_DIR;
  if (!app)
    app = TD_APP(APP);
  name = g_strdup_printf ("%s%s", dir, file);

  /*** Recherche du module - Searching the module */
  bool_tmp = FALSE;
  for (i=0; i<g_list_length (app->mod_core); i++)
    if (!strcmp (g_list_nth_data (app->mod_core, i), name))
      {
	bool_tmp = TRUE;
	break;
      }
  if (!bool_tmp)
    {
      td_palette_message (_("Executing plug-in"), g_strdup_printf (_("Executing plug-in : file '%s' not loaded"), name), "failed");
      return NULL;
    }

  /*** Activation */
  ret = td_dlsym (PLUGINS_CORE[i], dir, file, symbol);
  return ret;
}

/**
 * td_app_mod_core_activate:
 * @app: td_app object (NULL => default app (APP))
 * @dir: directory location of plug-in (NULL=> default directory (TD_PACKAGE_PLUGINS_DIR))
 * @file: file of plug-in
 * @symbol: symbol called
 * 
 * fr: Execute le symbole du greffon charg
 *
 * en: Execute symbol's plug-in loaded
 * 
 * Return value: FALSE on error
 **/

gboolean td_app_mod_core_activate (TdApp *app, gchar *dir, gchar *file, gchar *symbol)
{
  void (*symbol2)() = NULL;
  (gpointer*) symbol2 = td_app_mod_core_symbol (app, dir, file, symbol);
  if (!symbol2)
    return FALSE;
  symbol2 ();
  return TRUE;
}

/**
 * td_app_mod_data_symbol:
 * @mod_id: module identifier
 * @symbol: symbol called
 * 
 * fr: Retourne le pointer du symbole du greffon charg
 *
 * en: Return pointer of symbol's plug-in loaded
 * 
 * Return value: pointer
 **/

gpointer *td_app_mod_data_symbol (int mod_id, gchar *symbol)
{
  gpointer *ret;
  ret = td_dlsym (PLUGINS_DATA[mod_id], "", g_strdup_printf ("%s %d", _("data plugins"), mod_id), symbol);
  return ret;
}

/**
 * td_app_mod_data_activate:
 * @mod_id: module identifier
 * @symbol: symbol called
 * 
 * fr: Execute le symbole du greffon charg
 *
 * en: Execute symbol's plug-in loaded
 * 
 * Return value: FALSE on error
 **/

gboolean td_app_mod_data_activate (int mod_id, gchar *symbol)
{
  void (*symbol2)() = NULL;
  (gpointer*) symbol2 = td_app_mod_data_symbol (mod_id, symbol);
  if (!symbol2)
    return FALSE;
  symbol2 ();
  return TRUE;
}

