/*****************************************************************************/
/*** td_app_editprop.c : Objet Gtk+
 *** td_app_editprop.c : Gtk+ object
 *
 *
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "commons.h"
#include "gtdk.h"
#include "icons.h"
#include "database.h"

#include "td_app.h"
#include "td_app_editprop.h"

#include "td_mod.h"
#include "td_mod_editprop.h"
#include "td_mod_box.h"

/*****************************************************************************/
/*** Arguments */
/*****************************************************************************/
static GtkObjectClass *parent_class = NULL;

/*****************************************************************************/
/*** Initialisation */
/*****************************************************************************/
static void td_app_editprop_init (TdAppEditprop *editprop)
{
  editprop->window = NULL;
  editprop->notebook = NULL;
  editprop->notebook_vbox = NULL;
}

static void td_app_editprop_class_init (TdAppEditpropClass *klass)
{
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) klass;
  parent_class = gtk_type_class (gtk_object_get_type ());
  klass->create = td_app_editprop_create;
}

GtkType td_app_editprop_get_type (void)
{
  static GtkType app_editprop_type = 0;
  if (!app_editprop_type)
    {
      static const GtkTypeInfo app_editprop_info =
      {
  	"TdAppEditprop", sizeof (TdAppEditprop), sizeof (TdAppEditpropClass),
  	(GtkClassInitFunc) td_app_editprop_class_init,
	(GtkObjectInitFunc) td_app_editprop_init,
	NULL, NULL, (GtkClassInitFunc) NULL,
      };
      app_editprop_type = gtk_type_unique (GTK_TYPE_OBJECT, &app_editprop_info);
    }
  return app_editprop_type;
}

/**
 * td_app_editprop_new:
 * 
 * fr: Cr un nouveau tabli
 *
 * en: Creates a new bench
 * 
 * Return value: bench
 **/

GtkObject *td_app_editprop_new (void)
{
  return GTK_OBJECT (gtk_type_new (td_app_editprop_get_type ()));
}

/**
 * td_app_editprop_destroy:
 * @editprop: bench
 * 
 * fr: Dtruit l'tabli
 *
 * en: Destroys the bench
 **/

void td_app_editprop_destroy (TdAppEditprop *editprop)
{
  g_return_if_fail (editprop != NULL);
  g_return_if_fail (TD_IS_APP_EDITPROP (editprop));
  gtdk_window_save_conf ("bench", editprop->window);
  gtk_widget_destroy (editprop->window);
  gtk_object_destroy (GTK_OBJECT (editprop));
}

/**
 * td_app_editprop_hide:
 * @editprop: bench
 * 
 * fr: Cache l'tabli
 *
 * en: Hides the bench
 **/

void td_app_editprop_hide (TdAppEditprop *editprop)
{
  g_return_if_fail (editprop != NULL);
  g_return_if_fail (TD_IS_APP_EDITPROP (editprop));
  gtdk_window_save_conf ("bench", editprop->window);
  gtk_widget_hide (GTK_WIDGET (editprop->window));
  gnome_config_set_string ("geometry-open/bench", "0");
  gnome_config_sync ();
}

/**
 * td_app_editprop_show:
 * @editprop: bench
 * 
 * fr: Affiche l'tabli
 *
 * en: Displays the bench
 **/

void td_app_editprop_show (TdAppEditprop *editprop)
{
  g_return_if_fail (editprop != NULL);
  g_return_if_fail (TD_IS_APP_EDITPROP (editprop));
  gtk_widget_show (GTK_WIDGET (editprop->window));
  gtk_widget_hide_all (GNOME_APP (editprop->window)->menubar->parent->parent);
  gnome_config_set_string ("geometry-open/bench", "1");
  gnome_config_sync ();
}

/*****************************************************************************/
/*** Commandes - Commands */
/*****************************************************************************/

/**
 * td_app_editprop_create:
 * @editprop: bench
 * 
 * fr: Cr l'interface de l'tabli de l'application
 *
 * en: Creates the interface of bench of application
 **/

void td_app_editprop_create (TdAppEditprop *editprop)
{
  GtkWidget *dock;
  GtkWidget *viewport;
  GtkWidget *vbox;
  gchar* txt_tmp;
  g_return_if_fail (editprop != NULL);
  g_return_if_fail (TD_IS_APP_EDITPROP (editprop));

  /*** Fentre - Window */
  editprop->window = gnome_app_new ("bench", NULL);
  gnome_app_enable_layout_config (GNOME_APP (editprop->window), FALSE);
  gtk_window_set_policy (GTK_WINDOW (editprop->window),TRUE, TRUE, FALSE);
  gtk_window_set_title (GTK_WINDOW (editprop->window), g_strdup_printf ("%s : %s", TD_PACKAGE, _("Bench")));
  gnome_app_create_menus (GNOME_APP (editprop->window), TD_APP (APP)->menu_gnomeui);
  gtdk_window_save_defaultconf ("bench", 300, 150);
  gtdk_window_load_conf ("bench", editprop->window);
  dock = GNOME_APP (editprop->window)->dock;

  /*** Multi-pages */
  viewport = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport);
  gnome_app_set_contents (GNOME_APP (editprop->window), viewport);
  editprop->notebook_vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (editprop->notebook_vbox);
  gtk_container_add (GTK_CONTAINER (viewport), editprop->notebook_vbox);
  editprop->notebook = gtk_notebook_new ();
  gtk_widget_show (editprop->notebook);
  gtk_box_pack_start (GTK_BOX (editprop->notebook_vbox), editprop->notebook, TRUE, TRUE, 0);
  if (!TD_APP (APP)->mode_debug)
    gtk_notebook_set_show_tabs (GTK_NOTEBOOK (editprop->notebook), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (editprop->notebook), FALSE);
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (editprop->notebook), vbox);
  gtdk_vbox_title (_("No view selected"), vbox);

  /*** Appels - Calls */
  gtk_signal_connect_object (GTK_OBJECT (editprop->window), "delete_event", GTK_SIGNAL_FUNC (td_app_editprop_hide), GTK_OBJECT (editprop));

  /*** Affichage - Displaying */
  txt_tmp = g_strdup_printf ("%s", gnome_config_get_string ("geometry-open/bench"));
  if (!strcmp (txt_tmp, "1"))
    {
      gtk_widget_show (GTK_WIDGET (editprop->window));
      gtk_widget_hide_all (GNOME_APP (editprop->window)->menubar->parent->parent);
    }
  else
    gtk_widget_hide (editprop->window);
}

/**
 * td_app_editprop_add_mod:
 * @editprop: bench
 * @mod: module
 * 
 * fr: Ajoute un module  l'tabli
 *
 * en: Add a module to the bench
 **/

void td_app_editprop_add_mod (TdAppEditprop *editprop, GtkObject *mod)
{
  GtkWidget *table;
  GtkWidget *box_tmp;
  int i;
  gchar *txt_tmp;
  gchar *txt_tmp2;
  GList *list_tmp;
  g_return_if_fail (editprop != NULL);
  g_return_if_fail (TD_IS_APP_EDITPROP (editprop));

  /*** Go ! */
  if (TD_MOD (mod)->id != -1)
    {
      TD_MOD (mod)->editprop_vbox = gtk_vbox_new (FALSE, 0);
      gtk_widget_show (TD_MOD (mod)->editprop_vbox);
      gtk_container_add (GTK_CONTAINER (editprop->notebook), TD_MOD (mod)->editprop_vbox);
      gtdk_vbox_title (TD_MOD (mod)->name_intl, TD_MOD (mod)->editprop_vbox);
      table = gtk_table_new (g_list_length (TD_MOD (mod)->editprop), 2, FALSE);
      gtk_widget_show (table);
      gtk_box_pack_start (GTK_BOX (TD_MOD (mod)->editprop_vbox), table, FALSE, FALSE, 0);

      /*** Widgets */
      for (i=0; i< g_list_length (TD_MOD (mod)->editprop); i++)
	{
	  txt_tmp = td_mod_editprop_param_value (TD_MOD (mod)->name, TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name, "editprop name");
	  if (txt_tmp)
	    TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name_intl = txt_tmp;
	  TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->label = gtk_label_new (g_strdup_printf ("%s:", TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name_intl));
	  gtk_table_attach (GTK_TABLE (table), TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->label, 0, 1, i, i+1, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
	  gtk_misc_set_padding (GTK_MISC (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->label), 5, 0);
	  gtk_widget_hide (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->label);
	  if (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_list)
	    {
	      box_tmp = gtk_hbox_new (FALSE, 0);
	      gtk_table_attach (GTK_TABLE (table), box_tmp, 1, 2, i, i+1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	      gtk_widget_show (box_tmp);
	      gtk_box_pack_start (GTK_BOX (box_tmp), TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_parent, TRUE, TRUE, 0);
	      gtk_widget_hide (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_parent);
	      gtk_box_pack_start (GTK_BOX (box_tmp), TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_list, FALSE, FALSE, 0);
	      gtk_widget_hide (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_list);
	    }
	  else
	    {
	      gtk_table_attach (GTK_TABLE (table), TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_parent, 1, 2, i, i+1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	      gtk_widget_hide (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_parent);
	    }

	  /*** Editabilit - Editability */
	  txt_tmp = td_mod_editprop_param_value (TD_MOD (mod)->name, TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name, "editprop visibility");
	  if ((txt_tmp) && (!strcmp (txt_tmp, "uneditable")))
	    if (GTK_IS_ENTRY (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget))
	      gtk_entry_set_editable (GTK_ENTRY (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget), FALSE);

	  /*** Visibilit - Visibiliy */
	  txt_tmp = td_mod_editprop_param_value (TD_MOD (mod)->name, TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name, "editprop visibility");
	  if ((txt_tmp) && (!strcmp (txt_tmp, "uneditable")))
	    if (GTK_IS_ENTRY (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget))
	      gtk_entry_set_editable (GTK_ENTRY (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget), FALSE);

	  /*** Enumration - Enumeration */
	  list_tmp = NULL;
	  txt_tmp = g_strdup_printf ("%s", td_mod_editprop_param_value (TD_MOD (mod)->name, TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name, "editprop list item"));
	  if (strcmp (txt_tmp, "(null)"))
	    {
	      txt_tmp2 = strtok (txt_tmp, "',");
	      while (txt_tmp2)
		{
		  list_tmp = g_list_append (list_tmp, txt_tmp2);
		  txt_tmp2 = strtok (NULL, "',");
		}
  	      gtdk_option_menu_set_popdown_strings (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_list, list_tmp, TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget);
	    }

	  /*** Requete - Query */
	  txt_tmp = td_mod_editprop_param_value (TD_MOD (mod)->name, TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name, "editprop list query");
	  if (!txt_tmp)
	    txt_tmp = td_mod_editprop_param_value (TD_MOD (mod)->name, TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->name, "editprop list static query");
	  if (txt_tmp)
	    {
  	      list_tmp = td_database_column (txt_tmp);
	      gtdk_option_menu_set_popdown_strings (TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget_list, td_database_column (txt_tmp), TD_MOD_EDITPROP (g_list_nth_data (TD_MOD (mod)->editprop, i))->widget);
	    }
	}
    }
}
