/*****************************************************************************/
/* td_db_table_field.c : Objet Gtk+
 * td_db_table_field.c : Gtk+ object
 *
 *
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2000-2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "commons.h"
#include "td_db_table_field.h"

/*****************************************************************************/
/*** Arguments */
/*****************************************************************************/
static GtkObjectClass *parent_class = NULL;

enum {
  ARG_0,
  ARG_NAME,
  ARG_COMMENT,
};

static void td_db_table_field_set_name (TdDbTableField *db_table_field, gchar *name)
{
  g_return_if_fail (db_table_field != NULL);
  g_return_if_fail (TD_IS_DB_TABLE_FIELD (db_table_field));
  if (db_table_field->name != name)
    db_table_field->name = name;
}

static void td_db_table_field_set_comment (TdDbTableField *db_table_field, gchar *comment)
{
  g_return_if_fail (db_table_field != NULL);
  g_return_if_fail (TD_IS_DB_TABLE_FIELD (db_table_field));
  if (db_table_field->comment != comment)
    db_table_field->comment = comment;
}

static void td_db_table_field_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
{
  TdDbTableField *db_table_field;
  db_table_field = TD_DB_TABLE_FIELD (object);
  switch (arg_id)
    {
    case ARG_NAME:
      td_db_table_field_set_name (db_table_field, GTK_VALUE_STRING (*arg));
      break;
    case ARG_COMMENT:
      td_db_table_field_set_comment (db_table_field, GTK_VALUE_STRING (*arg));
      break;
    default:
      break;
    }
}

static void td_db_table_field_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
{
  TdDbTableField *db_table_field;
  db_table_field = TD_DB_TABLE_FIELD (object);
   switch (arg_id)
    {
    case ARG_NAME:
      GTK_VALUE_STRING (*arg) = db_table_field->name;
      break;
    case ARG_COMMENT:
      GTK_VALUE_STRING (*arg) = db_table_field->comment;
      break;
    default:
      arg->type = GTK_TYPE_INVALID;
      break;
    }
}

/*****************************************************************************/
/*** Initialisation */
/*****************************************************************************/
static void td_db_table_field_init (TdDbTableField *db_table_field)
{
  db_table_field->name = NULL;
  db_table_field->comment = NULL;
  db_table_field->property_name = NULL;
  db_table_field->property_value = NULL;
}

static void td_db_table_field_class_init (TdDbTableFieldClass *klass)
{
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) klass;
  parent_class = gtk_type_class (gtk_object_get_type());
  gtk_object_add_arg_type ("TdDbTableField::name", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_NAME);
  gtk_object_add_arg_type ("TdDbTableField::comment", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_COMMENT);
  object_class->set_arg = td_db_table_field_set_arg;
  object_class->get_arg = td_db_table_field_get_arg;
}

GtkType td_db_table_field_get_type (void)
{
  static GtkType db_table_field_type = 0;
  if (!db_table_field_type)
    {
      static const GtkTypeInfo db_table_field_info =
      {
  	"TdDbTableField", sizeof (TdDbTableField), sizeof (TdDbTableFieldClass),
  	(GtkClassInitFunc) td_db_table_field_class_init,
	(GtkObjectInitFunc) td_db_table_field_init,
	NULL, NULL, (GtkClassInitFunc) NULL,
      };
      db_table_field_type = gtk_type_unique (GTK_TYPE_OBJECT, &db_table_field_info);
    }
  return db_table_field_type;
}

/**
 * td_db_table_field_new:
 * 
 * fr: Cr un nouveau schma de champs
 *
 * en: Creates new field schema
 * 
 * Return value: field schema
 **/

GtkObject *td_db_table_field_new (void)
{
  return GTK_OBJECT (gtk_type_new (td_db_table_field_get_type ()));
}

/*****************************************************************************/
/*** Affectations - Allocations */
/*****************************************************************************/

/**
 * td_db_table_field_add_property:
 * @db_table_field: field schema
 * @property_name: property's name
 * @property_value: property's value
 * 
 * fr: Ajoute la proprit au schma du champs
 *
 * en: Add the property to field schema
 **/

void td_db_table_field_add_property (TdDbTableField *db_table_field, gchar *property_name, gchar *property_value)
{
  g_return_if_fail (db_table_field != NULL);
  g_return_if_fail (TD_IS_DB_TABLE_FIELD (db_table_field));
  db_table_field->property_name = g_list_append (db_table_field->property_name, property_name);
  db_table_field->property_value = g_list_append (db_table_field->property_value, property_value);
}

/*****************************************************************************/
/*** Commandes - Commands */
/*****************************************************************************/

/**
 * td_db_table_field_property_value:
 * @db_table_field: field parameters
 * @property_name: property's name
 * 
 * fr: Retourne la valeur de la proprit du paramtrage du champs
 *
 * en: Return the value of property of field parameters
 * 
 * Return value: string
 **/

gchar *td_db_table_field_property_value (TdDbTableField *db_table_field, gchar *property_name)
{
  int i;
  for (i=0; i<g_list_length (db_table_field->property_name); i++)
    if (!strcmp ((gchar*) (g_list_nth_data (db_table_field->property_name, i)), property_name))
      return (g_list_nth_data (db_table_field->property_value, i));
  return NULL;
}

/*****************************************************************************/
/*** Fichier - File */
/*****************************************************************************/

/**
 * td_db_table_field_load:
 * @doc: XML document
 * @ns: XML namespace
 * @cur: XML node
 * 
 * fr: Retourne le schma du champs correspondant  la localisation XML
 *
 * en: Return the field schema corresponding the XML location
 * 
 * Return value: field schema
 **/

GtkObject *td_db_table_field_load (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
  GtkObject *ret;

  /*** Champs - Fiedls */
  ret = td_db_table_field_new ();
  gtk_object_set (GTK_OBJECT (ret), 
		  "name", xmlGetProp (cur, "name"), 
		  "comment", xmlGetProp (cur, "comment"), NULL);

  /*** Proprits - Properties */
  cur = cur->xmlChildrenNode->xmlChildrenNode;
  while (cur)
    {
      if (!strcmp (cur->name, "property"))
	td_db_table_field_add_property (TD_DB_TABLE_FIELD (ret), xmlGetProp (cur, "name"), xmlGetProp (cur, "value"));
      cur = cur->next;
    }
  return (ret);
}






