/*****************************************************************************/
/* td_timeruler.c : Objet Gtk+
 * td_timeruler.c : Gtk+ object
 *
 *
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manager
 * Copyright (c) 2001 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@toutdoux.org>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "commons.h"
#include "gtdk.h"

#include "td_app.h"
#include "td_app_palette.h"

#include "td_timeruler.h"

/*****************************************************************************/
/*** Arguments */
/*****************************************************************************/
static GtkObjectClass *parent_class = NULL;

/*****************************************************************************/
/*** Initialisation */
/*****************************************************************************/
static void td_timeruler_init (TdTimeruler *timeruler)
{
  timeruler->widget = NULL;
  timeruler->canvas = NULL;
  timeruler->lower = NULL;
  timeruler->upper = NULL;
  timeruler->width = 0;
  timeruler->height = 0;
  timeruler->gc = NULL;
  timeruler->font = NULL;
  timeruler->zoom_label = NULL;
}

static void td_timeruler_class_init (TdTimerulerClass *klass)
{
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) klass;
  parent_class = gtk_type_class (gtk_object_get_type());
  klass->set_lower = td_timeruler_set_lower;
}

GtkType td_timeruler_get_type (void)
{
  static GtkType timeruler_type = 0;
  if (!timeruler_type)
    {
      static const GtkTypeInfo timeruler_info =
      {
  	"TdTimeruler", sizeof (TdTimeruler), sizeof (TdTimerulerClass),
  	(GtkClassInitFunc) td_timeruler_class_init,
	(GtkObjectInitFunc) td_timeruler_init,
	NULL, NULL, (GtkClassInitFunc) NULL,
      };
      timeruler_type = gtk_type_unique (GTK_TYPE_OBJECT, &timeruler_info);
    }
  return timeruler_type;
}

/**
 * td_timeruler_new:
 * 
 * fr: Cr une nouvelle rgle du temps
 *
 * en: Creates a new time ruler
 * 
 * Return value: time ruler
 **/

GtkObject *td_timeruler_new (void)
{
  return GTK_OBJECT (gtk_type_new (td_timeruler_get_type ()));
}

/**
 * td_timeruler_destroy:
 * @timeruler: time ruler
 * 
 * fr: Dtruit la rgle du temps
 *
 * en: Destroys the time ruler
 **/

void td_timeruler_destroy (TdTimeruler *timeruler)
{
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  gtk_object_destroy (GTK_OBJECT (timeruler));
  g_date_free (timeruler->lower);
  g_date_free (timeruler->upper);
  gdk_gc_destroy (timeruler->gc);
  if (WINDOW_DATE_SELECTION)
    {
      gtk_widget_destroy (WINDOW_DATE_SELECTION);
      WINDOW_DATE_SELECTION = NULL;
    }
}

/**
 * td_timeruler_hide:
 * @timeruler: time ruler
 * 
 * fr: Cache la rgle du temps
 *
 * en: Hides the time ruler
 **/

void td_timeruler_hide (TdTimeruler *timeruler)
{
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  if (WINDOW_DATE_SELECTION)
    {
      gtk_widget_destroy (WINDOW_DATE_SELECTION);
      WINDOW_DATE_SELECTION = NULL;
    }
}

/*****************************************************************************/
/*** Affectations - Allocations */
/*****************************************************************************/

/**
 * td_timeruler_set_canvas:
 * @timeruler: time ruler
 * @canvas: canvas
 * 
 * fr: Affecte le cavenas  la rgle du temps
 *
 * en: Allocates the canvas to time ruler
 **/

void td_timeruler_set_canvas (TdTimeruler *timeruler, GtkWidget *canvas)
{
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  if (timeruler->canvas != canvas)
    timeruler->canvas = canvas;
}

/**
 * td_timeruler_set_lower:
 * @timeruler: time ruler
 * @lower: lower date
 * 
 * fr: Affecte la date basse  la rgle du temps
 *
 * en: Allocates the lower date to time ruler
 **/

void td_timeruler_set_lower (TdTimeruler *timeruler, GDate *lower)
{
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  if (timeruler->lower != lower)
    timeruler->lower = lower;
}

/*****************************************************************************/
/*** Commandes - Commands */
/*****************************************************************************/

/**
 * td_timeruler_create:
 * @timeruler: time ruler
 * @vbox1: container of slider
 * @vbox2: container of calendar
 * 
 * fr: Cr l'interface de la rgle du temps
 *
 * en: Creates the interface of time ruler
 **/

void td_timeruler_create (TdTimeruler *timeruler, GtkWidget *vbox1, GtkWidget *vbox2)
{
  GdkWindow *window = GTK_WIDGET (TD_APP_PALETTE (TD_APP (APP)->palette)->window)->window;
  GdkColor color;
  GtkAdjustment *adj;
  GtkWidget *hscale;
  GtkWidget *hbox1;
  GtkWidget *calendar_arrow;
  GtkWidget *calendar_button;
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));

  /*** Crayon - Pencil */
  timeruler->gc = gdk_gc_new (window);
  timeruler->font = gtdk_widget_get_default_font();
  gdk_gc_set_font (timeruler->gc, timeruler->font);
  gdk_gc_set_line_attributes (timeruler->gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
  color = gtdk_color_parse ("fg[normal]");
  gdk_color_alloc (gtk_widget_get_default_colormap (), &color);
  gdk_gc_set_foreground (timeruler->gc, &color);
  timeruler->height = gdk_string_height (timeruler->font, "0")+5;

  /*** Zoom */
  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
  adj = GTK_ADJUSTMENT (gtk_adjustment_new (13, 1, 19, 1, 1, 0));
  hscale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
  gtk_scale_set_draw_value (GTK_SCALE (hscale), FALSE);
  gtk_widget_show (hscale);
  gtk_box_pack_start (GTK_BOX (hbox1), hscale, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (td_timeruler_zoom_changed), timeruler);
  timeruler->zoom_label = gtk_label_new (g_strdup_printf ("%d %s ", 1, _("day")));
  gtk_widget_show (timeruler->zoom_label);
  gtk_widget_set_usize (timeruler->zoom_label, -1, 18);
  gtk_box_pack_start (GTK_BOX (hbox1), timeruler->zoom_label, FALSE, FALSE, 5);

  /*** Rgle - Ruler */
  timeruler->widget = gtk_drawing_area_new ();
  gtk_widget_show (timeruler->widget);
  gtk_box_pack_start (GTK_BOX (vbox1), timeruler->widget, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (timeruler->widget), "expose_event", GTK_SIGNAL_FUNC (td_timeruler_expose_event), timeruler);

  /*** Calendrier - Calendar */
  calendar_button = gtk_button_new ();
  gtk_widget_show (calendar_button);
  gtk_widget_set_usize (calendar_button, 18, 18);
  gtk_box_pack_start (GTK_BOX (vbox2), calendar_button, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (calendar_button), "button_press_event", GTK_SIGNAL_FUNC (td_timeruler_calendar_button_press), timeruler);
  calendar_arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
  gtk_widget_show (calendar_arrow);
  gtk_container_add (GTK_CONTAINER (calendar_button), calendar_arrow);
}

/**
 * td_timeruler_expose_event:
 * @widget: draw area of timeruler
 * @event: expose event
 * @timeruler: time ruler
 * 
 * fr: Evnements 'expose_event' de l'aire de la rgle du temps
 *
 * en: Event 'expose_event' of draw area of time ruler
 **/

void td_timeruler_expose_event (GtkWidget *widget, GdkEventExpose *event, TdTimeruler *timeruler)
{
  GdkWindow *window = widget->window;
  GtkWidget *window2;
  GdkColor color;
  int days;
  float y = 2;
  gchar *tmp_format;

  /*** Priode - Period */
  if (!timeruler->upper)
    td_timeruler_set_scale (timeruler, 13);
  gdk_window_clear (window);
  color = gtdk_color_parse ("fg[normal]");
  gdk_color_alloc (gtk_widget_get_default_colormap (), &color);
  gdk_gc_set_foreground (timeruler->gc, &color);

  /*** Anne - Year */
  tmp_format = td_timeruler_year_optimal_format (timeruler);
  if (tmp_format)
    {
      td_timeruler_year_draw (timeruler, y, y+timeruler->height);
      y=y+timeruler->height;
      gdk_draw_line (window, timeruler->gc, 0, y, widget->allocation.width-1, y);
    }

  /*** Mois - Month */
  tmp_format = td_timeruler_month_optimal_format (timeruler);
  if (tmp_format)
    {
      td_timeruler_month_draw (timeruler, y, y+timeruler->height, tmp_format);
      y=y+timeruler->height;
      gdk_draw_line (window, timeruler->gc, 0, y, widget->allocation.width-1, y);
    }

  /*** Semaine - Week */
  tmp_format = td_timeruler_week_optimal_format (timeruler);
  if (tmp_format)
    {
      td_timeruler_week_draw (timeruler, y, y+timeruler->height, tmp_format);
      y=y+timeruler->height;
      gdk_draw_line (window, timeruler->gc, 0, y, widget->allocation.width-1, y);
    }

  /*** Jour - Day */
  tmp_format = td_timeruler_day_optimal_format (timeruler);
  if (tmp_format)
    {
      td_timeruler_day_draw (timeruler, y, y+timeruler->height, tmp_format);
      y=y+timeruler->height;
      gdk_draw_line (window, timeruler->gc, 0, y, widget->allocation.width-1, y);
    }

  /*** Cadre - Frame */
  if (widget->allocation.height != y)
    {
      gtk_widget_set_usize (widget, -1, y);
      if ((GTK_IS_TABLE (widget->parent->parent)) && (widget->parent->parent->parent->parent->parent->parent))
	{
	  window2 = widget->parent->parent->parent->parent->parent->parent;
	  gtk_widget_set_usize (window2, window2->allocation.width, window2->allocation.height);
	}
    }
  else
    {
      color = gtdk_color_parse ("light[normal]");
      gdk_color_alloc (gtk_widget_get_default_colormap (), &color);
      gdk_gc_set_foreground (timeruler->gc, &color);
      gdk_draw_line (window, timeruler->gc, 0, 0, widget->allocation.width, 0);
      gdk_draw_line (window, timeruler->gc, 0, 0, 0, widget->allocation.height);
      color = gtdk_color_parse ("black");
      gdk_color_alloc (gtk_widget_get_default_colormap (), &color);
      gdk_gc_set_foreground (timeruler->gc, &color);
      gdk_draw_line (window, timeruler->gc, widget->allocation.width-1, 0, widget->allocation.width-1, widget->allocation.height-1);
      gdk_draw_line (window, timeruler->gc, 0, widget->allocation.height-1, widget->allocation.width, widget->allocation.height-1);
      color = gtdk_color_parse ("dark[normal]");
      gdk_color_alloc (gtk_widget_get_default_colormap (), &color);
      gdk_gc_set_foreground (timeruler->gc, &color);
      gdk_draw_line (window, timeruler->gc, widget->allocation.width-2, 2, widget->allocation.width-2, widget->allocation.height-2);
    }
}

/**
 * td_timeruler_zoom_changed:
 * @adj: adjustement of zoom of timeruler
 * @timeruler: time ruler
 * 
 * fr: Evnement 'value_changed' du zoom de la rgle du temps
 *
 * en: Event 'value_changed' of zoom of time ruler
 **/

void td_timeruler_zoom_changed (GtkAdjustment *adj, TdTimeruler *timeruler)
{
  int x1, x2, i;
  float width_init, width;
  GnomeCanvasGroup *root;
  root = gnome_canvas_root (GNOME_CANVAS (timeruler->canvas));

  /*** Rgle - Ruler */
  width_init = timeruler->width;
  td_timeruler_set_scale (timeruler, adj->value);
  width = timeruler->width;
  gtk_widget_queue_draw (GTK_WIDGET (timeruler->widget));
  
  /*** Items */
  for (i=0; i<g_list_length (root->item_list); i++)
    {
      x1 = td_float_round (gtdk_canvas_i2w_x (g_list_nth_data (root->item_list, i), 0)/width_init);
      x2 = td_float_round ((GNOME_CANVAS_RE (g_list_nth_data (GNOME_CANVAS_GROUP (g_list_nth_data (root->item_list, i))->item_list, 0))->x2)/width_init);
      gnome_canvas_item_move (g_list_nth_data (root->item_list, i), -(width_init*x1)+(width*x1), 0);
      gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (g_list_nth_data (root->item_list, i))->item_list, 0),
			     "x2", width*x2, NULL);
    }
}

/**
 * td_timeruler_calendar_button_press:
 * @widget: button clicked for calendar creation
 * @event: expose event
 * @timeruler: time ruler
 * 
 * fr: Cr l'interface du calendier de la rgle du temps
 *
 * en: Creates the interface of calendar of time ruler
 **/

void td_timeruler_calendar_button_press (GtkWidget *widget, GdkEventButton *event, TdTimeruler *timeruler)
{
  GtkWidget *window;
  GtkWidget *calendar;
  if (WINDOW_DATE_SELECTION)
    {
      gtk_widget_destroy (WINDOW_DATE_SELECTION);
      WINDOW_DATE_SELECTION = NULL;
    }
  WINDOW_DATE_SELECTION = gtk_window_new (GTK_WINDOW_POPUP);
  gtk_window_set_policy (GTK_WINDOW (WINDOW_DATE_SELECTION), FALSE, FALSE, TRUE);
  gtk_window_set_position (GTK_WINDOW (WINDOW_DATE_SELECTION), GTK_WIN_POS_MOUSE);
  calendar = gtk_calendar_new ();
  gtk_widget_show (calendar);
  gtk_container_add (GTK_CONTAINER (WINDOW_DATE_SELECTION), calendar);
  gtk_calendar_display_options (GTK_CALENDAR (calendar), GTK_CALENDAR_SHOW_HEADING | GTK_CALENDAR_SHOW_DAY_NAMES | GTK_CALENDAR_SHOW_WEEK_NUMBERS | GTK_CALENDAR_WEEK_START_MONDAY);
  gtdk_calendar_select_date (GTK_CALENDAR (calendar), timeruler->lower);
  gtk_widget_show (WINDOW_DATE_SELECTION);
  gtk_signal_connect (GTK_OBJECT (calendar), "day_selected", GTK_SIGNAL_FUNC (td_timeruler_calendar_day_selected), timeruler);
}

/**
 * td_timeruler_calendar_day_selected:
 * @calendar: calendar widget
 * @timeruler: time ruler
 * 
 * fr: Evnement 'day_selected' du calendrier de la rgle du temps
 *
 * en: Event 'day_selected' of calendar of time ruler
 **/

void td_timeruler_calendar_day_selected (GtkCalendar *calendar, TdTimeruler *timeruler)
{
  int x1, x2, i;
  float width_init, width;
  GDate *lower_init;
  GnomeCanvasGroup *root;
  gchar *txt_tmp;
  gchar *txt_tmp2;
  root = gnome_canvas_root (GNOME_CANVAS (timeruler->canvas));

  /*** Go ! */
  txt_tmp = td_date_print (timeruler->lower, "year-month-day");
  txt_tmp2 = td_date_print (gtdk_calendar_get_date (calendar), "year-month-day");
  if (strcmp (txt_tmp, txt_tmp2))
    {
      /*** Rgle - Ruler */
      width_init = timeruler->width;
      lower_init = g_date_new_dmy (g_date_day (timeruler->lower), g_date_month (timeruler->lower), g_date_year (timeruler->lower));
      td_timeruler_set_lower (timeruler, gtdk_calendar_get_date (calendar));
      td_timeruler_set_scale (timeruler, timeruler->scale);
      width = timeruler->width;
      gtk_widget_queue_draw (GTK_WIDGET (timeruler->widget));

      /*** Items */
      for (i=0; i<g_list_length (root->item_list); i++)
	{
	  x1 = td_float_round (gtdk_canvas_i2w_x (g_list_nth_data (root->item_list, i), 0)/width_init);
	  x2 = td_float_round ((GNOME_CANVAS_RE (g_list_nth_data (GNOME_CANVAS_GROUP (g_list_nth_data (root->item_list, i))->item_list, 0))->x2)/width_init);
  	  gnome_canvas_item_move (g_list_nth_data (root->item_list, i), -(width_init*x1)+(width*x1), 0);
	  if (td_date_period_day (timeruler->lower, lower_init) == 0)
	    gnome_canvas_item_move (g_list_nth_data (root->item_list, i), (-width*td_date_period_day (lower_init, timeruler->lower)), 0);
	  else
	    gnome_canvas_item_move (g_list_nth_data (root->item_list, i), (width*td_date_period_day (timeruler->lower, lower_init)), 0);
	  gnome_canvas_item_set (g_list_nth_data (GNOME_CANVAS_GROUP (g_list_nth_data (root->item_list, i))->item_list, 0),
				 "x2", width*x2, NULL);
	}
      g_date_free (lower_init);
    }
  else
    if (WINDOW_DATE_SELECTION)
      {
	gtk_widget_destroy (WINDOW_DATE_SELECTION);
	WINDOW_DATE_SELECTION = NULL;
      }
}

/*****************************************************************************/
/*** Dessin - Drawing */
/*****************************************************************************/

/**
 * td_timeruler_year_optimal_format:
 * @timeruler: time ruler
 * 
 * fr: Retourne le format optimal pour l'affichage des annes 
 *
 * en: Return the optimal format for displaying year
 * 
 * Return value: string
 **/

gchar *td_timeruler_year_optimal_format (TdTimeruler *timeruler)
{
  if ((timeruler->width*28)>gdk_string_width (timeruler->font, _("0000-00"))) /*** 28 jours est la priode minimale d'un mois - 28 days is the minimal period of a month */
    return NULL;
  return "year";
}

/**
 * td_timeruler_year_draw:
 * @timeruler: time ruler
 * @y0: y0 coordinate
 * @y1: y1 coordinate
 * 
 * fr: Affiche les annes sur la rgle du temps 
 *
 * en: Displays years on time ruler
 **/

void td_timeruler_year_draw (TdTimeruler *timeruler, int y0, int y1)
{
  GdkWindow *window = timeruler->widget->window;
  int day;
  int i, j;
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));

  /*** Go ! */
  day = 0;
  for (i=g_date_year (timeruler->lower); i<g_date_year (timeruler->upper)+1; i++)
    {
      gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
      gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, g_strdup_printf ("%d", i));

      /*** Premire et dernire anne - First and last year */
      if ((i==g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=g_date_month (timeruler->lower); j<g_date_month (timeruler->upper)+1; j++)
	  {
	    if ((j==g_date_month (timeruler->lower)) && (j==g_date_month (timeruler->upper))) /*** Premier et dernier mois - First and last month */
	      day = day + g_date_day (timeruler->upper) - (g_date_day (timeruler->lower)-1);
	    if ((j==g_date_month (timeruler->lower)) && (j!=g_date_month (timeruler->upper))) /*** Premier mois - First month */
	      day = day + g_date_days_in_month (j, i) - (g_date_day (timeruler->lower)-1);
	    if ((j!=g_date_month (timeruler->lower)) && (j==g_date_month (timeruler->upper))) /*** Dernier mois - Last month */
	      day = day + g_date_day (timeruler->upper);
	    if ((j!=g_date_month (timeruler->lower)) && (j!=g_date_month (timeruler->upper)))
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Premire anne - First year */
      if ((i==g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=g_date_month (timeruler->lower); j<13; j++)
	  {
	    if (j==g_date_month (timeruler->lower)) /*** Premier mois - First month */
	      day = day + g_date_days_in_month (j, i) - (g_date_day (timeruler->lower)-1);
	    else
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Dernire anne - Last year */
      if ((i!=g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=1; j<g_date_month (timeruler->upper)+1; j++)
	  {
	    if (j==g_date_month (timeruler->upper)) /*** Dernier mois - Last month */
	      day = day + g_date_day (timeruler->upper);
	    else
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Anne intermdiaire - Intermediate year */
      if ((i!=g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=1; j<13; j++)
	  day = day + g_date_days_in_month (j, i);
    }
}

/**
 * td_timeruler_month_optimal_format:
 * @timeruler: time ruler
 * 
 * fr: Retourne le format optimal pour l'affichage des mois
 *
 * en: Return the optimal format for displaying months
 * 
 * Return value: string
 **/

gchar *td_timeruler_month_optimal_format (TdTimeruler *timeruler)
{
  if (((timeruler->width)>gdk_string_width (timeruler->font, _("0000-00-00"))) ||
      ((timeruler->width*28)<gdk_string_width (timeruler->font, "00")))
    return NULL;
  if ((timeruler->width*28)>gdk_string_width (timeruler->font, _("0000-00")))
    return _("year-month");
  return "month";
}

/**
 * td_timeruler_month_draw:
 * @timeruler: time ruler
 * @y0: y0 coordinate
 * @y1: y1 coordinate
 * @format: date format
 * 
 * fr: Affiche les mois sur la rgle du temps 
 *
 * en: Displays months on time ruler
 **/

void td_timeruler_month_draw (TdTimeruler *timeruler, int y0, int y1, gchar *format)
{
  GdkWindow *window = timeruler->widget->window;
  int day;
  int i, j;
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  if (!format)
    format = "month";

  /*** Go ! */
  day = 0;
  for (i=g_date_year (timeruler->lower); i<g_date_year (timeruler->upper)+1; i++)
    {

      /*** Premire et dernire anne - First and last year */
      if ((i==g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=g_date_month (timeruler->lower); j<g_date_month (timeruler->upper)+1; j++)
	  {
	    gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
	    gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_month (j, i, format));
	    if ((j==g_date_month (timeruler->lower)) && (j==g_date_month (timeruler->upper))) /*** Premier et dernier mois - First and last month */
	      day = day + g_date_day (timeruler->upper) - (g_date_day (timeruler->lower)-1);
	    if ((j==g_date_month (timeruler->lower)) && (j!=g_date_month (timeruler->upper))) /*** Premier mois - First month */
	      day = day + g_date_days_in_month (j, i) - (g_date_day (timeruler->lower)-1);
	    if ((j!=g_date_month (timeruler->lower)) && (j==g_date_month (timeruler->upper))) /*** Dernier mois - Last month */
	      day = day + g_date_day (timeruler->upper);
	    if ((j!=g_date_month (timeruler->lower)) && (j!=g_date_month (timeruler->upper)))
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Premire anne - Fisrt year */
      if ((i==g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=g_date_month (timeruler->lower); j<13; j++)
	  {
	    gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
	    gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_month (j, i, format));
	    if (j==g_date_month (timeruler->lower)) /*** Premier mois - First month */
	      day = day + g_date_days_in_month (j, i) - (g_date_day (timeruler->lower)-1);
	    else
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Dernire anne - Last year */
      if ((i!=g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=1; j<g_date_month (timeruler->upper)+1; j++)
	  {
	    gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
	    gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_month (j, i, format));
	    if (j==g_date_month (timeruler->upper)) /*** Dernier mois - Last month */
	      day = day + g_date_day (timeruler->upper);
	    else
	      day = day + g_date_days_in_month (j, i);
	  }

      /*** Anne intermdiaire - Intermediate year */
      if ((i!=g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=1; j<13; j++)
	  {
	    gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
	    gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_month (j, i, format));
	    day = day + g_date_days_in_month (j, i);
	  }
    }
}

/**
 * td_timeruler_week_optimal_format:
 * @timeruler: time ruler
 * 
 * fr: Retourne le format optimal pour l'affichage des semaines
 *
 * en: Return the optimal format for displaying weeks
 * 
 * Return value: string
 **/

gchar *td_timeruler_week_optimal_format (TdTimeruler *timeruler)
{
  if (((timeruler->width)>gdk_string_width (timeruler->font, _("W00 0000-00-00"))) ||
      ((timeruler->width*7)<gdk_string_width (timeruler->font, "W00")))
    return NULL;
  if ((timeruler->width*7)>gdk_string_width (timeruler->font, _("0000-W00")))
    return _("year-week");
  return "week";
}

/**
 * td_timeruler_week_draw:
 * @timeruler: time ruler
 * @y0: y0 coordinate
 * @y1: y1 coordinate
 * @format: date format
 * 
 * fr: Affiche les semaines sur la rgle du temps 
 *
 * en: Displays weeks on time ruler
 **/

void td_timeruler_week_draw (TdTimeruler *timeruler, int y0, int y1, gchar *format)
{
  GdkWindow *window = timeruler->widget->window;
  int day;
  int week;
  int i, j, k;
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  if (!format)
    format = "week";

  /*** Go ! */
  day = 0;
  week = g_date_monday_week_of_year (timeruler->lower);
  for (i=g_date_year (timeruler->lower); i<g_date_year (timeruler->upper)+1; i++)
    {

      /*** Premire et dernire anne - First and last year */
      if ((i==g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=g_date_monday_week_of_year (timeruler->lower); j<g_date_monday_week_of_year (timeruler->upper)+1; j++)
	  {
	    if (j==g_date_monday_week_of_year (timeruler->lower)) /*** Premire semaine - First week */
	      {
		gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_week (j, i, format));
		day = day-g_date_weekday (timeruler->lower)+8;
	      }
	    else /*** Semaine intermdiaire - Intermediate week */
	      {
		gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_week (j, i, format));
		day = day+7;
	      }
	  }

      /*** Premire anne - First year */
      if ((i==g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=g_date_monday_week_of_year (timeruler->lower); j<g_date_monday_weeks_in_year (i)+1; j++)
	  {
	    if (j==g_date_monday_week_of_year (timeruler->lower)) /*** First week - Premire semaine */
	      {
		gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_week (j, i, format));
		day = day-g_date_weekday (timeruler->lower)+7;
	      }
	    else /*** Semaine intermdiaire - Intermediate week */
	      {
		gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_week (j, i, format));
		day = day+7;
	      }
	  }

      /*** Dernire anne - Last year */
      if ((i!=g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=1; j<g_date_monday_week_of_year (timeruler->upper)+1; j++)
	  {
	    gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
	    gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_week (j, i, format));
	    day = day+7;
	  }

      /*** Anne intermdiaire - Intermediate year */
      if ((i!=g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=1; j<g_date_monday_weeks_in_year (i)+1; j++)
	  {
	    gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
	    gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_week (j, i, format));
	    day = day+7;
	  }
    }
}

/**
 * td_timeruler_day_optimal_format:
 * @timeruler: time ruler
 * 
 * fr: Retourne le format optimal pour l'affichage des jours
 *
 * en: Return the optimal format for displaying days
 * 
 * Return value: string
 **/

gchar *td_timeruler_day_optimal_format (TdTimeruler *timeruler)
{
  if ((timeruler->width)<gdk_string_width (timeruler->font, "00"))
    return NULL;
  if ((timeruler->width)>gdk_string_width (timeruler->font, _("0000-00-00")))
    return _("year-month-day");
  if ((timeruler->width)>gdk_string_width (timeruler->font, _("W00 0000-00-00")))
    return _("week year-month-day");
  return "day";
}

/**
 * td_timeruler_day_draw:
 * @timeruler: time ruler
 * @y0: y0 coordinate
 * @y1: y1 coordinate
 * @format: date format
 * 
 * fr: Affiche les jours sur la rgle du temps 
 *
 * en: Displays days on time ruler
 **/

void td_timeruler_day_draw (TdTimeruler *timeruler, int y0, int y1, gchar *format)
{
  GdkWindow *window = timeruler->widget->window;
  int day;
  int i, j, k;
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  if (!format)
    format = "day";

  /*** Go ! */
  day = 0;
  for (i=g_date_year (timeruler->lower); i<g_date_year (timeruler->upper)+1; i++)
    {

      /*** Premire et dernire anne - First and last year */
      if ((i==g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=g_date_month (timeruler->lower); j<g_date_month (timeruler->upper)+1; j++)
	  {

	    /*** Premier et dernier mois - First and last month */
	    if ((j==g_date_month (timeruler->lower)) && (j==g_date_month (timeruler->upper)))
	      for (k=g_date_day (timeruler->lower); k<g_date_day (timeruler->upper)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}

	    /*** Premier mois - First month */
	    if ((j==g_date_month (timeruler->lower)) && (j!=g_date_month (timeruler->upper)))
	      for (k=g_date_day (timeruler->lower); k<g_date_days_in_month (j, i)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}

	    /*** Dernier mois - Last month */
	    if ((j!=g_date_month (timeruler->lower)) && (j==g_date_month (timeruler->upper)))
	      for (k=1; k<g_date_day (timeruler->upper)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}

	    /*** Mois intermdiare - Intermediate month */
	    if ((j!=g_date_month (timeruler->lower)) && (j!=g_date_month (timeruler->upper)))
	      for (k=1; k<g_date_days_in_month (j, i)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}
	  }

      /*** Premire anne - First year */
      if ((i==g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=g_date_month (timeruler->lower); j<13; j++)
	  {
	    if (j==g_date_month (timeruler->lower))
	      for (k=g_date_day (timeruler->lower); k<g_date_days_in_month (j, i)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}
	    else
	      for (k=1; k<g_date_days_in_month (j, i)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}
	  }

      /*** Dernire anne - Last year */
      if ((i!=g_date_year (timeruler->lower)) && (i==g_date_year (timeruler->upper)))
	for (j=1; j<g_date_month (timeruler->upper)+1; j++)
	  {
	    if (j==g_date_month (timeruler->upper))
	      for (k=1; k<g_date_day (timeruler->upper)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}
	    else
	      for (k=1; k<g_date_days_in_month (j, i)+1; k++)
		{
		  gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
		  gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
		  day++;
		}
	  }

      /*** Anne intermdiaire - Intermediate year */
      if ((i!=g_date_year (timeruler->lower)) && (i!=g_date_year (timeruler->upper)))
	for (j=1; j<13; j++)
	  for (k=1; k<g_date_days_in_month (j, i)+1; k++)
	    {
	      gdk_draw_line (window, timeruler->gc, timeruler->width*day, y0, timeruler->width*day, y1);
	      gdk_draw_string (window, timeruler->font, timeruler->gc, timeruler->width*day+3, y1-2, td_date_day (k, j, i, format));
	      day++;
	    }
    }
}

/**
 * td_timeruler_set_scale:
 * @timeruler: time ruler
 * @scale: zoom scale
 * 
 * fr: Rafrachis la graduation de la rgle du temps
 *
 * en: Refresh the scale of time ruler
 **/

void td_timeruler_set_scale (TdTimeruler *timeruler, int scale)
{
  int txt_num;
  gchar *txt_unit;
  g_return_if_fail (timeruler != NULL);
  g_return_if_fail (TD_IS_TIMERULER (timeruler));
  /*** FIXME: pour les heures => +tards - Hours => this night */

  /*** Go ! */
  if (timeruler->scale != scale)
    timeruler->scale = scale;
  switch (scale) 
    {
    case 1: /*** 5 ans - 5 years */
      timeruler->upper = td_date_add_years (timeruler->lower, 5);
      txt_num = 5;
      txt_unit = _("years");
      break;
    case 2: /*** 4 ans - 4 years */
      timeruler->upper = td_date_add_years (timeruler->lower, 4);
      txt_num = 4;
      txt_unit = _("years");
      break;
    case 3: /*** 3 ans - 3 years */
      timeruler->upper = td_date_add_years (timeruler->lower, 3);
      txt_num = 3;
      txt_unit = _("years");
      break;
    case 4: /*** 2 ans - 2 years */
      timeruler->upper = td_date_add_years (timeruler->lower, 2);
      txt_num = 2;
      txt_unit = _("years");
      break;

    case 5: /*** 1 an - 1 year */
      timeruler->upper = td_date_add_years (timeruler->lower, 1);
      txt_num = 1;
      txt_unit = _("year");
      break;
    case 6: /*** 6 mois - 6 months */
      timeruler->upper = td_date_add_months (timeruler->lower, 6);
      txt_num = 6;
      txt_unit = _("months");
      break;
    case 7: /*** 4 mois - 4 months */
      timeruler->upper = td_date_add_months (timeruler->lower, 4);
      txt_num = 4;
      txt_unit = _("months");
      break;
    case 8: /*** 3 mois - 3 months */
      timeruler->upper = td_date_add_months (timeruler->lower, 3);
      txt_num = 3;
      txt_unit = _("months");
      break;
    case 9: /*** 2 mois - 2 months */
      timeruler->upper = td_date_add_months (timeruler->lower, 2);
      txt_num = 2;
      txt_unit = _("months");
      break;

    case 10: /*** 1 mois - 1 month */
      timeruler->upper = td_date_add_months (timeruler->lower, 1);
      txt_num = 1;
      txt_unit = _("month");
      break;
    case 11: /*** 3 semaines - 3 weeks */
      timeruler->upper = td_date_add_days (timeruler->lower, 7*3);
      txt_num = 3;
      txt_unit = _("weeks");
      break;
    case 12: /*** 2 semaines - 2 weeks */
      timeruler->upper = td_date_add_days (timeruler->lower, 7*2);
      txt_num = 2;
      txt_unit = _("weeks");
      break;

    case 13: /*** 1 semaine - 1 week */
      timeruler->upper = td_date_add_days (timeruler->lower, 7);
      txt_num = 1;
      txt_unit = _("week");
      break;
    case 14: /*** 6 jours - 6 days */
      timeruler->upper = td_date_add_days (timeruler->lower, 6);
      txt_num = 6;
      txt_unit = _("days");
      break;
    case 15: /*** 5 jours - 5 days */
      timeruler->upper = td_date_add_days (timeruler->lower, 5);
      txt_num = 5;
      txt_unit = _("days");
      break;
    case 16: /*** 4 jours - 4 days */
      timeruler->upper = td_date_add_days (timeruler->lower, 4);
      txt_num = 4;
      txt_unit = _("days");
      break;
    case 17: /*** 3 jours - 3 days */
      timeruler->upper = td_date_add_days (timeruler->lower, 3);
      txt_num = 3;
      txt_unit = _("days");
      break;
    case 18: /*** 2 jours - 2 days */
      timeruler->upper = td_date_add_days (timeruler->lower, 2);
      txt_num = 2;
      txt_unit = _("days");
      break;
    case 19: /*** 1 jour - 1 day */
      timeruler->upper = td_date_add_days (timeruler->lower, 1);
      txt_num = 1;
      txt_unit = _("day");
      break;
    default:
      txt_num = 1;
      txt_unit = _("day");
      timeruler->upper = td_date_add_days (timeruler->lower, 1);
      break;
    }
  if (txt_num==1)
    gtk_label_set_text (GTK_LABEL (timeruler->zoom_label), g_strdup_printf ("%d %s ", txt_num, txt_unit));
  else
    gtk_label_set_text (GTK_LABEL (timeruler->zoom_label), g_strdup_printf ("%d %s", txt_num, txt_unit));
  timeruler->width = ((float) timeruler->widget->allocation.width) /td_date_period_day (timeruler->lower, timeruler->upper);
}






